class
	P_CMDBUF_STATIC

inherit
	CE_UTILITIES
	S_STRING_STATIC
	S_STDLIB_STATIC

feature {ANY} -- public attributes

	cmdbuf: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_ARRAY}.make (<<512>>)
		end

	cmd_col: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	prompt_col: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	cp: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	cmd_offset: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	literal: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	in_completion: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	tk_text: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	tk_original: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	tk_ipoint: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	tk_trial: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	tk_tlist: CE_STATIC [CE_ARRAY [PD_TEXTLIST]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_TEXTLIST]}.make (<<1>>)
		end

	openquote: CE_STATIC [INTEGER_8]
		once ("PROCESS")
			Result.item := (('%"').code).to_integer_8
		end

	closequote: CE_STATIC [INTEGER_8]
		once ("PROCESS")
			Result.item := (('%"').code).to_integer_8
		end

	mlist_search: CE_STATIC [CE_ARRAY [PD_MLIST]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_MLIST]}.make (<<1>>)
			Result.item[0] := create {PD_MLIST}.make_initialized ([mlist_search.item, mlist_search.item, mlist_search.item, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)])
		end

	ml_search: CE_STATIC [CE_POINTER [ANY]]
		once ("PROCESS")
			Result.item := create {CE_VOID_POINTER}.make_cast (mlist_search.item)
		end

	mlist_examine: CE_STATIC [CE_ARRAY [PD_MLIST]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_MLIST]}.make (<<1>>)
			Result.item[0] := create {PD_MLIST}.make_initialized ([mlist_examine.item, mlist_examine.item, mlist_examine.item, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)])
		end

	ml_examine: CE_STATIC [CE_POINTER [ANY]]
		once ("PROCESS")
			Result.item := create {CE_VOID_POINTER}.make_cast (mlist_examine.item)
		end

	mlist_shell: CE_STATIC [CE_ARRAY [PD_MLIST]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_MLIST]}.make (<<1>>)
			Result.item[0] := create {PD_MLIST}.make_initialized ([mlist_shell.item, mlist_shell.item, mlist_shell.item, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)])
		end

	ml_shell: CE_STATIC [CE_POINTER [ANY]]
		once ("PROCESS")
			Result.item := create {CE_VOID_POINTER}.make_cast (mlist_shell.item)
		end

	curr_mlist: CE_STATIC [CE_POINTER [PD_MLIST]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [PD_MLIST]}.make_shared (default_pointer)
		end

	curr_cmdflags: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

feature {ANY} -- public routines

	cmd_reset
		do
			cp.item := cmdbuf.item
			cp.item[0] := (('%/0/').code).to_integer_8
			cmd_col.item := 0
			cmd_offset.item := 0
			literal.item := 0
		end

	clear_cmd
		do
			(create {P_SCREEN_STATIC}).clear_bot
			prompt_col.item := 0
			cmd_col.item := prompt_col.item
		end

	cmd_putstr (a_s: CE_POINTER [INTEGER_8])
		require
			not a_s.is_void
		do
			(create {P_OUTPUT_STATIC}).putstr (a_s)
			cmd_col.item := (cmd_col.item.to_natural_32 + (strlen (a_s))).to_integer_32
			prompt_col.item := (prompt_col.item.to_natural_32 + (strlen (a_s))).to_integer_32
		end

	len_cmdbuf: INTEGER_32
		do
			Result := (strlen (cmdbuf.item)).to_integer_32
		end

	cmd_repaint (a_old_cp: CE_POINTER [INTEGER_8])
		local
			l_p: CE_POINTER [INTEGER_8]
			l_break: BOOLEAN
		do
			(create {P_SCREEN_STATIC}).clear_eol
			from
			until
				not (not (l_break) and then ((cp.item[0]).to_integer_32 /= 0))
			loop
				l_break := False
				l_p := (create {P_CHARSET_STATIC}).prchar ((cp.item[0]).to_integer_32)
				if cmd_col.item + (strlen (l_p)).to_integer_32 >= (create {P_SCREEN_STATIC}).sc_width.item then
					l_break := True
				end
				if not (l_break) then
					(create {P_OUTPUT_STATIC}).putstr (l_p)
					cmd_col.item := (cmd_col.item.to_natural_32 + (strlen (l_p))).to_integer_32
					cp.item := cp.item + 1
				end
			end
			l_break := False
			from
			until
				not ((cp.item.c_pointer.to_integer_32).to_natural_32 > (a_old_cp.c_pointer.to_integer_32).to_natural_32)
			loop
				cmd_left.do_nothing
			end
		end

	cmd_home
		do
			from
			until
				not (cmd_col.item > prompt_col.item)
			loop
				(create {P_SCREEN_STATIC}).putbs
				cmd_col.item := cmd_col.item - 1
			end
			cp.item := cmdbuf.item + cmd_offset.item
		end

	cmd_lshift
		local
			l_s: CE_POINTER [INTEGER_8]
			l_save_cp: CE_POINTER [INTEGER_8]
			l_cols: INTEGER_32
			l_tmp: CE_POINTER [INTEGER_8]
			l_break: BOOLEAN
		do
			l_s := cmdbuf.item + cmd_offset.item
			l_cols := 0
			from
			until
				(l_break)
			loop
				l_break := False
				if l_cols < ((create {P_SCREEN_STATIC}).sc_width.item - prompt_col.item) // 2 then
					if (l_s[0]).to_integer_32 = 0 then
						l_break := True
					end
				else
					l_break := True
				end
				if not (l_break) then
					l_tmp := l_s
					l_s := l_s + 1
					l_cols := (l_cols.to_natural_32 + (strlen (((create {P_CHARSET_STATIC}).prchar ((l_tmp[0]).to_integer_32))))).to_integer_32
				end
			end
			l_break := False
			cmd_offset.item := l_s.distance (cmdbuf.item)
			l_save_cp := cp.item
			cmd_home
			cmd_repaint (l_save_cp)
		end

	cmd_rshift
		local
			l_s: CE_POINTER [INTEGER_8]
			l_p: CE_POINTER [INTEGER_8]
			l_save_cp: CE_POINTER [INTEGER_8]
			l_cols: INTEGER_32
			l_break: BOOLEAN
		do
			l_s := cmdbuf.item + cmd_offset.item
			l_cols := 0
			from
			until
				(l_break)
			loop
				l_break := False
				if l_cols < ((create {P_SCREEN_STATIC}).sc_width.item - prompt_col.item) // 2 then
					if not ((l_s.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32) then
						l_break := True
					end
				else
					l_break := True
				end
				if not (l_break) then
					l_s := l_s - 1
					l_p := (create {P_CHARSET_STATIC}).prchar ((l_s[0]).to_integer_32)
					l_cols := (l_cols.to_natural_32 + (strlen (l_p))).to_integer_32
				end
			end
			l_break := False
			cmd_offset.item := l_s.distance (cmdbuf.item)
			l_save_cp := cp.item
			cmd_home
			cmd_repaint (l_save_cp)
		end

	cmd_right: INTEGER_32
		local
			l_p: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			if (cp.item[0]).to_integer_32 = 0 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_p := (create {P_CHARSET_STATIC}).prchar ((cp.item[0]).to_integer_32)
				if cmd_col.item + (strlen (l_p)).to_integer_32 >= (create {P_SCREEN_STATIC}).sc_width.item then
					cmd_lshift
				elseif cmd_col.item + (strlen (l_p)).to_integer_32 = (create {P_SCREEN_STATIC}).sc_width.item - 1 then
					if (cp.item[1]).to_integer_32 /= 0 then
						cmd_lshift
					end
				end
				cp.item := cp.item + 1;
				(create {P_OUTPUT_STATIC}).putstr (l_p)
				cmd_col.item := (cmd_col.item.to_natural_32 + (strlen (l_p))).to_integer_32
				Result := 0
			end
		end

	cmd_left: INTEGER_32
		local
			l_p: CE_POINTER [INTEGER_8]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_return, l_break: BOOLEAN
		do
			if (cp.item.c_pointer.to_integer_32).to_natural_32 <= (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
				Result := 0
				l_return := True
			end
			if not l_return then
				l_p := (create {P_CHARSET_STATIC}).prchar ((cp.item[-1]).to_integer_32)
				if cmd_col.item < prompt_col.item + (strlen (l_p)).to_integer_32 then
					cmd_rshift
				end
				cp.item := cp.item - 1
				cmd_col.item := (cmd_col.item.to_natural_32 - (strlen (l_p))).to_integer_32
				from
				until
					(l_break)
				loop
					l_break := False
					l_tmp___1 := l_p
					l_p := l_p + 1
					if (l_tmp___1[0]).to_integer_32 = 0 then
						l_break := True
					end
					if not (l_break) then
						(create {P_SCREEN_STATIC}).putbs
					end
				end
				l_break := False
				Result := 0
			end
		end

	cmd_ichar (a_c: INTEGER_32): INTEGER_32
		local
			l_s: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			if (strlen (cmdbuf.item)) >= ({PLATFORM}.Integer_8_bytes.to_natural_32 * (512).to_natural_32) - {NATURAL_32} 2 then
				(create {P_SCREEN_STATIC}).bell
				Result := 2
				l_return := True
			end
			if not l_return then
				l_s := cmdbuf.item + (strlen (cmdbuf.item)).to_integer_32
				from
				until
					not ((l_s.c_pointer.to_integer_32).to_natural_32 >= (cp.item.c_pointer.to_integer_32).to_natural_32)
				loop
					l_s[1] := l_s[0]
					l_s := l_s - 1
				end
				cp.item[0] := a_c.to_integer_8
				cmd_repaint (cp.item)
				cmd_right.do_nothing
				Result := 0
			end
		end

	cmd_erase: INTEGER_32
		local
			l_s: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			if cp.item.is_equal (cmdbuf.item) then
				Result := 1
				l_return := True
			end
			if not l_return then
				cmd_left.do_nothing
				l_s := cp.item
				from
				until
					(l_s[0]).to_integer_32 = 0
				loop
					l_s[0] := l_s[1]
					l_s := l_s + 1
				end
				cmd_repaint (cp.item)
				if ((curr_cmdflags.item).bit_and (1)).to_boolean then
					if cp.item.is_equal (cmdbuf.item) then
						if (cp.item[0]).to_integer_32 = 0 then
							Result := 1
							l_return := True
						end
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

	cmd_delete: INTEGER_32
		local
			l_return: BOOLEAN
		do
			if (cp.item[0]).to_integer_32 = 0 then
				Result := 0
				l_return := True
			end
			if not l_return then
				cmd_right.do_nothing
				cmd_erase.do_nothing
				Result := 0
			end
		end

	cmd_werase: INTEGER_32
		local
			l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not ((cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if (cp.item[-1]).to_integer_32 = 32 then
							from
							until
								(l_break)
							loop
								l_break := False
								if (cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
									if (cp.item[-1]).to_integer_32 /= 32 then
										l_break := True
									end
								else
									l_break := True
								end
								if not (l_break) then
									cmd_erase.do_nothing
								end
							end
							l_break := False
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						from
						until
							(l_break)
						loop
							l_break := False
							if (cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
								if (cp.item[-1]).to_integer_32 = 32 then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_break) then
								cmd_erase.do_nothing
							end
						end
						l_break := False
				when 4 then
					Result := 0
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cmd_wdelete: INTEGER_32
		local
			l_break: BOOLEAN
		do
			if (cp.item[0]).to_integer_32 = 32 then
				from
				until
					(cp.item[0]).to_integer_32 /= 32
				loop
					cmd_delete.do_nothing
				end
			else
				from
				until
					(l_break)
				loop
					l_break := False
					if (cp.item[0]).to_integer_32 /= 32 then
						if (cp.item[0]).to_integer_32 = 0 then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						cmd_delete.do_nothing
					end
				end
				l_break := False
			end
			Result := 0
		end

	cmd_kill: INTEGER_32
		local
			l_return: BOOLEAN
		do
			if (cmdbuf.item[0]).to_integer_32 = 0 then
				Result := 1
				l_return := True
			end
			if not l_return then
				cmd_offset.item := 0
				cmd_home
				cp.item[0] := (('%/0/').code).to_integer_8
				cmd_repaint (cp.item)
				if ((curr_cmdflags.item).bit_and (1)).to_boolean then
					Result := 1
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	set_mlist (a_mlist: CE_POINTER [ANY]; a_cmdflags: INTEGER_32)
		do
			curr_mlist.item := create {CE_CLASS_POINTER [PD_MLIST]}.make_cast (a_mlist)
			curr_cmdflags.item := a_cmdflags
		end

	cmd_updown (a_action: INTEGER_32): INTEGER_32
		local
			l_s: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			if curr_mlist.item.is_void then
				(create {P_SCREEN_STATIC}).bell
				Result := 0
				l_return := True
			end
			if not l_return then
				cmd_home;
				(create {P_SCREEN_STATIC}).clear_eol
				if a_action = 13 then
					curr_mlist.item.item.curr_mp := (curr_mlist.item.item.curr_mp).item.prev
				else
					curr_mlist.item.item.curr_mp := (curr_mlist.item.item.curr_mp).item.next
				end
				l_s := (curr_mlist.item.item.curr_mp).item.string
				if l_s.is_void then
					l_s := ce_string ("")
				end
				cp.item := cmdbuf.item
				from
				until
					(l_s[0]).to_integer_32 = 0
				loop
					cp.item[0] := l_s[0]
					cmd_right.do_nothing
					l_s := l_s + 1
				end
				cp.item[0] := (('%/0/').code).to_integer_8
				Result := 0
			end
		end

	cmd_addhist (a_mlist: CE_POINTER [PD_MLIST]; a_cmd: CE_POINTER [INTEGER_8])
		require
			not a_cmd.is_void
		local
			l_ml: CE_POINTER [PD_MLIST]
			l_return, l_break: BOOLEAN
		do
			if (strlen (a_cmd)) = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_ml := a_mlist.item.next
				from
				until
					not (not (l_break) and then (not l_ml.is_equal (a_mlist)))
				loop
					l_break := False
					if (strcmp (l_ml.item.string, a_cmd)) = 0 then
						l_break := True
					end
					if not (l_break) then
						l_ml := l_ml.item.next
					end
				end
				l_break := False
				if l_ml.is_equal (a_mlist) then
					l_ml := create {CE_CLASS_POINTER [PD_MLIST]}.make_cast (((create {P_MAIN_STATIC}).ecalloc (1, (create {PD_MLIST}.make).structure_size.to_natural_32)))
					l_ml.item.string := (create {P_MAIN_STATIC}).save (a_cmd)
					l_ml.item.next := a_mlist
					l_ml.item.prev := a_mlist.item.prev;
					(a_mlist.item.prev).item.next := l_ml
					a_mlist.item.prev := l_ml
				end
				a_mlist.item.curr_mp := l_ml.item.next
			end
		end

	cmd_accept
		local
			l_return: BOOLEAN
		do
			if curr_mlist.item.is_void then
				l_return := True
			end
			if not l_return then
				cmd_addhist (curr_mlist.item, cmdbuf.item)
			end
		end

	cmd_edit (a_c: INTEGER_32): INTEGER_32
		local
			l_action: INTEGER_32
			l_flags: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_flags := 0
					if curr_mlist.item.is_void then
						l_flags := (l_flags).bit_or ((2))
					end
					if curr_mlist.item.is_equal (ml_search.item) then
						l_flags := (l_flags).bit_or ((4))
					end
					l_action := (create {P_DECODE_STATIC}).editchar (a_c, l_flags)
					inspect l_action -- switch 1 head
					when 3 then l_pos := -1 + 1
					when 4 then l_pos := -1 + 2
					when 6 then l_pos := -1 + 3
					when 5 then l_pos := -1 + 4
					when 9 then l_pos := -1 + 5
					when 10 then l_pos := -1 + 6
					when 7 then l_pos := -1 + 7
					when 1 then l_pos := -1 + 8
					when 2 then l_pos := -1 + 9
					when 11 then l_pos := -1 + 10
					when 8 then l_pos := -1 + 11
					when 12 then l_pos := -1 + 12
					when 19 then l_pos := -1 + 13
					when 13 then l_pos := -1 + 14
					when 14 then l_pos := -1 + 14
					when 17 then l_pos := -1 + 15
					when 18 then l_pos := -1 + 15
					when 15 then l_pos := -1 + 15
					when 101 then l_pos := -1 + 16
					else l_pos := -1 + 17
					end
				when 1 then -- switch 1, case 3
					in_completion.item := 0
					Result := (cmd_right)
					l_return := True
				when 2 then -- switch 1, case 4
					if not l_return then
						in_completion.item := 0
						Result := (cmd_left)
						l_return := True
					end
				when 3 then -- switch 1, case 6
					if not l_return then
						in_completion.item := 0
						from
						until
							(l_break)
						loop
							l_break := False
							if (cp.item[0]).to_integer_32 /= 0 then
								if (cp.item[0]).to_integer_32 = 32 then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_break) then
								cmd_right.do_nothing
							end
						end
						l_break := False
						from
						until
							(cp.item[0]).to_integer_32 /= 32
						loop
							cmd_right.do_nothing
						end
						Result := 0
						l_return := True
					end
				when 4 then -- switch 1, case 5
					if not l_return then
						in_completion.item := 0
						from
						until
							(l_break)
						loop
							l_break := False
							if (cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
								if (cp.item[-1]).to_integer_32 /= 32 then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_break) then
								cmd_left.do_nothing
							end
						end
						l_break := False
						from
						until
							(l_break)
						loop
							l_break := False
							if (cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
								if (cp.item[-1]).to_integer_32 = 32 then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_break) then
								cmd_left.do_nothing
							end
						end
						l_break := False
						Result := 0
						l_return := True
					end
				when 5 then -- switch 1, case 9
					if not l_return then
						in_completion.item := 0
						cmd_offset.item := 0
						cmd_home
						cmd_repaint (cp.item)
						Result := 0
						l_return := True
					end
				when 6 then -- switch 1, case 10
					if not l_return then
						in_completion.item := 0
						from
						until
							(cp.item[0]).to_integer_32 = 0
						loop
							cmd_right.do_nothing
						end
						Result := 0
						l_return := True
					end
				when 7 then -- switch 1, case 7
					if not l_return then
						in_completion.item := 0
						Result := 0
						l_return := True
					end
				when 8 then -- switch 1, case 1
					if not l_return then
						in_completion.item := 0
						Result := (cmd_erase)
						l_return := True
					end
				when 9 then -- switch 1, case 2
					if not l_return then
						in_completion.item := 0
						Result := (cmd_kill)
						l_return := True
					end
				when 10 then -- switch 1, case 11
					if not l_return then
						in_completion.item := 0
						Result := (cmd_werase)
						l_return := True
					end
				when 11 then -- switch 1, case 8
					if not l_return then
						in_completion.item := 0
						Result := (cmd_delete)
						l_return := True
					end
				when 12 then -- switch 1, case 12
					if not l_return then
						in_completion.item := 0
						Result := (cmd_wdelete)
						l_return := True
					end
				when 13 then -- switch 1, case 19
					if not l_return then
						literal.item := 1
						Result := 0
						l_return := True
					end
				when 14 then -- switch 1, case 13 | switch 1, case 14
					if not l_return then
						in_completion.item := 0
						Result := (cmd_updown (l_action))
						l_return := True
					end
				when 15 then -- switch 1, case 17 | switch 1, case 18 | switch 1, case 15
					if not l_return then
						Result := (cmd_complete (l_action))
						l_return := True
					end
				when 16 then -- switch 1, case 101
					if not l_return then
						Result := 0
						l_return := True
					end
				when 17 then -- switch 1, default:
					if not l_return then
						in_completion.item := 0
						Result := 3
					end
				when 18 then -- switch 1, done:
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cmd_istr (a_str: CE_POINTER [INTEGER_8]): INTEGER_32
		require
			not a_str.is_void
		local
			l_s: CE_POINTER [INTEGER_8]
			l_action: INTEGER_32
			l_return: BOOLEAN
		do
			l_s := a_str
			from
			until
				not (not (l_return) and then ((l_s[0]).to_integer_32 /= 0))
			loop
				l_action := cmd_ichar ((l_s[0]).to_integer_32)
				if l_action /= 0 then
					(create {P_SCREEN_STATIC}).bell
					Result := l_action
					l_return := True
				end
				if not l_return then
					l_s := l_s + 1
				end
			end
			if not l_return then
				Result := 0
			end
		end

	delimit_word: CE_POINTER [INTEGER_8]
		local
			l_word: CE_POINTER [INTEGER_8]
			l_p: CE_POINTER [INTEGER_8]
			l_delim_quoted: INTEGER_32
			l_meta_quoted: INTEGER_32
			l_esc: CE_POINTER [INTEGER_8]
			l_esclen: INTEGER_32
			l_tmp___1: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_delim_quoted := 0
					l_meta_quoted := 0
					l_esc := ((create {P_FILENAME_STATIC}).get_meta_escape)
					l_esclen := (strlen (l_esc)).to_integer_32
					if (cp.item[0]).to_integer_32 = 32 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if (cp.item[0]).to_integer_32 /= 0 then
							from
							until
								(l_break)
							loop
								l_break := False
								if (cp.item[0]).to_integer_32 /= 32 then
									if (cp.item[0]).to_integer_32 = 0 then
										l_break := True
									end
								else
									l_break := True
								end
								if not (l_break) then
									cmd_right.do_nothing
								end
							end
							l_break := False
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						if (cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
							if (cp.item[-1]).to_integer_32 /= 32 then
								l_tmp___1 := 1
							else
								l_tmp___1 := 0
							end
						else
							l_tmp___1 := 0
						end
				when 4 then
					if cp.item.is_equal (cmdbuf.item) then
						Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						l_word := cmdbuf.item
						from
						until
							not (not (l_break) and then ((l_word.c_pointer.to_integer_32).to_natural_32 < (cp.item.c_pointer.to_integer_32).to_natural_32))
						loop
							l_break := False
							if (l_word[0]).to_integer_32 /= 32 then
								l_break := True
							end
							if not (l_break) then
								l_word := l_word + 1
							end
						end
						l_break := False
						if (l_word.c_pointer.to_integer_32).to_natural_32 >= (cp.item.c_pointer.to_integer_32).to_natural_32 then
							Result := cp.item
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 16 -- skip if block
					end
				when 5 then -- if:
						l_p := cmdbuf.item
				when 6 then -- loop condition:
						if not ((l_p.c_pointer.to_integer_32).to_natural_32 < (cp.item.c_pointer.to_integer_32).to_natural_32) then
							l_pos := -1 + 15 -- skip loop
						end
				when 7 then -- loop body:
						if not (l_meta_quoted.to_boolean) then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						l_meta_quoted := 0
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 14
				when 10 then -- else:
						if not (l_esclen > 0) then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if ((l_p + l_esclen).c_pointer.to_integer_32).to_natural_32 < (cp.item.c_pointer.to_integer_32).to_natural_32 then
							if (strncmp (l_p, l_esc, l_esclen.to_natural_32)) = 0 then
								l_meta_quoted := 1
								l_p := l_p + (l_esclen - 1)
							else
								l_pos := -1 + 13 -- goto _L___1
							end
						else
							l_pos := -1 + 13 -- goto _L___1
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 14
				when 13 then -- else: | _L___1:
						if l_delim_quoted.to_boolean then
							if (l_p[0]).to_integer_32 = closequote.item.to_integer_32 then
								l_delim_quoted := 0
							end
						elseif (l_p[0]).to_integer_32 = openquote.item.to_integer_32 then
							l_delim_quoted := 1
						elseif (l_p[0]).to_integer_32 = 32 then
							l_word := l_p + 1
						end
				when 14 then -- if done
						l_p := l_p + 1
						l_pos := -1 + 6 -- back to loop condition
				when 15 then -- loop done
						Result := l_word
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	init_compl
		local
			l_word: CE_POINTER [INTEGER_8]
			l_c: INTEGER_8
			l_qword: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			if not tk_text.item.is_void then
				free (create {CE_VOID_POINTER}.make_cast (tk_text.item))
				tk_text.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			end
			l_word := delimit_word
			if l_word.is_void then
				l_return := True
			end
			if not l_return then
				tk_ipoint.item := l_word
				if not tk_original.item.is_void then
					free (create {CE_VOID_POINTER}.make_cast (tk_original.item))
				end
				tk_original.item := create {CE_INTEGER_8_POINTER}.make_cast (((create {P_MAIN_STATIC}).ecalloc ((cp.item.distance (l_word)) + 1, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				strncpy (tk_original.item, l_word, (cp.item.distance (l_word)).to_natural_32).do_nothing
				l_c := cp.item[0]
				cp.item[0] := (('%/0/').code).to_integer_8
				if (l_word[0]).to_integer_32 /= openquote.item.to_integer_32 then
					tk_text.item := (create {P_FILENAME_STATIC}).fcomplete (l_word)
				else
					l_qword := ((create {P_FILENAME_STATIC}).shell_quote (l_word + 1))
					if l_qword.is_void then
						tk_text.item := (create {P_FILENAME_STATIC}).fcomplete (l_word + 1)
					else
						tk_text.item := (create {P_FILENAME_STATIC}).fcomplete (l_qword)
						free (create {CE_VOID_POINTER}.make_cast (l_qword))
					end
				end
				cp.item[0] := l_c
			end
		end

	next_compl (a_action: INTEGER_32; a_prev: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		local
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					inspect a_action -- switch 1 head
					when 17 then l_pos := -1 + 1
					when 18 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 17
					Result := (tk_tlist.item.item.forw_textlist (a_prev))
					l_return := True
				when 2 then -- switch 1, case 18
					if not l_return then
						Result := (tk_tlist.item.item.back_textlist (a_prev))
						l_return := True
					end
				when 3 then -- switch 1, done:
					if not l_return then
						Result := (ce_string ("?"))
					end
				when 4 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cmd_complete (a_action: INTEGER_32): INTEGER_32
		local
			l_s: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if in_completion.item.to_boolean then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_action /= 15 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						init_compl
						if tk_text.item.is_void then
							(create {P_SCREEN_STATIC}).bell
							Result := 0
							l_return := True
						end
						if not l_return then
							if a_action = 15 then
								tk_trial.item := tk_text.item
							else
								in_completion.item := 1
								tk_tlist.item.item.init_textlist (tk_text.item)
								tk_trial.item := next_compl (a_action, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						tk_trial.item := next_compl (a_action, tk_trial.item)
				when 7 then -- if done
					if l_return then
						l_pos := -1 + 14 -- skip if block
					end
				when 8 then -- if:
						from
						until
							not ((cp.item.c_pointer.to_integer_32).to_natural_32 > (tk_ipoint.item.c_pointer.to_integer_32).to_natural_32)
						loop
							cmd_erase.do_nothing
						end
						if not (tk_trial.item.is_void) then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						in_completion.item := 0
						if (cmd_istr (tk_original.item)) /= 0 then
							l_pos := -1 + 15 -- goto fail
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 11 then -- else:
						if (cmd_istr (tk_trial.item)) /= 0 then
							l_pos := -1 + 15 -- goto fail
						end
				when 12 then
						if ((create {P_FILENAME_STATIC}).is_dir (tk_trial.item)).to_boolean then
							if (cp.item.c_pointer.to_integer_32).to_natural_32 > (cmdbuf.item.c_pointer.to_integer_32).to_natural_32 then
								if (cp.item[-1]).to_integer_32 = closequote.item.to_integer_32 then
									cmd_erase.do_nothing
								end
							end
							l_s := (create {P_DECODE_STATIC}).lgetenv (ce_string ("LESSSEPARATOR"))
							if l_s.is_void then
								l_s := ce_string ("/")
							end
							if (cmd_istr (l_s)) /= 0 then
								l_pos := -1 + 15 -- goto fail
							end
						end
				when 13 then
						Result := 0
						l_return := True
				when 14 then -- if done
					if l_return then
						l_pos := -1 + 16 -- skip if block
					end
				when 15 then -- if: | fail:
						in_completion.item := 0;
						(create {P_SCREEN_STATIC}).bell
						Result := 0
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cmd_char (a_c: INTEGER_32): INTEGER_32
		local
			l_action: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if literal.item.to_boolean then
						literal.item := 0
						Result := (cmd_ichar (a_c))
						l_return := True
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if not (((create {P_COMMAND_STATIC}).in_mca).to_boolean) then
							l_pos := -1 + 5 -- skip if block
						end
				when 2 then -- if:
						l_action := cmd_edit (a_c)
						inspect l_action -- switch 1 head
						when 0 then l_pos := -1 + 3
						when 1 then l_pos := -1 + 3
						when 3 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 3 then -- switch 1, case 0 | switch 1, case 1
						Result := l_action
						l_return := True
				when 4 then -- switch 1, case 3
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done: | if done | if done
					if not l_return then
						Result := (cmd_ichar (a_c))
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cmd_int: INTEGER_32
		local
			l_p: CE_POINTER [INTEGER_8]
			l_n: INTEGER_32
		do
			l_n := (0).to_integer_32
			l_p := cmdbuf.item
			from
			until
				(l_p[0]).to_integer_32 = 0
			loop
				l_n := 10 * l_n + ((l_p[0]).to_integer_32 - 48).to_integer_32
				l_p := l_p + 1
			end
			Result := l_n
		end

	get_cmdbuf: CE_POINTER [INTEGER_8]
		do
			Result := cmdbuf.item
		end

end
