class
	PD_IFILE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_IFILE]} h_next.make
			create {CE_CLASS_POINTER [PD_IFILE]} h_prev.make
			create {CE_INTEGER_8_POINTER} h_filename.make
			create {CE_VOID_POINTER} h_filestate.make
			create h_scrpos.make
		end

feature {ANY} -- fields

	h_next: CE_POINTER [PD_IFILE] assign set_h_next

	h_prev: CE_POINTER [PD_IFILE] assign set_h_prev

	h_filename: CE_POINTER [INTEGER_8] assign set_h_filename

	h_filestate: CE_POINTER [ANY] assign set_h_filestate

	h_index: INTEGER_32 assign set_h_index

	h_hold: INTEGER_32 assign set_h_hold

	h_opened: INTEGER_8 assign set_h_opened

	h_scrpos: PD_SCRPOS assign set_h_scrpos

feature {ANY} -- field setters

	set_h_next (a_h_next: CE_POINTER [PD_IFILE])
		do
			h_next := a_h_next
			update_memory_field ("h_next")
		end

	set_h_prev (a_h_prev: CE_POINTER [PD_IFILE])
		do
			h_prev := a_h_prev
			update_memory_field ("h_prev")
		end

	set_h_filename (a_h_filename: CE_POINTER [INTEGER_8])
		do
			h_filename := a_h_filename
			update_memory_field ("h_filename")
		end

	set_h_filestate (a_h_filestate: CE_POINTER [ANY])
		do
			h_filestate := a_h_filestate
			update_memory_field ("h_filestate")
		end

	set_h_index (a_h_index: INTEGER_32)
		do
			h_index := a_h_index
			update_memory_field ("h_index")
		end

	set_h_hold (a_h_hold: INTEGER_32)
		do
			h_hold := a_h_hold
			update_memory_field ("h_hold")
		end

	set_h_opened (a_h_opened: INTEGER_8)
		do
			h_opened := a_h_opened
			update_memory_field ("h_opened")
		end

	set_h_scrpos (a_h_scrpos: PD_SCRPOS)
		do
			h_scrpos := a_h_scrpos
			update_memory_field ("h_scrpos")
		end

feature {ANY} -- public routines

	link_ifile (a_prev: CE_POINTER [PD_IFILE])
		local
			l_prev: CE_POINTER [PD_IFILE]
		do
			l_prev := a_prev
			if l_prev.is_void then
				l_prev := (create {P_IFILE_STATIC}).anchor.item
			end
			Current.h_next := l_prev.item.h_next
			update_class_fields -- Current might have been changed; synchronize
			Current.h_prev := l_prev
			update_class_fields; -- Current might have been changed; synchronize
			(l_prev.item.h_next).item.h_prev := (create {CE_CLASS_POINTER [PD_IFILE]}.make_shared (c_pointer))
			update_class_fields -- Current might have been changed; synchronize
			l_prev.item.h_next := (create {CE_CLASS_POINTER [PD_IFILE]}.make_shared (c_pointer))
			update_class_fields -- Current might have been changed; synchronize
			Current.h_index := l_prev.item.h_index + 1
			update_class_fields -- Current might have been changed; synchronize
			incr_index (h_next, 1)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_IFILE_STATIC}).ifiles.item := (create {P_IFILE_STATIC}).ifiles.item + 1
			update_class_fields -- Current might have been changed; synchronize
		end

	unlink_ifile
		do
			h_next.item.h_prev := h_prev
			update_class_fields -- Current might have been changed; synchronize
			h_prev.item.h_next := h_next
			update_class_fields -- Current might have been changed; synchronize
			incr_index (h_next, -1)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_IFILE_STATIC}).ifiles.item := (create {P_IFILE_STATIC}).ifiles.item - 1
			update_class_fields -- Current might have been changed; synchronize
		end

feature {PD_IFILE} -- private routines

	incr_index (a_p: CE_POINTER [PD_IFILE]; a_incr: INTEGER_32)
		local
			l_p: CE_POINTER [PD_IFILE]
		do
			l_p := a_p
			from
			until
				l_p.is_equal ((create {P_IFILE_STATIC}).anchor.item)
			loop
				l_p.item.h_index := l_p.item.h_index + a_incr
				l_p := l_p.item.h_next
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			h_next := a_other.h_next
			h_prev := a_other.h_prev
			h_filename := a_other.h_filename
			h_filestate := a_other.h_filestate
			h_index := a_other.h_index
			h_hold := a_other.h_hold
			h_opened := a_other.h_opened
			h_scrpos := a_other.h_scrpos.twin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				h_next.is_equal (a_other.h_next) and
				h_prev.is_equal (a_other.h_prev) and
				h_filename.is_equal (a_other.h_filename) and
				h_filestate.is_equal (a_other.h_filestate) and
				h_index = a_other.h_index and
				h_hold = a_other.h_hold and
				h_opened = a_other.h_opened and
				h_scrpos.is_equal (a_other.h_scrpos)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("h_next"),
					create {CE_STRUCTURE_FIELD}.make ("h_prev"),
					create {CE_STRUCTURE_FIELD}.make ("h_filename"),
					create {CE_STRUCTURE_FIELD}.make ("h_filestate"),
					create {CE_STRUCTURE_FIELD}.make ("h_index"),
					create {CE_STRUCTURE_FIELD}.make ("h_hold"),
					create {CE_STRUCTURE_FIELD}.make ("h_opened"),
					create {CE_STRUCTURE_FIELD}.make ("h_scrpos")
				>>)
			init_structure_info
		end

end
