class
	P_CONNECT_STATIC

inherit
	CE_UTILITIES
	S_SYS_SOCKET_STATIC
	S_UNISTD_STATIC
	S_GCC_BUILTIN_STATIC
	S_ASSERT_STATIC
	S_STDLIB_STATIC

feature {ANY} -- public attributes

	transport_map: CE_STATIC [CE_POINTER [PD_HASH_TABLE]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [PD_HASH_TABLE]}.make
		end

	transport_map_modified_tick: CE_STATIC [NATURAL_32]
		once ("PROCESS")
		end

feature {ANY} -- public routines

	connect_with_timeout_callback (a_arg: CE_POINTER [ANY])
		local
			l_ctx: CE_POINTER [PD_CWT_CONTEXT]
		do
			l_ctx := create {CE_CLASS_POINTER [PD_CWT_CONTEXT]}.make_cast (a_arg)
			l_ctx.item.result2 := connect (l_ctx.item.fd, l_ctx.item.addr, l_ctx.item.addrlen)
		end

	sock_close (a_fd: INTEGER_32)
		local
			l_break: BOOLEAN
		do
			close (a_fd).do_nothing
			from
			until
				(l_break)
			loop
				l_break := False
				if (builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)).to_boolean then
					(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("Closed fd %%d%N"), a_fd])
				end
				l_break := True
			end
			l_break := False
		end

	fd_register_transport (a_fd: INTEGER_32; a_imp: CE_POINTER [PD_TRANSPORT_IMPLEMENTATION]; a_ctx: CE_POINTER [ANY])
		local
			l_info: CE_POINTER [PD_TRANSPORT_INFO]
		do
			if not (a_fd >= 0) then
				assert_fail (ce_string ("fd >= 0"), ce_string ("connect.c"), {NATURAL_32} 792, ce_string ("fd_register_transport"))
			end
			l_info := create {CE_CLASS_POINTER [PD_TRANSPORT_INFO]}.make_cast (((create {P_XMALLOC_STATIC}).xmalloc ((create {PD_TRANSPORT_INFO}.make).structure_size.to_natural_32)))
			l_info.item.imp := a_imp
			l_info.item.ctx := a_ctx
			if transport_map.item.is_void then
				transport_map.item := (create {P_HASH_STATIC}).hash_table_new (0, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_ROUTINE}.make_shared (default_pointer))
			end
			transport_map.item.item.hash_table_put (create {CE_VOID_POINTER}.make_shared (default_pointer + a_fd), create {CE_VOID_POINTER}.make_cast (l_info))
			transport_map_modified_tick.item := transport_map_modified_tick.item + 1
		end

	fd_transport_context (a_fd: INTEGER_32): CE_POINTER [ANY]
		local
			l_info: CE_POINTER [PD_TRANSPORT_INFO]
		do
			l_info := create {CE_CLASS_POINTER [PD_TRANSPORT_INFO]}.make_cast ((transport_map.item.item.hash_table_get (create {CE_VOID_POINTER}.make_shared (default_pointer + a_fd))))
			Result := (l_info.item.ctx)
		end

	fd_close (a_fd: INTEGER_32)
		local
			l_info: CE_POINTER [PD_TRANSPORT_INFO]
			l_return: BOOLEAN
		do
			if a_fd < 0 then
				l_return := True
			end
			if not l_return then
				l_info := create {CE_CLASS_POINTER [PD_TRANSPORT_INFO]}.make_shared (default_pointer)
				if (not transport_map.item.is_void) then
					l_info := create {CE_CLASS_POINTER [PD_TRANSPORT_INFO]}.make_cast ((transport_map.item.item.hash_table_get (create {CE_VOID_POINTER}.make_shared (default_pointer + a_fd))))
				end
				if (not l_info.is_void) then
					if (not ((l_info.item.imp).item.closer).is_void) then
						((l_info.item.imp).item.closer).call ([a_fd, l_info.item.ctx])
					else
						sock_close (a_fd)
					end
				else
					sock_close (a_fd)
				end
				if (not l_info.is_void) then
					transport_map.item.item.hash_table_remove (create {CE_VOID_POINTER}.make_shared (default_pointer + a_fd)).do_nothing
					free (create {CE_VOID_POINTER}.make_cast (l_info))
					transport_map_modified_tick.item := transport_map_modified_tick.item + 1
				end
			end
		end

feature {ANY} -- routine addresses

	connect_with_timeout_callback_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent connect_with_timeout_callback)
		end

end
