class
	P_MAIN_STATIC

inherit
	CE_UTILITIES
	S_UNISTD_STATIC
	S_SIGNAL_STATIC
	S_FCNTL_STATIC

feature {ANY} -- public attributes

	retval: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	fg_poll_timer: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := -1
		end

	terminal_pipe: CE_STATIC [CE_ARRAY [INTEGER_32]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_32_ARRAY}.make (<<2>>)
		end

	dump_stat: CE_STATIC [CE_ARRAY [PD_STATUS]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_STATUS]}.make (<<1>>)
		end

	dump_pos: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	init_b: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

feature {ANY} -- public routines

	sig_terminate (a_t: CE_POINTER [PD_TERMINAL])
		do
			unhandle_basic_signals (a_t);
			(create {P_SELECT_STATIC}).terminate.item := 1
			retval.item := 2
		end

	sig_intr (a_t: CE_POINTER [PD_TERMINAL])
		do
			if a_t.is_void then
				unhandle_basic_signals (a_t);
				(create {P_SELECT_STATIC}).terminate.item := 1
			else
				unhandle_basic_signals (a_t);
				(create {P_MENU_STATIC}).exit_prog (a_t, create {CE_VOID_POINTER}.make_shared (default_pointer), create {CE_CLASS_POINTER [PD_SESSION]}.make_shared (default_pointer))
			end
		end

	sig_ctrl_c (a_t: CE_POINTER [PD_TERMINAL])
		do
			if not ((create {P_KBD_STATIC}).is_blocked).to_boolean then
				(create {P_KBD_STATIC}).kbd_ctrl_c
			end
		end

	sig_ign (a_x: CE_POINTER [ANY])
		do
		end

	sig_tstp (a_t: CE_POINTER [PD_TERMINAL])
		local
			l_pid: INTEGER_32
			l_newpid: INTEGER_32
		do
			l_pid := (getpid);
			(create {P_KBD_STATIC}).block_itrm (1)
			l_newpid := fork
			if not l_newpid.to_boolean then
				from
				until
					False
				loop
					sleep ({NATURAL_32} 1).do_nothing
					kill (l_pid, 18).do_nothing
				end
			end
			raise (19).do_nothing
			if l_newpid /= -1 then
				kill (l_newpid, 9).do_nothing
			end
			if fg_poll_timer.item /= -1 then
				(create {P_SELECT_STATIC}).kill_timer (fg_poll_timer.item)
			end
			fg_poll_timer.item := (create {P_SELECT_STATIC}).install_timer ((500).to_integer_32, poll_fg_address, create {CE_VOID_POINTER}.make_cast (a_t))
		end

	poll_fg (a_t: CE_POINTER [ANY])
		local
			l_r: INTEGER_32
		do
			fg_poll_timer.item := -1
			l_r := (create {P_KBD_STATIC}).unblock_itrm (1)
			if l_r = -1 then
				fg_poll_timer.item := (create {P_SELECT_STATIC}).install_timer ((500).to_integer_32, poll_fg_address, a_t)
			end
			if l_r = -2 then
				kill (0, 18).do_nothing
			end
		end

	sig_cont (a_t: CE_POINTER [PD_TERMINAL])
		do
			(create {P_KBD_STATIC}).unblock_itrm (1).do_nothing
		end

	handle_basic_signals (a_term: CE_POINTER [PD_TERMINAL])
		do
			(create {P_SELECT_STATIC}).install_signal_handler (1, (sig_intr_address), create {CE_VOID_POINTER}.make_cast (a_term), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (2, (sig_ctrl_c_address), create {CE_VOID_POINTER}.make_cast (a_term), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (20, (sig_tstp_address), create {CE_VOID_POINTER}.make_cast (a_term), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (21, (sig_tstp_address), create {CE_VOID_POINTER}.make_cast (a_term), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (22, sig_ign_address, create {CE_VOID_POINTER}.make_cast (a_term), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (18, (sig_cont_address), create {CE_VOID_POINTER}.make_cast (a_term), 0)
		end

	unhandle_basic_signals (a_term: CE_POINTER [PD_TERMINAL])
		do
			(create {P_SELECT_STATIC}).install_signal_handler (1, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (2, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (20, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (21, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (22, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer), 0);
			(create {P_SELECT_STATIC}).install_signal_handler (18, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer), 0)
			if fg_poll_timer.item /= -1 then
				(create {P_SELECT_STATIC}).kill_timer (fg_poll_timer.item)
				fg_poll_timer.item := -1
			end
		end

	attach_terminal (a_in, a_out, a_ctl: INTEGER_32; a_info: CE_POINTER [ANY]; a_len: INTEGER_32): INTEGER_32
		local
			l_term: CE_POINTER [PD_TERMINAL]
			l_return: BOOLEAN
		do
			fcntl ([terminal_pipe.item[0], 4, 2_048]).do_nothing
			fcntl ([terminal_pipe.item[1], 4, 2_048]).do_nothing;
			(create {P_KBD_STATIC}).handle_trm (a_in, a_out, a_out, terminal_pipe.item[1], a_ctl, a_info, a_len);
			(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("main.c")), 150, a_info)
			l_term := (create {P_TERMINAL_STATIC}).init_term (terminal_pipe.item[0], a_out, (create {PD_WINDOW_STATIC}).win_func_address)
			if (not l_term.is_void) then
				handle_basic_signals (l_term)
				Result := (terminal_pipe.item[1])
				l_return := True
			end
			if not l_return then
				close (terminal_pipe.item[0]).do_nothing
				close (terminal_pipe.item[1]).do_nothing
				Result := (-1)
			end
		end

	init
		local
			l_uh: INTEGER_32
			l_info: CE_POINTER [ANY]
			l_len: CE_ARRAY [INTEGER_32]
			l_u: CE_POINTER [NATURAL_8]
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_uu: CE_POINTER [NATURAL_8]
			l_wd: CE_POINTER [NATURAL_8]
			l_tmp___7: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_len.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:;
					(create {P_LANGUAGE_STATIC}).init_trans;
					(create {P_SELECT_STATIC}).set_sigcld;
					(create {P_DEFAULT_STATIC}).init_home;
					(create {P_CACHE_STATIC}).init_cache;
					(create {P_KBDBIND_STATIC}).init_keymaps
					if ((create {P_OS_DEP_STATIC}).c2_pipe (terminal_pipe.item)).to_boolean then
						(create {P_ERROR_STATIC}).error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: can%'t create pipe for internal communication"))])
						retval.item := 4
						l_pos := -1 + 8 -- goto ttt
					end
				when 1 then
					l_u := (create {P_DEFAULT_STATIC}).parse_options ((create {P_LINKS_STATIC}).g_argc.item - 1, (create {P_LINKS_STATIC}).g_argv.item + 1)
					if l_u.is_void then
						retval.item := 3
						l_pos := -1 + 8 -- goto ttt
					end
				when 2 then
					if ((create {P_DEFAULT_STATIC}).no_connect.item).to_boolean then
						l_pos := -1 + 6 -- skip if block
					end
				when 3 then -- if:
						l_uh := (create {P_AF_UNIX_STATIC}).bind_to_af_unix
						if l_uh = -1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						close (terminal_pipe.item[0]).do_nothing
						close (terminal_pipe.item[1]).do_nothing
						l_info := (create {P_SESSION_STATIC}).create_session_info ((create {P_DEFAULT_STATIC}).base_session.item.item, l_u, l_len)
						if l_info.is_void then
							retval.item := 4
							l_pos := -1 + 8 -- goto ttt
						end
				when 5 then
						l_tmp___0 := (create {P_OS_DEP_STATIC}).get_ctl_handle
						l_tmp___1 := (create {P_OS_DEP_STATIC}).get_output_handle;
						(create {P_KBD_STATIC}).handle_trm (((create {P_OS_DEP_STATIC}).get_input_handle), l_tmp___1, l_uh, l_uh, l_tmp___0, l_info, l_len.item)
						handle_basic_signals (create {CE_CLASS_POINTER [PD_TERMINAL]}.make_shared (default_pointer));
						(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("main.c")), 279, l_info)
						l_return := True
				when 6 then -- if done | if done
					if l_return then
						l_pos := -1 + 9 -- skip if block
					end
				when 7 then -- if:;
						(create {P_DEFAULT_STATIC}).dds.item.item.assume_cp := (create {P_CHARSETS_STATIC}).get_cp_index (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ISO-8859-1")))
						if (create {P_DEFAULT_STATIC}).dds.item.item.assume_cp = -1 then
							(create {P_DEFAULT_STATIC}).dds.item.item.assume_cp := 0
						end
						(create {P_DEFAULT_STATIC}).load_config
						init_b.item := 1;
						(create {P_BOOKMARK_STATIC}).read_bookmarks;
						(create {P_DEFAULT_STATIC}).load_url_history;
						(create {P_COOKIES_STATIC}).init_cookies
						l_u := (create {P_DEFAULT_STATIC}).parse_options ((create {P_LINKS_STATIC}).g_argc.item - 1, (create {P_LINKS_STATIC}).g_argv.item + 1)
						if (not l_u.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 8 then -- if: | ttt:;
						(create {P_SELECT_STATIC}).terminate.item := 1
						l_return := True
				when 9 then -- if done | if done
					if l_return then
						l_pos := -1 + 16 -- skip if block
					end
				when 10 then -- if:
						if ((create {P_DEFAULT_STATIC}).dmp.item).to_boolean then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						l_info := (create {P_SESSION_STATIC}).create_session_info ((create {P_DEFAULT_STATIC}).base_session.item.item, l_u, l_len)
						if (not l_info.is_void) then
							l_tmp___3 := (create {P_OS_DEP_STATIC}).get_ctl_handle
							l_tmp___4 := (create {P_OS_DEP_STATIC}).get_output_handle
							if (attach_terminal (((create {P_OS_DEP_STATIC}).get_input_handle), l_tmp___4, l_tmp___3, l_info, l_len.item)) = -1 then
								retval.item := 4;
								(create {P_SELECT_STATIC}).terminate.item := 1
							end
						else
							retval.item := 4;
							(create {P_SELECT_STATIC}).terminate.item := 1
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 13 then -- else:
						close (terminal_pipe.item[0]).do_nothing
						close (terminal_pipe.item[1]).do_nothing
						if not (l_u.item).to_boolean then
							if (create {P_DEFAULT_STATIC}).dmp.item = 1 then
								l_tmp___7 := ce_string ("-dump")
							else
								l_tmp___7 := ce_string ("-source")
							end
							Io.error.put_string ("URL expected after " + eif_string (l_tmp___7) + "%N.")
							retval.item := 3
							l_pos := -1 + 8 -- goto ttt
						end
				when 14 then
						dump_stat.item.item.end2 := (create {PD_STATUS_STATIC}).end_dump_address
						dump_pos.item := (0).to_integer_32
						l_wd := (create {P_TERMINAL_STATIC}).get_cwd
						l_uu := (create {P_URL_STATIC}).translate_url (l_u, l_wd)
						if l_uu.is_void then
							l_uu := (create {P_LINKS_STATIC}).debug_stracpy (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("main.c")), 310, l_u)
						end
						if ((create {P_SCHED_STATIC}).load_url (l_uu, dump_stat.item, 0, 0)).to_boolean then
							retval.item := 4
							l_pos := -1 + 8 -- goto ttt
						end
				when 15 then;
						(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("main.c")), 315, create {CE_VOID_POINTER}.make_cast (l_uu))
						if (not l_wd.is_void) then
							(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("main.c")), 316, create {CE_VOID_POINTER}.make_cast (l_wd))
						end
				when 16 then -- if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	terminate_all_subsystems
		do
			(create {P_AF_UNIX_STATIC}).af_unix_close;
			(create {P_SESSION_STATIC}).destroy_all_sessions;
			(create {P_SELECT_STATIC}).check_bottom_halves;
			(create {P_SESSION_STATIC}).abort_all_downloads;
			(create {P_SELECT_STATIC}).check_bottom_halves;
			(create {P_TERMINAL_STATIC}).destroy_all_terminals;
			(create {P_SELECT_STATIC}).check_bottom_halves;
			(create {P_KBD_STATIC}).free_all_itrms;
			(create {P_SCHED_STATIC}).abort_all_connections
			shrink_memory (1)
			if init_b.item.to_boolean then
				(create {P_DEFAULT_STATIC}).save_url_history
			end
			(create {P_MENU_STATIC}).free_history_lists;
			(create {P_TERMINAL_STATIC}).free_term_specs;
			(create {P_TYPES_STATIC}).free_types
			if init_b.item.to_boolean then
				(create {P_BOOKMARK_STATIC}).finalize_bookmarks
			end
			(create {P_KBDBIND_STATIC}).free_keymaps;
			(create {P_CHARSETS_STATIC}).free_conv_table;
			(create {P_SCHED_STATIC}).free_blacklist
			if init_b.item.to_boolean then
				(create {P_COOKIES_STATIC}).cleanup_cookies
			end
			(create {P_SELECT_STATIC}).check_bottom_halves;
			(create {P_DEFAULT_STATIC}).end_config;
			(create {P_SESSION_STATIC}).free_strerror_buf;
			(create {P_LANGUAGE_STATIC}).shutdown_trans;
			(create {P_OS_DEP_STATIC}).terminate_osdep
		end

	main (a_argc: INTEGER_32; a_argv: CE_POINTER [CE_POINTER [INTEGER_8]]): INTEGER_32
		require
			not a_argv.is_void
		do
			(create {P_LINKS_STATIC}).path_to_exe.item := create {CE_NATURAL_8_POINTER}.make_cast (a_argv[0]);
			(create {P_LINKS_STATIC}).g_argc.item := a_argc;
			(create {P_LINKS_STATIC}).g_argv.item := create {CE_POINTER_POINTER [NATURAL_8]}.make_cast (a_argv);
			(create {P_OS_DEP_STATIC}).get_path_to_exe;
			(create {P_SELECT_STATIC}).select_loop (init_address)
			terminate_all_subsystems
			if fg_poll_timer.item /= -1 then
				(create {P_SELECT_STATIC}).kill_timer (fg_poll_timer.item)
				fg_poll_timer.item := -1
			end
			(create {P_ERROR_STATIC}).check_memory_leaks
			Result := retval.item
		end

	shrink_memory (a_u: INTEGER_32)
		do
			(create {P_DNS_STATIC}).shrink_dns_cache (a_u);
			(create {P_HTML_R_STATIC}).shrink_format_cache (a_u);
			(create {P_CACHE_STATIC}).garbage_collection (a_u);
			(create {P_HTML_R_STATIC}).delete_unused_format_cache_entries
		end

feature {ANY} -- routine addresses

	sig_intr_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sig_intr)
		end

	sig_ctrl_c_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sig_ctrl_c)
		end

	sig_ign_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sig_ign)
		end

	sig_tstp_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sig_tstp)
		end

	poll_fg_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent poll_fg)
		end

	sig_cont_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sig_cont)
		end

	init_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent init)
		end

end
