class
	PD_DIALOG_ITEM_DATA

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_DIALOG_ITEM]} item.make
			create history.make
			create {CE_CLASS_POINTER [PD_HISTORY_ITEM]} cur_hist.make
			create {CE_NATURAL_8_POINTER} cdata.make
			Precursor
		end

feature {ANY} -- fields

	x: INTEGER_32 assign set_x

	y: INTEGER_32 assign set_y

	l: INTEGER_32 assign set_l

	vpos: INTEGER_32 assign set_vpos

	cpos: INTEGER_32 assign set_cpos

	checked: INTEGER_32 assign set_checked

	item: CE_POINTER [PD_DIALOG_ITEM] assign set_item

	history: PD_LIST_HEAD assign set_history

	cur_hist: CE_POINTER [PD_HISTORY_ITEM] assign set_cur_hist

	cdata: CE_POINTER [NATURAL_8] assign set_cdata

feature {ANY} -- field setters

	set_x (a_x: INTEGER_32)
		do
			x := a_x
			update_memory_field ("x")
		end

	set_y (a_y: INTEGER_32)
		do
			y := a_y
			update_memory_field ("y")
		end

	set_l (a_l: INTEGER_32)
		do
			l := a_l
			update_memory_field ("l")
		end

	set_vpos (a_vpos: INTEGER_32)
		do
			vpos := a_vpos
			update_memory_field ("vpos")
		end

	set_cpos (a_cpos: INTEGER_32)
		do
			cpos := a_cpos
			update_memory_field ("cpos")
		end

	set_checked (a_checked: INTEGER_32)
		do
			checked := a_checked
			update_memory_field ("checked")
		end

	set_item (a_item: CE_POINTER [PD_DIALOG_ITEM])
		do
			item := a_item
			update_memory_field ("item")
		end

	set_history (a_history: PD_LIST_HEAD)
		do
			history := a_history
			update_memory_field ("history")
		end

	set_cur_hist (a_cur_hist: CE_POINTER [PD_HISTORY_ITEM])
		do
			cur_hist := a_cur_hist
			update_memory_field ("cur_hist")
		end

	set_cdata (a_cdata: CE_POINTER [NATURAL_8])
		do
			cdata := a_cdata
			update_memory_field ("cdata")
		end

feature {ANY} -- public routines

	box_sel_move (a_dist: INTEGER_32)
		local
			l_box: CE_POINTER [PD_DLG_DATA_ITEM_DATA_BOX]
			l_new_sel: INTEGER_32
			l_new_top: INTEGER_32
		do
			l_box := create {CE_CLASS_POINTER [PD_DLG_DATA_ITEM_DATA_BOX]}.make_cast (item.item.data)
			l_new_sel := l_box.item.sel + a_dist
			l_new_top := l_box.item.box_top
			if l_new_sel < 0 then
				l_new_sel := 0
			elseif l_new_sel >= l_box.item.list_len then
				l_new_sel := l_box.item.list_len - 1
			end
			if l_new_sel >= l_new_top + item.item.gid then
				l_new_top := (l_new_sel - item.item.gid) + 1
				if l_new_top < 0 then
					(create {P_LINKS_STATIC}).errfile.item := create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bfu.c"))
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_LINKS_STATIC}).errline.item := 1_439
					update_class_fields -- Current might have been changed; synchronize
					debug_msg ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("Newly calculated box_top is an extremely wrong value (%%d). It should not be below zero.")), l_new_top])
					update_class_fields -- Current might have been changed; synchronize
				end
			elseif l_new_sel < l_new_top then
				l_new_top := l_new_sel
			end
			l_box.item.sel := l_new_sel
			update_class_fields -- Current might have been changed; synchronize
			l_box.item.box_top := l_new_top
			update_class_fields -- Current might have been changed; synchronize
		end

feature {PD_DIALOG_ITEM_DATA} -- private routines

	debug_msg (a_args: TUPLE [a_m: CE_POINTER [NATURAL_8]])
		local
			l_l: CE_VA_LIST
		do
			create l_l.make_shared (default_pointer)

			l_l := create {CE_VA_LIST}.make (a_args, agent debug_msg)
			sprintf ([create {CE_INTEGER_8_POINTER}.make_cast (((create {P_ERROR_STATIC}).errbuf.item)), ce_string ("DEBUG MESSAGE at %%s:%%d: "), (create {P_LINKS_STATIC}).errfile.item, (create {P_LINKS_STATIC}).errline.item]).do_nothing
			strcat (create {CE_INTEGER_8_POINTER}.make_cast (((create {P_ERROR_STATIC}).errbuf.item)), create {CE_INTEGER_8_POINTER}.make_cast (a_args.a_m)).do_nothing;
			(create {P_ERROR_STATIC}).er (0, (create {P_ERROR_STATIC}).errbuf.item, l_l.twin)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			x := a_other.x
			y := a_other.y
			l := a_other.l
			vpos := a_other.vpos
			cpos := a_other.cpos
			checked := a_other.checked
			item := a_other.item
			history := a_other.history.twin
			cur_hist := a_other.cur_hist
			cdata := a_other.cdata
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				x = a_other.x and
				y = a_other.y and
				l = a_other.l and
				vpos = a_other.vpos and
				cpos = a_other.cpos and
				checked = a_other.checked and
				item.is_equal (a_other.item) and
				history.is_equal (a_other.history) and
				cur_hist.is_equal (a_other.cur_hist) and
				cdata.is_equal (a_other.cdata)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("x"),
					create {CE_STRUCTURE_FIELD}.make ("y"),
					create {CE_STRUCTURE_FIELD}.make ("l"),
					create {CE_STRUCTURE_FIELD}.make ("vpos"),
					create {CE_STRUCTURE_FIELD}.make ("cpos"),
					create {CE_STRUCTURE_FIELD}.make ("checked"),
					create {CE_STRUCTURE_FIELD}.make ("item"),
					create {CE_STRUCTURE_FIELD}.make ("history"),
					create {CE_STRUCTURE_FIELD}.make ("cur_hist"),
					create {CE_STRUCTURE_FIELD}.make ("cdata")
				>>)
			init_structure_info
		end

end
