class
	PD_RGB

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {ANY} -- fields

	r: NATURAL_8 assign set_r

	g: NATURAL_8 assign set_g

	b: NATURAL_8 assign set_b

	pad: NATURAL_8 assign set_pad

feature {ANY} -- field setters

	set_r (a_r: NATURAL_8)
		do
			r := a_r
			update_memory_field ("r")
		end

	set_g (a_g: NATURAL_8)
		do
			g := a_g
			update_memory_field ("g")
		end

	set_b (a_b: NATURAL_8)
		do
			b := a_b
			update_memory_field ("b")
		end

	set_pad (a_pad: NATURAL_8)
		do
			pad := a_pad
			update_memory_field ("pad")
		end

feature {PD_RGB} -- private attributes

	palette: CE_STATIC [CE_ARRAY [PD_RGB]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_RGB]}.make_initialized (<<
					create {PD_RGB}.make_initialized ([(0).to_natural_8, (0).to_natural_8, (0).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(128).to_natural_8, (0).to_natural_8, (0).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(0).to_natural_8, (128).to_natural_8, (0).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(170).to_natural_8, (85).to_natural_8, (0).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(0).to_natural_8, (0).to_natural_8, (128).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(128).to_natural_8, (0).to_natural_8, (128).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(0).to_natural_8, (128).to_natural_8, (128).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(170).to_natural_8, (170).to_natural_8, (170).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(85).to_natural_8, (85).to_natural_8, (85).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(255).to_natural_8, (85).to_natural_8, (85).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(85).to_natural_8, (255).to_natural_8, (85).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(255).to_natural_8, (255).to_natural_8, (85).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(85).to_natural_8, (85).to_natural_8, (255).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(255).to_natural_8, (85).to_natural_8, (255).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(85).to_natural_8, (255).to_natural_8, (255).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(255).to_natural_8, (255).to_natural_8, (255).to_natural_8, (0).to_natural_8]),
					create {PD_RGB}.make_initialized ([(255).to_natural_8, (255).to_natural_8, (255).to_natural_8, (0).to_natural_8])
				>>)
		end

	rgb_cache: CE_STATIC [CE_ARRAY [PD_RGB_CACHE_ENTRY]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_RGB_CACHE_ENTRY]}.make (<<4_096>>)
		end

	cache_init: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

feature {ANY} -- public routines

	color_distance (a_c2: CE_POINTER [PD_RGB]): INTEGER_32
		require
			not a_c2.is_void
		do
			Result := (((3 * (r.to_integer_32 - (a_c2.item.r).to_integer_32)) * (r.to_integer_32 - (a_c2.item.r).to_integer_32) + (4 * (g.to_integer_32 - (a_c2.item.g).to_integer_32)) * (g.to_integer_32 - (a_c2.item.g).to_integer_32)) + (2 * (b.to_integer_32 - (a_c2.item.b).to_integer_32)) * (b.to_integer_32 - (a_c2.item.b).to_integer_32))
		end

	find_nearest_color (a_l: INTEGER_32): INTEGER_32
		local
			l_dist: INTEGER_32
			l_dst: INTEGER_32
			l_min: INTEGER_32
			l_i: INTEGER_32
			l_h: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not cache_init.item.to_boolean then
						l_pos := -1 + 2 -- goto initialize
					end
				when 1 then -- back:
					l_h := ((((((r.to_integer_32).bit_shift_left (3)) + ((g.to_integer_32).bit_shift_left (2))) + b.to_integer_32) + a_l)).bit_and (4_095)
					if rgb_cache.item[l_h].color /= -1 then
						if rgb_cache.item[l_h].l = a_l then
							if (rgb_cache.item[l_h].rgb.r).to_integer_32 = r.to_integer_32 then
								if (rgb_cache.item[l_h].rgb.g).to_integer_32 = g.to_integer_32 then
									if (rgb_cache.item[l_h].rgb.b).to_integer_32 = b.to_integer_32 then
										Result := (rgb_cache.item[l_h].color)
										l_return := True
									end
								end
							end
						end
					end
					if not l_return then
						l_dist := 16_777_215
						l_min := 0
						l_i := 0
						from
						until
							not (l_i < a_l)
						loop
							l_dst := ((create {CE_CLASS_POINTER [PD_RGB]}.make_shared (c_pointer))).item.color_distance (palette.item + l_i)
							update_class_fields -- Current might have been changed; synchronize
							if l_dst < l_dist then
								l_dist := l_dst
								l_min := l_i
							end
							l_i := l_i + 1
						end
						rgb_cache.item[l_h].color := l_min
						update_class_fields -- Current might have been changed; synchronize
						rgb_cache.item[l_h].l := a_l
						update_class_fields -- Current might have been changed; synchronize
						rgb_cache.item[l_h].rgb.r := r
						update_class_fields -- Current might have been changed; synchronize
						rgb_cache.item[l_h].rgb.g := g
						update_class_fields -- Current might have been changed; synchronize
						rgb_cache.item[l_h].rgb.b := b
						update_class_fields -- Current might have been changed; synchronize
						Result := l_min
						l_return := True
					end
					if l_return then
						l_pos := -1 + 3 -- skip if block
					end
				when 2 then -- if: | initialize:
						l_h := 0
						from
						until
							not (l_h < 4_096)
						loop
							rgb_cache.item[l_h].color := -1
							update_class_fields -- Current might have been changed; synchronize
							l_h := l_h + 1
						end
						cache_init.item := 1
						l_pos := -1 + 1 -- goto back
				when 3 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			r := a_other.r
			g := a_other.g
			b := a_other.b
			pad := a_other.pad
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				r = a_other.r and
				g = a_other.g and
				b = a_other.b and
				pad = a_other.pad
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("r"),
					create {CE_STRUCTURE_FIELD}.make ("g"),
					create {CE_STRUCTURE_FIELD}.make ("b"),
					create {CE_STRUCTURE_FIELD}.make ("pad")
				>>)
			init_structure_info
		end

end
