class
	P_EASY_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_STRING_STATIC

feature {ANY} -- public attributes

	initialized: CE_STATIC [NATURAL_32]
		once ("PROCESS")
		end

	init_flags: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	Curl_cmalloc: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := malloc_address
		end

	Curl_cfree: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := free_address
		end

	Curl_crealloc: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := realloc_address
		end

	Curl_cstrdup: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := strdup_address
		end

	Curl_ccalloc: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := calloc_address
		end

feature {ANY} -- public routines

	win32_cleanup
		do
		end

	win32_init: NATURAL_32
		do
			Result := ({PC_CURLCODE}.CURLE_OK)
		end

	curl_global_init (a_flags: INTEGER_32): NATURAL_32
		local
			l_tmp: NATURAL_32
			l_return: BOOLEAN
		do
			l_tmp := initialized.item
			initialized.item := initialized.item + 1
			if l_tmp.to_boolean then
				Result := ({PC_CURLCODE}.CURLE_OK)
				l_return := True
			end
			if not l_return then
				Curl_cmalloc.item := malloc_address
				Curl_cfree.item := free_address
				Curl_crealloc.item := realloc_address
				Curl_cstrdup.item := strdup_address
				Curl_ccalloc.item := calloc_address
				if ((a_flags).bit_and (((1).bit_shift_left (1)).to_integer_32)).to_boolean then
					if (win32_init) /= {NATURAL_32} 0 then
						Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
						l_return := True
					end
				end
			end
			if not l_return then
				init_flags.item := a_flags;
				(create {P_CURL_RAND_STATIC}).Curl_srand
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	curl_global_init_mem (a_flags: INTEGER_32; a_m, a_f, a_r, a_s, a_c: CE_ROUTINE): NATURAL_32
		local
			l_code: NATURAL_32
			l_return: BOOLEAN
		do
			l_code := {PC_CURLCODE}.CURLE_OK
			if a_m.is_void then
				Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
				l_return := True
			elseif a_f.is_void then
				Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
				l_return := True
			elseif a_r.is_void then
				Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
				l_return := True
			elseif a_s.is_void then
				Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
				l_return := True
			elseif a_c.is_void then
				Result := ({PC_CURLCODE}.CURLE_FAILED_INIT)
				l_return := True
			end
			if not l_return and then (initialized.item.to_boolean) then
				Result := ({PC_CURLCODE}.CURLE_OK)
				l_return := True
			end
			if not l_return then
				l_code := curl_global_init (a_flags)
				if l_code = {NATURAL_32} 0 then
					Curl_cmalloc.item := a_m
					Curl_cfree.item := a_f
					Curl_cstrdup.item := a_s
					Curl_crealloc.item := a_r
					Curl_ccalloc.item := a_c
				end
				Result := l_code
			end
		end

	curl_global_cleanup
		local
			l_return: BOOLEAN
		do
			if not initialized.item.to_boolean then
				l_return := True
			end
			if not l_return then
				initialized.item := initialized.item - 1
				if initialized.item.to_boolean then
					l_return := True
				end
			end
			if not l_return then
				(create {P_HOSTIP_STATIC}).Curl_global_host_cache_dtor
				if ((init_flags.item).bit_and (1)).to_boolean then
				end
				if ((init_flags.item).bit_and (((1).bit_shift_left (1)).to_integer_32)).to_boolean then
					win32_cleanup
				end
				init_flags.item := 0
			end
		end

	curl_easy_init: CE_POINTER [ANY]
		local
			l_res: NATURAL_32
			l_data: CE_ARRAY [CE_POINTER [PD_SESSIONHANDLE]]
			l_return: BOOLEAN
		do
			create {CE_POINTER_ARRAY [PD_SESSIONHANDLE]} l_data.make (<<1>>)

			if not initialized.item.to_boolean then
				l_res := curl_global_init (((1).bit_or (((1).bit_shift_left (1)))).to_integer_32)
				if l_res.to_boolean then
					Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				l_res := (create {P_URL_STATIC}).Curl_open (l_data)
				if l_res /= {NATURAL_32} 0 then
					Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				Result := (create {CE_VOID_POINTER}.make_cast (l_data.item))
			end
		end

	curl_easy_setopt (a_args: TUPLE [a_curl: CE_POINTER [ANY]; a_tag: NATURAL_32]): NATURAL_32
		local
			l_arg: CE_VA_LIST
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ret: NATURAL_32
			l_return: BOOLEAN
		do
			create l_arg.make_shared (default_pointer)

			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_args.a_curl)
			if a_args.a_curl.is_void then
				Result := ({PC_CURLCODE}.CURLE_BAD_FUNCTION_ARGUMENT)
				l_return := True
			end
			if not l_return then
				l_arg := create {CE_VA_LIST}.make (a_args, agent curl_easy_setopt)
				l_ret := (create {P_URL_STATIC}).Curl_setopt (l_data, a_args.a_tag, l_arg.twin)
				Result := l_ret
			end
		end

	curl_easy_perform (a_curl: CE_POINTER [ANY]): NATURAL_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_ptr: CE_POINTER [PD_CURL_HASH]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_curl)
					if l_data.is_void then
						Result := ({PC_CURLCODE}.CURLE_BAD_FUNCTION_ARGUMENT)
						l_return := True
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if (l_data.item.share).is_void then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if ((l_data.item.share).item.hostcache).is_void then
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if l_data.item.set.global_dns_cache then
							if l_data.item.dns.hostcachetype /= {NATURAL_32} 2 then
								if l_data.item.dns.hostcachetype = {NATURAL_32} 1 then
									(create {P_HASH_STATIC}).Curl_hash_destroy (l_data.item.dns.hostcache)
									l_data.item.dns.hostcachetype := {PC_HOSTCACHETYPE}.HCACHE_NONE
									l_data.item.dns.hostcache := create {CE_CLASS_POINTER [PD_CURL_HASH]}.make_shared (default_pointer)
								end
								l_ptr := (create {P_HOSTIP_STATIC}).Curl_global_host_cache_init
								if (not l_ptr.is_void) then
									l_data.item.dns.hostcache := l_ptr
									l_data.item.dns.hostcachetype := {PC_HOSTCACHETYPE}.HCACHE_GLOBAL
								end
							end
						end
						if (l_data.item.dns.hostcache).is_void then
							l_data.item.dns.hostcachetype := {PC_HOSTCACHETYPE}.HCACHE_PRIVATE
							l_data.item.dns.hostcache := (create {P_HOSTIP_STATIC}).Curl_mk_dnscache
							if (l_data.item.dns.hostcache).is_void then
								Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
								l_return := True
							end
						end
				when 5 then -- if done
					if not l_return and then ((l_data.item.state.connc).is_void) then
						l_data.item.state.connc := (create {P_URL_STATIC}).Curl_mk_connc (0, -1)
						if (l_data.item.state.connc).is_void then
							Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
							l_return := True
						end
					end
					if not l_return then
						Result := (l_data.item.Curl_perform)
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	curl_easy_cleanup (a_curl: CE_POINTER [ANY])
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_return: BOOLEAN
		do
			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_curl)
			if l_data.is_void then
				l_return := True
			end
			if not l_return then
				l_data.item.Curl_close.do_nothing
			end
		end

	curl_easy_getinfo (a_args: TUPLE [a_curl: CE_POINTER [ANY]; a_info: NATURAL_32]): NATURAL_32
		local
			l_arg: CE_VA_LIST
			l_paramp: CE_POINTER [ANY]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
		do
			create l_arg.make_shared (default_pointer)

			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_args.a_curl)
			l_arg := create {CE_VA_LIST}.make (a_args, agent curl_easy_getinfo)
			l_paramp := (create {CE_VOID_POINTER}.make_cast (l_arg.pointer_item))
			Result := ((create {P_GETINFO_STATIC}).Curl_getinfo ([l_data, a_args.a_info, l_paramp]))
		end

	curl_easy_duphandle (a_incurl: CE_POINTER [ANY]): CE_POINTER [ANY]
		local
			l_fail: BOOLEAN
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_outcurl: CE_POINTER [PD_SESSIONHANDLE]
			l_return, l_break: BOOLEAN
		do
			l_fail := (1).to_boolean
			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_incurl)
			l_outcurl := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast ((create {CE_VOID_POINTER}.make_cast (Curl_ccalloc.item.pointer_item ([(1).to_natural_32, (create {PD_SESSIONHANDLE}.make).structure_size.to_natural_32]))))
			if l_outcurl.is_void then
				Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				from
				until
					(l_break)
				loop
					l_break := False
					l_outcurl.item.state.headerbuff := create {CE_INTEGER_8_POINTER}.make_cast ((create {CE_VOID_POINTER}.make_cast (Curl_cmalloc.item.pointer_item ([(256).to_natural_32]))))
					if (l_outcurl.item.state.headerbuff).is_void then
						l_break := True
					end
					if not (l_break) then
						l_outcurl.item.state.headersize := (256).to_natural_32
						if (l_outcurl.item.Curl_dupset (l_data)) /= {NATURAL_32} 0 then
							l_break := True
						end
					end
					if not (l_break) then
						l_outcurl.item.state.connc := create {CE_CLASS_POINTER [PD_CONNCACHE]}.make_shared (default_pointer)
						l_outcurl.item.state.lastconnect := -1
						l_outcurl.item.progress.flags := l_data.item.progress.flags
						l_outcurl.item.progress.callback := l_data.item.progress.callback
						if (not (l_data.item.cookies).is_void) then
							l_outcurl.item.cookies := (create {P_COOKIE_STATIC}).Curl_cookie_init (l_data, (l_data.item.cookies).item.filename, l_outcurl.item.cookies, l_data.item.set.cookiesession)
							if (l_outcurl.item.cookies).is_void then
								l_break := True
							end
						end
					end
					if not (l_break) and then (not (l_data.item.change.cookielist).is_void) then
						l_outcurl.item.change.cookielist := (create {P_SLIST_STATIC}).Curl_slist_duplicate (l_data.item.change.cookielist)
						if (l_outcurl.item.change.cookielist).is_void then
							l_break := True
						end
					end
					if not (l_break) and then (not (l_data.item.change.url).is_void) then
						l_outcurl.item.change.url := create {CE_INTEGER_8_POINTER}.make_cast (Curl_cstrdup.item.pointer_item ([l_data.item.change.url]))
						if (l_outcurl.item.change.url).is_void then
							l_break := True
						end
						if not (l_break) then
							l_outcurl.item.change.url_alloc := (1).to_boolean
						end
					end
					if not (l_break) and then (not (l_data.item.change.referer).is_void) then
						l_outcurl.item.change.referer := create {CE_INTEGER_8_POINTER}.make_cast (Curl_cstrdup.item.pointer_item ([l_data.item.change.referer]))
						if (l_outcurl.item.change.referer).is_void then
							l_break := True
						end
						if not (l_break) then
							l_outcurl.item.change.referer_alloc := (1).to_boolean
						end
					end
					if not (l_break) then
						l_outcurl.item.Curl_easy_initHandleData
						l_outcurl.item.magic := {NATURAL_32} 3_235_830_701
						l_fail := (0).to_boolean
						l_break := True
					end
				end
				l_break := False
				if l_fail then
					if (not l_outcurl.is_void) then
						if (not (l_outcurl.item.state.connc).is_void) then
							if (l_outcurl.item.state.connc).item.type = {NATURAL_32} 0 then
								(l_outcurl.item.state.connc).item.Curl_rm_connc
							end
						end
						if (not (l_outcurl.item.state.headerbuff).is_void) then
							Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_outcurl.item.state.headerbuff)])
						end
						if (not (l_outcurl.item.change.cookielist).is_void) then
							(create {P_SLIST_STATIC}).curl_slist_free_all (l_outcurl.item.change.cookielist)
						end
						if (not (l_outcurl.item.change.url).is_void) then
							Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_outcurl.item.change.url)])
						end
						if (not (l_outcurl.item.change.referer).is_void) then
							Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_outcurl.item.change.referer)])
						end
						(create {P_URL_STATIC}).Curl_freeset (l_outcurl)
						Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_outcurl)])
						l_outcurl := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_shared (default_pointer)
					end
				end
				Result := (create {CE_VOID_POINTER}.make_cast (l_outcurl))
			end
		end

	curl_easy_reset (a_curl: CE_POINTER [ANY])
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
		do
			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_curl);
			(create {P_URL_STATIC}).Curl_safefree (create {CE_VOID_POINTER}.make_cast (l_data.item.state.pathbuffer))
			l_data.item.state.pathbuffer := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer);
			(create {P_URL_STATIC}).Curl_safefree (l_data.item.state.proto.generic)
			l_data.item.state.proto.generic := create {CE_VOID_POINTER}.make_shared (default_pointer);
			(create {P_URL_STATIC}).Curl_freeset (l_data)
			memset (create {CE_VOID_POINTER}.make_cast ((create {CE_CLASS_POINTER [PD_USERDEFINED]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("set")))), 0, (create {PD_USERDEFINED}.make).structure_size.to_natural_32).do_nothing;
			(create {CE_CLASS_POINTER [PD_USERDEFINED]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("set"))).item.Curl_init_userdefined.do_nothing
			memset (create {CE_VOID_POINTER}.make_cast ((create {CE_CLASS_POINTER [PD_PROGRESS]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("progress")))), 0, (create {PD_PROGRESS}.make).structure_size.to_natural_32).do_nothing
			l_data.item.Curl_easy_initHandleData
			l_data.item.progress.flags := (l_data.item.progress.flags).bit_or (((1).bit_shift_left (4)))
			l_data.item.state.current_speed := (-1).to_integer_64
		end

	curl_easy_pause (a_curl: CE_POINTER [ANY]; a_action: INTEGER_32): NATURAL_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_k: CE_POINTER [PD_SINGLEREQUEST]
			l_result: NATURAL_32
			l_newstate: INTEGER_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tempwrite: CE_POINTER [INTEGER_8]
			l_freewrite: CE_POINTER [INTEGER_8]
			l_tempsize: NATURAL_32
			l_temptype: INTEGER_32
			l_chunklen: NATURAL_32
			l_newptr: CE_POINTER [INTEGER_8]
			l_break: BOOLEAN
		do
			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_curl)
			l_k := create {CE_CLASS_POINTER [PD_SINGLEREQUEST]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("req"))
			l_result := {PC_CURLCODE}.CURLE_OK
			l_newstate := (l_k.item.keepon).bit_and ((((((1).bit_shift_left (4))).bit_or (((1).bit_shift_left (5))))).bit_not)
			if ((a_action).bit_and (1)).to_boolean then
				l_tmp := (1).bit_shift_left (4)
			else
				l_tmp := 0
			end
			if ((a_action).bit_and (((1).bit_shift_left (2)))).to_boolean then
				l_tmp___0 := (1).bit_shift_left (5)
			else
				l_tmp___0 := 0
			end
			l_newstate := (l_newstate).bit_or (((l_tmp).bit_or (l_tmp___0)))
			l_k.item.keepon := l_newstate
			if not ((l_newstate).bit_and (((1).bit_shift_left (4)))).to_boolean then
				if (not (l_data.item.state.tempwrite).is_void) then
					l_tempwrite := l_data.item.state.tempwrite
					l_freewrite := l_tempwrite
					l_tempsize := l_data.item.state.tempwritesize
					l_temptype := l_data.item.state.tempwritetype
					l_data.item.state.tempwrite := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					from
					until
						(l_break)
					loop
						l_break := False
						if l_tempsize > {NATURAL_32} 16_384 then
							l_chunklen := (16_384).to_natural_32
						else
							l_chunklen := l_tempsize
						end
						l_result := (l_data.item.state.current_conn).item.Curl_client_write (l_temptype, l_tempwrite, l_chunklen)
						if l_result.to_boolean then
							l_break := True
						end
						if not (l_break) then
							if (not (l_data.item.state.tempwrite).is_void) then
								if (l_tempsize - l_chunklen).to_boolean then
									l_newptr := create {CE_INTEGER_8_POINTER}.make_cast ((create {CE_VOID_POINTER}.make_cast (Curl_crealloc.item.pointer_item ([create {CE_VOID_POINTER}.make_cast (l_data.item.state.tempwrite), l_tempsize]))))
									if l_newptr.is_void then
										Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_data.item.state.tempwrite)])
										l_data.item.state.tempwrite := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
										l_result := {PC_CURLCODE}.CURLE_OUT_OF_MEMORY
										l_break := True
									end
									if not (l_break) then
										l_data.item.state.tempwrite := l_newptr
										memcpy (create {CE_VOID_POINTER}.make_cast (l_newptr), create {CE_VOID_POINTER}.make_cast (l_tempwrite), l_tempsize).do_nothing
										l_data.item.state.tempwritesize := l_tempsize
										l_break := True
									end
								else
									l_tempsize := l_tempsize - l_chunklen
									l_tempwrite := l_tempwrite + l_chunklen.to_integer_32
								end
							else
								l_tempsize := l_tempsize - l_chunklen
								l_tempwrite := l_tempwrite + l_chunklen.to_integer_32
							end
						end
						if not (l_break) then
							if l_result = {NATURAL_32} 0 then
								if not l_tempsize.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
						end
					end
					l_break := False
					Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (l_freewrite)])
				end
			end
			Result := l_result
		end

	easy_connection (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_sfd: CE_POINTER [INTEGER_32]; a_connp: CE_POINTER [CE_POINTER [PD_CONNECTDATA]]): NATURAL_32
		local
			l_return: BOOLEAN
		do
			if a_data.is_void then
				Result := ({PC_CURLCODE}.CURLE_BAD_FUNCTION_ARGUMENT)
				l_return := True
			end
			if not l_return and then (not a_data.item.set.connect_only) then
				a_data.item.Curl_failf ([ce_string ("CONNECT_ONLY is required!")])
				Result := ({PC_CURLCODE}.CURLE_UNSUPPORTED_PROTOCOL)
				l_return := True
			end
			if not l_return then
				a_sfd.item := a_data.item.Curl_getconnectinfo (a_connp)
				if a_sfd.item = -1 then
					a_data.item.Curl_failf ([ce_string ("Failed to get recent socket")])
					Result := ({PC_CURLCODE}.CURLE_UNSUPPORTED_PROTOCOL)
					l_return := True
				end
			end
			if not l_return then
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	curl_easy_recv (a_curl, a_buffer: CE_POINTER [ANY]; a_buflen: NATURAL_32; a_n: CE_POINTER [NATURAL_32]): NATURAL_32
		local
			l_sfd: CE_ARRAY [INTEGER_32]
			l_ret: NATURAL_32
			l_n1: CE_ARRAY [INTEGER_32]
			l_c: CE_ARRAY [CE_POINTER [PD_CONNECTDATA]]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_sfd.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_n1.make (<<1>>)
			create {CE_POINTER_ARRAY [PD_CONNECTDATA]} l_c.make (<<1>>)

			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_curl)
			l_ret := easy_connection (l_data, l_sfd, l_c)
			if l_ret.to_boolean then
				Result := l_ret
				l_return := True
			end
			if not l_return then
				a_n.item := (0).to_natural_32
				l_ret := (l_c.item).item.Curl_read (l_sfd.item, create {CE_INTEGER_8_POINTER}.make_cast (a_buffer), a_buflen, l_n1)
				if l_ret /= {NATURAL_32} 0 then
					Result := l_ret
					l_return := True
				end
			end
			if not l_return then
				a_n.item := (l_n1.item).to_natural_32
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	curl_easy_send (a_curl, a_buffer: CE_POINTER [ANY]; a_buflen: NATURAL_32; a_n: CE_POINTER [NATURAL_32]): NATURAL_32
		local
			l_sfd: CE_ARRAY [INTEGER_32]
			l_ret: NATURAL_32
			l_n1: CE_ARRAY [INTEGER_32]
			l_c: CE_ARRAY [CE_POINTER [PD_CONNECTDATA]]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_sfd.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_n1.make (<<1>>)
			create {CE_POINTER_ARRAY [PD_CONNECTDATA]} l_c.make (<<1>>)

			l_c.item := create {CE_CLASS_POINTER [PD_CONNECTDATA]}.make_shared (default_pointer)
			l_data := create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (a_curl)
			l_ret := easy_connection (l_data, l_sfd, l_c)
			if l_ret.to_boolean then
				Result := l_ret
				l_return := True
			end
			if not l_return then
				a_n.item := (0).to_natural_32
				l_ret := (l_c.item).item.Curl_write (l_sfd.item, a_buffer, a_buflen, l_n1)
				if l_n1.item = -1 then
					Result := ({PC_CURLCODE}.CURLE_SEND_ERROR)
					l_return := True
				end
			end
			if not l_return and then ({NATURAL_32} 0 = l_ret) then
				if 0 = l_n1.item then
					Result := ({PC_CURLCODE}.CURLE_AGAIN)
					l_return := True
				end
			end
			if not l_return then
				a_n.item := (l_n1.item).to_natural_32
				Result := l_ret
			end
		end

end
