class
	PD_CURL_LLIST

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]} head.make
			create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]} tail.make
			create dtor.make
		end

feature {ANY} -- fields

	head: CE_POINTER [PD_CURL_LLIST_ELEMENT] assign set_head

	tail: CE_POINTER [PD_CURL_LLIST_ELEMENT] assign set_tail

	dtor: CE_ROUTINE assign set_dtor

	size: NATURAL_32 assign set_size

feature {ANY} -- field setters

	set_head (a_head: CE_POINTER [PD_CURL_LLIST_ELEMENT])
		do
			head := a_head
			update_memory_field ("head")
		end

	set_tail (a_tail: CE_POINTER [PD_CURL_LLIST_ELEMENT])
		do
			tail := a_tail
			update_memory_field ("tail")
		end

	set_dtor (a_dtor: CE_ROUTINE)
		do
			dtor := a_dtor
			update_memory_field ("dtor")
		end

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

feature {ANY} -- public routines

	gethandleathead: CE_POINTER [PD_SESSIONHANDLE]
		local
			l_curr: CE_POINTER [PD_CURL_LLIST_ELEMENT]
			l_return: BOOLEAN
		do
			l_curr := head
			if (not l_curr.is_void) then
				Result := (create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_cast (l_curr.item.ptr))
				l_return := True
			end
			if not l_return then
				Result := (create {CE_CLASS_POINTER [PD_SESSIONHANDLE]}.make_shared (default_pointer))
			end
		end

	llist_init (a_dtor: CE_ROUTINE)
		do
			Current.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			Current.dtor := a_dtor
			update_class_fields -- Current might have been changed; synchronize
			Current.head := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			Current.tail := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
		end

	Curl_llist_remove (a_e: CE_POINTER [PD_CURL_LLIST_ELEMENT]; a_user: CE_POINTER [ANY]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if a_e.is_void then
				Result := 1
				l_return := True
			elseif size = {NATURAL_32} 0 then
				Result := 1
				l_return := True
			end
			if not l_return then
				if a_e.is_equal (head) then
					Current.head := a_e.item.next
					update_class_fields -- Current might have been changed; synchronize
					if head.is_void then
						Current.tail := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
					else
						(a_e.item.next).item.prev := create {CE_CLASS_POINTER [PD_CURL_LLIST_ELEMENT]}.make_shared (default_pointer)
						update_class_fields -- Current might have been changed; synchronize
					end
				else
					(a_e.item.prev).item.next := a_e.item.next
					update_class_fields -- Current might have been changed; synchronize
					if (a_e.item.next).is_void then
						Current.tail := a_e.item.prev
						update_class_fields -- Current might have been changed; synchronize
					else
						(a_e.item.next).item.prev := a_e.item.prev
						update_class_fields -- Current might have been changed; synchronize
					end
				end
				dtor.call ([a_user, a_e.item.ptr])
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_EASY_STATIC}).Curl_cfree.item.call ([create {CE_VOID_POINTER}.make_cast (a_e)])
				update_class_fields -- Current might have been changed; synchronize
				Current.size := size - 1
				update_class_fields -- Current might have been changed; synchronize
				Result := 1
			end
		end

	Curl_llist_count: NATURAL_32
		do
			Result := size
		end

feature {ANY}

	copy (a_other: like Current)
		do
			head := a_other.head
			tail := a_other.tail
			dtor := a_other.dtor
			size := a_other.size
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				head.is_equal (a_other.head) and
				tail.is_equal (a_other.tail) and
				dtor.is_equal (a_other.dtor) and
				size = a_other.size
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("head"),
					create {CE_STRUCTURE_FIELD}.make ("tail"),
					create {CE_STRUCTURE_FIELD}.make ("dtor"),
					create {CE_STRUCTURE_FIELD}.make ("size")
				>>)
			init_structure_info
		end

end
