class
	PD_FORM

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_FORMDATA]} data.make
			create {CE_CLASS_POINTER [SD_IO_FILE]} fp.make
			create fread_func.make
			Precursor
		end

feature {ANY} -- fields

	data: CE_POINTER [PD_FORMDATA] assign set_data

	sent: NATURAL_32 assign set_sent

	fp: CE_POINTER [SD_IO_FILE] assign set_fp

	fread_func: CE_ROUTINE assign set_fread_func

feature {ANY} -- field setters

	set_data (a_data: CE_POINTER [PD_FORMDATA])
		do
			data := a_data
			update_memory_field ("data")
		end

	set_sent (a_sent: NATURAL_32)
		do
			sent := a_sent
			update_memory_field ("sent")
		end

	set_fp (a_fp: CE_POINTER [SD_IO_FILE])
		do
			fp := a_fp
			update_memory_field ("fp")
		end

	set_fread_func (a_fread_func: CE_ROUTINE)
		do
			fread_func := a_fread_func
			update_memory_field ("fread_func")
		end

feature {ANY} -- public routines

	Curl_FormInit (a_formdata: CE_POINTER [PD_FORMDATA]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if a_formdata.is_void then
				Result := 1
				l_return := True
			end
			if not l_return then
				Current.data := a_formdata
				update_class_fields -- Current might have been changed; synchronize
				Current.sent := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				Current.fp := create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				Current.fread_func := create {CE_ROUTINE}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	readfromfile (a_buffer: CE_POINTER [INTEGER_8]; a_size: NATURAL_32): NATURAL_32
		local
			l_nread: NATURAL_32
			l_callback: BOOLEAN
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_callback := (data.item.type = {NATURAL_32} 2)
					if l_callback then
						l_nread := fread_func.natural_32_item ([a_buffer, (1).to_natural_32, a_size, create {CE_VOID_POINTER}.make_cast (data.item.line)])
						update_class_fields -- Current might have been changed; synchronize
					else
						if fp.is_void then
							Current.fp := fopen (data.item.line, ce_string ("rb"))
							update_class_fields -- Current might have been changed; synchronize
							if fp.is_void then
								Result := (-1).to_natural_32
								l_return := True
							end
						end
						if not l_return then
							l_nread := fread (create {CE_VOID_POINTER}.make_cast (a_buffer), (1).to_natural_32, a_size, fp)
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					if l_return then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:
						if l_nread.to_boolean then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (l_nread > a_size) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						if not l_callback then
							fclose (fp).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							Current.fp := create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (default_pointer)
							update_class_fields -- Current might have been changed; synchronize
						end
						Current.data := data.item.next
						update_class_fields -- Current might have been changed; synchronize
				when 6 then -- if done
						Result := l_nread
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			data := a_other.data
			sent := a_other.sent
			fp := a_other.fp
			fread_func := a_other.fread_func
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				data.is_equal (a_other.data) and
				sent = a_other.sent and
				fp.is_equal (a_other.fp) and
				fread_func.is_equal (a_other.fread_func)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("data"),
					create {CE_STRUCTURE_FIELD}.make ("sent"),
					create {CE_STRUCTURE_FIELD}.make ("fp"),
					create {CE_STRUCTURE_FIELD}.make ("fread_func")
				>>)
			init_structure_info
		end

end
