class
	PD_TFTP_STATE_DATA

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_TIME_STATIC
		redefine
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end
	S_SYS_SOCKET_STATIC
		redefine
			copy,
			is_equal
		end
	S_BITS_ERRNO_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_CONNECTDATA]} conn.make
			create local_addr.make
			create remote_addr.make
			create rpacket.make
			create spacket.make
			Precursor
		end

feature {ANY} -- fields

	state: NATURAL_32 assign set_state

	mode: NATURAL_32 assign set_mode

	error: INTEGER_32 assign set_error

	event: INTEGER_32 assign set_event

	conn: CE_POINTER [PD_CONNECTDATA] assign set_conn

	sockfd: INTEGER_32 assign set_sockfd

	retries: INTEGER_32 assign set_retries

	retry_time: INTEGER_32 assign set_retry_time

	retry_max: INTEGER_32 assign set_retry_max

	start_time: INTEGER_32 assign set_start_time

	max_time: INTEGER_32 assign set_max_time

	rx_time: INTEGER_32 assign set_rx_time

	block: NATURAL_16 assign set_block

	local_addr: PD_CURL_SOCKADDR_STORAGE assign set_local_addr

	remote_addr: PD_CURL_SOCKADDR_STORAGE assign set_remote_addr

	remote_addrlen: NATURAL_32 assign set_remote_addrlen

	rbytes: INTEGER_32 assign set_rbytes

	sbytes: INTEGER_32 assign set_sbytes

	blksize: INTEGER_32 assign set_blksize

	requested_blksize: INTEGER_32 assign set_requested_blksize

	rpacket: PD_TFTP_PACKET assign set_rpacket

	spacket: PD_TFTP_PACKET assign set_spacket

feature {ANY} -- field setters

	set_state (a_state: NATURAL_32)
		do
			state := a_state
			update_memory_field ("state")
		end

	set_mode (a_mode: NATURAL_32)
		do
			mode := a_mode
			update_memory_field ("mode")
		end

	set_error (a_error: INTEGER_32)
		do
			error := a_error
			update_memory_field ("error")
		end

	set_event (a_event: INTEGER_32)
		do
			event := a_event
			update_memory_field ("event")
		end

	set_conn (a_conn: CE_POINTER [PD_CONNECTDATA])
		do
			conn := a_conn
			update_memory_field ("conn")
		end

	set_sockfd (a_sockfd: INTEGER_32)
		do
			sockfd := a_sockfd
			update_memory_field ("sockfd")
		end

	set_retries (a_retries: INTEGER_32)
		do
			retries := a_retries
			update_memory_field ("retries")
		end

	set_retry_time (a_retry_time: INTEGER_32)
		do
			retry_time := a_retry_time
			update_memory_field ("retry_time")
		end

	set_retry_max (a_retry_max: INTEGER_32)
		do
			retry_max := a_retry_max
			update_memory_field ("retry_max")
		end

	set_start_time (a_start_time: INTEGER_32)
		do
			start_time := a_start_time
			update_memory_field ("start_time")
		end

	set_max_time (a_max_time: INTEGER_32)
		do
			max_time := a_max_time
			update_memory_field ("max_time")
		end

	set_rx_time (a_rx_time: INTEGER_32)
		do
			rx_time := a_rx_time
			update_memory_field ("rx_time")
		end

	set_block (a_block: NATURAL_16)
		do
			block := a_block
			update_memory_field ("block")
		end

	set_local_addr (a_local_addr: PD_CURL_SOCKADDR_STORAGE)
		do
			local_addr := a_local_addr
			update_memory_field ("local_addr")
		end

	set_remote_addr (a_remote_addr: PD_CURL_SOCKADDR_STORAGE)
		do
			remote_addr := a_remote_addr
			update_memory_field ("remote_addr")
		end

	set_remote_addrlen (a_remote_addrlen: NATURAL_32)
		do
			remote_addrlen := a_remote_addrlen
			update_memory_field ("remote_addrlen")
		end

	set_rbytes (a_rbytes: INTEGER_32)
		do
			rbytes := a_rbytes
			update_memory_field ("rbytes")
		end

	set_sbytes (a_sbytes: INTEGER_32)
		do
			sbytes := a_sbytes
			update_memory_field ("sbytes")
		end

	set_blksize (a_blksize: INTEGER_32)
		do
			blksize := a_blksize
			update_memory_field ("blksize")
		end

	set_requested_blksize (a_requested_blksize: INTEGER_32)
		do
			requested_blksize := a_requested_blksize
			update_memory_field ("requested_blksize")
		end

	set_rpacket (a_rpacket: PD_TFTP_PACKET)
		do
			rpacket := a_rpacket
			update_memory_field ("rpacket")
		end

	set_spacket (a_spacket: PD_TFTP_PACKET)
		do
			spacket := a_spacket
			update_memory_field ("spacket")
		end

feature {ANY} -- public routines

	tftp_set_timeouts: NATURAL_32
		local
			l_maxtime: INTEGER_32
			l_timeout: INTEGER_32
			l_timeout_ms: INTEGER_32
			l_start: BOOLEAN
			l_return: BOOLEAN
		do
			l_start := (state = {NATURAL_32} 0)
			time (create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("start_time"))).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_timeout_ms := conn.item.Curl_timeleft (create {CE_CLASS_POINTER [SD_TIMEVAL]}.make_shared (default_pointer), l_start)
			update_class_fields -- Current might have been changed; synchronize
			if l_timeout_ms < 0 then
				(conn.item.data).item.Curl_failf ([ce_string ("Connection time-out")])
				update_class_fields -- Current might have been changed; synchronize
				Result := ({PC_CURLCODE}.CURLE_OPERATION_TIMEDOUT)
				l_return := True
			end
			if not l_return then
				if l_start then
					l_maxtime := (l_timeout_ms + 500) // 1_000
					Current.max_time := start_time + l_maxtime
					update_class_fields -- Current might have been changed; synchronize
					l_timeout := l_maxtime
					Current.retry_max := l_timeout.to_integer_32 // 5
					update_class_fields -- Current might have been changed; synchronize
					if retry_max < 1 then
						Current.retry_max := 1
						update_class_fields -- Current might have been changed; synchronize
					end
					Current.retry_time := l_timeout.to_integer_32 // retry_max
					update_class_fields -- Current might have been changed; synchronize
					if retry_time < 1 then
						Current.retry_time := 1
						update_class_fields -- Current might have been changed; synchronize
					end
				else
					if l_timeout_ms > 0 then
						l_maxtime := (l_timeout_ms + 500) // 1_000
					else
						l_maxtime := (3_600).to_integer_32
					end
					Current.max_time := start_time + l_maxtime
					update_class_fields -- Current might have been changed; synchronize
					l_timeout := l_maxtime // 10
					Current.retry_max := l_timeout.to_integer_32 // 15
					update_class_fields -- Current might have been changed; synchronize
				end
				if retry_max < 3 then
					Current.retry_max := 3
					update_class_fields -- Current might have been changed; synchronize
				end
				if retry_max > 50 then
					Current.retry_max := 50
					update_class_fields -- Current might have been changed; synchronize
				end
				Current.retry_time := (l_timeout // retry_max.to_integer_32).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				if retry_time < 1 then
					Current.retry_time := 1
					update_class_fields -- Current might have been changed; synchronize
				end
				(create {P_SENDF_STATIC}).Curl_infof ([conn.item.data, ce_string ("set timeouts for state %%d; Total %%ld, retry %%d maxtry %%d%N"), state.to_integer_32, max_time - start_time, retry_time, retry_max])
				update_class_fields -- Current might have been changed; synchronize
				time (create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("rx_time"))).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	tftp_parse_option_ack (a_ptr: CE_POINTER [INTEGER_8]; a_len: INTEGER_32): NATURAL_32
		local
			l_tmp: CE_POINTER [INTEGER_8]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_option: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_value: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_blksize: INTEGER_32
			l_tsize: INTEGER_32
			l_tmp___0: NATURAL_32
			l_tmp___1: INTEGER_32
			l_tmp___2: NATURAL_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			create {CE_POINTER_ARRAY [INTEGER_8]} l_option.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_value.make (<<1>>)

			l_tmp := a_ptr
			l_data := conn.item.data
			Current.blksize := 512
			update_class_fields -- Current might have been changed; synchronize
			from
			until
				not (not (l_return) and then ((l_tmp.c_pointer.to_integer_32).to_natural_32 < ((a_ptr + a_len).c_pointer.to_integer_32).to_natural_32))
			loop
				l_tmp := tftp_option_get (l_tmp, ((a_ptr + a_len).distance (l_tmp)).to_natural_32, l_option, l_value)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp.is_void then
					l_data.item.Curl_failf ([ce_string ("Malformed ACK packet, rejecting")])
					update_class_fields -- Current might have been changed; synchronize
					Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
					l_return := True
				end
				if not l_return then
					(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("got option=(%%s) value=(%%s)%N"), l_option.item, l_value.item])
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___2 := strlen (l_option.item)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := (create {P_RAWSTR_STATIC}).Curl_raw_nequal (l_option.item, ce_string ("blksize"), l_tmp___2)
					update_class_fields -- Current might have been changed; synchronize
					if l_tmp___3.to_boolean then
						l_blksize := strtol (l_value.item, create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer), 10)
						update_class_fields -- Current might have been changed; synchronize
						if not l_blksize.to_boolean then
							l_data.item.Curl_failf ([ce_string ("invalid blocksize value in OACK packet")])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
							l_return := True
						elseif l_blksize > 65_464 then
							l_data.item.Curl_failf ([ce_string ("%%s (%%d)"), ce_string ("blksize is larger than max supported"), 65_464])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
							l_return := True
						elseif l_blksize < 8 then
							l_data.item.Curl_failf ([ce_string ("%%s (%%d)"), ce_string ("blksize is smaller than min supported"), 8])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
							l_return := True
						elseif l_blksize > requested_blksize.to_integer_32 then
							l_data.item.Curl_failf ([ce_string ("%%s (%%ld)"), ce_string ("server requested blksize larger than allocated"), l_blksize])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
							l_return := True
						end
						if not l_return then
							Current.blksize := l_blksize.to_integer_32
							update_class_fields; -- Current might have been changed; synchronize
							(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("%%s (%%d) %%s (%%d)%N"), ce_string ("blksize parsed from OACK"), blksize, ce_string ("requested"), requested_blksize])
							update_class_fields -- Current might have been changed; synchronize
						end
					else
						l_tmp___0 := strlen (l_option.item)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___1 := (create {P_RAWSTR_STATIC}).Curl_raw_nequal (l_option.item, ce_string ("tsize"), l_tmp___0)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___1.to_boolean then
							l_tsize := 0
							l_tsize := strtol (l_value.item, create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer), 10)
							update_class_fields; -- Current might have been changed; synchronize
							(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("%%s (%%ld)%N"), ce_string ("tsize parsed from OACK"), l_tsize])
							update_class_fields -- Current might have been changed; synchronize
							if not l_data.item.set.upload then
								if not l_tsize.to_boolean then
									l_data.item.Curl_failf ([ce_string ("invalid tsize -:%%s:- value in OACK packet"), l_value.item])
									update_class_fields -- Current might have been changed; synchronize
									Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
									l_return := True
								end
								if not l_return then
									l_data.item.Curl_pgrsSetDownloadSize (l_tsize.to_integer_64)
									update_class_fields -- Current might have been changed; synchronize
								end
							end
						end
					end
				end
			end
			if not l_return then
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	tftp_option_add (a_csize: NATURAL_32; a_buf, a_option: CE_POINTER [INTEGER_8]): NATURAL_32
		require
			not a_option.is_void
		local
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			l_return: BOOLEAN
		do
			l_tmp := strlen (a_option)
			update_class_fields -- Current might have been changed; synchronize
			if (l_tmp + a_csize) + {NATURAL_32} 1 > blksize.to_natural_32 then
				Result := (0).to_natural_32
				l_return := True
			end
			if not l_return then
				strcpy (a_buf, a_option).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___0 := strlen (a_option)
				update_class_fields -- Current might have been changed; synchronize
				Result := (l_tmp___0 + {NATURAL_32} 1)
			end
		end

	tftp_connect_for_tx (a_event: INTEGER_32): NATURAL_32
		local
			l_res: NATURAL_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_tmp: NATURAL_32
			l_return: BOOLEAN
		do
			l_data := conn.item.data;
			(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("%%s%N"), ce_string ("Connected for transmit")])
			update_class_fields -- Current might have been changed; synchronize
			Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_TX
			update_class_fields -- Current might have been changed; synchronize
			l_res := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_set_timeouts
			update_class_fields -- Current might have been changed; synchronize
			if l_res /= {NATURAL_32} 0 then
				Result := l_res
				l_return := True
			end
			if not l_return then
				l_tmp := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_tx (a_event)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
			end
		end

	tftp_connect_for_rx (a_event: INTEGER_32): NATURAL_32
		local
			l_res: NATURAL_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_tmp: NATURAL_32
			l_return: BOOLEAN
		do
			l_data := conn.item.data;
			(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("%%s%N"), ce_string ("Connected for receive")])
			update_class_fields -- Current might have been changed; synchronize
			Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_RX
			update_class_fields -- Current might have been changed; synchronize
			l_res := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_set_timeouts
			update_class_fields -- Current might have been changed; synchronize
			if l_res /= {NATURAL_32} 0 then
				Result := l_res
				l_return := True
			end
			if not l_return then
				l_tmp := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_rx (a_event)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
			end
		end

	tftp_send_first (a_event: INTEGER_32): NATURAL_32
		local
			l_sbytes: NATURAL_32
			l_senddata: INTEGER_32
			l_mode___1: CE_POINTER [INTEGER_8]
			l_filename: CE_POINTER [INTEGER_8]
			l_buf: CE_ARRAY [INTEGER_8]
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_res: NATURAL_32
			l_tmp: NATURAL_32
			l_tmp___0: NATURAL_32
			l_tmp___1: NATURAL_32
			l_tmp___2: NATURAL_32
			l_tmp___3: NATURAL_32
			l_tmp___4: NATURAL_32
			l_tmp___5: NATURAL_32
			l_tmp___6: NATURAL_32
			l_tmp___7: CE_POINTER [INTEGER_32]
			l_tmp___8: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_8_ARRAY} l_buf.make (<<64>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_mode___1 := ce_string ("octet")
					l_data := conn.item.data
					l_res := {PC_CURLCODE}.CURLE_OK
					if l_data.item.set.prefer_ascii then
						l_mode___1 := ce_string ("netascii")
					end
					inspect a_event -- switch 1 head
					when 0 then l_pos := -1 + 1
					when 7 then l_pos := -1 + 1
					when 6 then l_pos := -1 + 2
					when 4 then l_pos := -1 + 3
					when 3 then l_pos := -1 + 4
					when 5 then l_pos := -1 + 5
					else l_pos := -1 + 6
					end
				when 1 then -- switch 1, case 0 | switch 1, case 7
					Current.retries := retries + 1
					update_class_fields -- Current might have been changed; synchronize
					if retries > retry_max then
						Current.error := -98
						update_class_fields -- Current might have been changed; synchronize
						Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
						update_class_fields -- Current might have been changed; synchronize
						Result := l_res
						l_return := True
					end
					if not l_return then
						if l_data.item.set.upload then
							setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (2).to_natural_16)
							update_class_fields; -- Current might have been changed; synchronize
							(conn.item.data).item.req.upload_fromhere := create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + 4
							update_class_fields -- Current might have been changed; synchronize
							if l_data.item.set.infilesize /= -{INTEGER_64} 1 then
								l_data.item.Curl_pgrsSetUploadSize (l_data.item.set.infilesize)
								update_class_fields -- Current might have been changed; synchronize
							end
						else
							setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (1).to_natural_16)
							update_class_fields -- Current might have been changed; synchronize
						end
						l_filename := (create {P_ESCAPE_STATIC}).curl_easy_unescape (create {CE_VOID_POINTER}.make_cast (l_data), ((conn.item.data).item.state.path + 1), 0, create {CE_INTEGER_32_POINTER}.make_shared (default_pointer))
						update_class_fields -- Current might have been changed; synchronize
						if l_filename.is_void then
							Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
							l_return := True
						end
					end
					if not l_return then
						(create {P_MPRINTF_STATIC}).curl_msnprintf ([create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + 2, blksize.to_natural_32, ce_string ("%%s%%c%%s%%c"), l_filename, ('%/0/').code, l_mode___1, ('%/0/').code]).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp := strlen (l_filename)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := strlen (l_mode___1)
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := ({NATURAL_32} 4 + l_tmp) + l_tmp___0
						if l_data.item.set.upload then
							if l_data.item.set.infilesize /= -{INTEGER_64} 1 then
								(create {P_MPRINTF_STATIC}).curl_msnprintf ([l_buf, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (64).to_natural_32), ce_string ("%%lld"), l_data.item.set.infilesize]).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							else
								strcpy (l_buf, ce_string ("0")).do_nothing
								update_class_fields -- Current might have been changed; synchronize
							end
						else
							strcpy (l_buf, ce_string ("0")).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						l_tmp___1 := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_option_add (l_sbytes, create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + l_sbytes.to_integer_32, ce_string ("tsize"))
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := l_sbytes + l_tmp___1
						l_tmp___2 := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_option_add (l_sbytes, create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + l_sbytes.to_integer_32, l_buf)
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := l_sbytes + l_tmp___2;
						(create {P_MPRINTF_STATIC}).curl_msnprintf ([l_buf, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (64).to_natural_32), ce_string ("%%d"), requested_blksize]).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_option_add (l_sbytes, create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + l_sbytes.to_integer_32, ce_string ("blksize"))
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := l_sbytes + l_tmp___3
						l_tmp___4 := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_option_add (l_sbytes, create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + l_sbytes.to_integer_32, l_buf)
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := l_sbytes + l_tmp___4;
						(create {P_MPRINTF_STATIC}).curl_msnprintf ([l_buf, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (64).to_natural_32), ce_string ("%%d"), retry_time]).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___5 := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_option_add (l_sbytes, create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + l_sbytes.to_integer_32, ce_string ("timeout"))
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := l_sbytes + l_tmp___5
						l_tmp___6 := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_option_add (l_sbytes, create {CE_INTEGER_8_POINTER}.make_cast (spacket.data) + l_sbytes.to_integer_32, l_buf)
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := l_sbytes + l_tmp___6
						l_senddata := sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), l_sbytes, 0, (conn.item.ip_addr).item.ai_addr, (conn.item.ip_addr).item.ai_addrlen)
						update_class_fields -- Current might have been changed; synchronize
						if l_senddata /= l_sbytes.to_integer_32 then
							l_tmp___7 := errno_location
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___8 := conn.item.Curl_strerror (l_tmp___7.item)
							update_class_fields -- Current might have been changed; synchronize
							l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___8])
							update_class_fields -- Current might have been changed; synchronize
						end
						(create {P_URL_STATIC}).Curl_safefree (create {CE_VOID_POINTER}.make_cast (l_filename))
						update_class_fields -- Current might have been changed; synchronize
					end
					l_pos := -1 + 7 -- switch 1; break
				when 2 then -- switch 1, case 6
					if l_data.item.set.upload then
						l_res := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_connect_for_tx (a_event)
						update_class_fields -- Current might have been changed; synchronize
					else
						l_res := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_connect_for_rx (a_event)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_pos := -1 + 7 -- switch 1; break
				when 3 then -- switch 1, case 4
					l_res := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_connect_for_tx (a_event)
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 7 -- switch 1; break
				when 4 then -- switch 1, case 3
					l_res := ((create {CE_CLASS_POINTER [PD_TFTP_STATE_DATA]}.make_shared (c_pointer))).item.tftp_connect_for_rx (a_event)
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 7 -- switch 1; break
				when 5 then -- switch 1, case 5
					Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 7 -- switch 1; break
				when 6 then -- switch 1, default:;
					(conn.item.data).item.Curl_failf ([ce_string ("tftp_send_first: internal error")])
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 7 -- switch 1; break
				when 7 then -- switch 1, done:
					if not l_return then
						Result := l_res
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	tftp_rx (a_event: INTEGER_32): NATURAL_32
		local
			l_sbytes: INTEGER_32
			l_rblock: INTEGER_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_tmp: NATURAL_16
			l_tmp___0: CE_POINTER [INTEGER_32]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___2: CE_POINTER [INTEGER_32]
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_tmp___4: CE_POINTER [INTEGER_32]
			l_tmp___5: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data := conn.item.data
					inspect a_event -- switch 1 head
					when 3 then l_pos := -1 + 1
					when 6 then l_pos := -1 + 3
					when 7 then l_pos := -1 + 4
					when 5 then l_pos := -1 + 5
					else l_pos := -1 + 6
					end
				when 1 then -- switch 1, case 3
					l_tmp := (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("rpacket"))).item.getrpacketblock
					update_class_fields -- Current might have been changed; synchronize
					l_rblock := l_tmp.to_integer_32
					if (((block.to_integer_32 + 1)).bit_and (65_535)) /= l_rblock then
						(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("Received unexpected DATA packet block %%d%N"), l_rblock])
						update_class_fields -- Current might have been changed; synchronize
						Current.retries := retries + 1
						update_class_fields -- Current might have been changed; synchronize
						if retries > retry_max then
							l_data.item.Curl_failf ([ce_string ("tftp_rx: giving up waiting for block %%d"), ((block.to_integer_32 + 1)).bit_and (65_535)])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
							l_return := True
						end
						l_pos := -1 + 7 -- switch 1; break
					end
				when 2 then
					if not l_return then
						Current.block := l_rblock.to_natural_16
						update_class_fields -- Current might have been changed; synchronize
						Current.retries := 0
						update_class_fields -- Current might have been changed; synchronize
						setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (4).to_natural_16)
						update_class_fields -- Current might have been changed; synchronize
						setpacketblock (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), block)
						update_class_fields -- Current might have been changed; synchronize
						l_sbytes := sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen)
						update_class_fields -- Current might have been changed; synchronize
						if l_sbytes < 0 then
							l_tmp___0 := errno_location
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___1 := conn.item.Curl_strerror (l_tmp___0.item)
							update_class_fields -- Current might have been changed; synchronize
							l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___1])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_SEND_ERROR)
							l_return := True
						end
					end
					if not l_return then
						if rbytes < blksize + 4 then
							Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
							update_class_fields -- Current might have been changed; synchronize
						else
							Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_RX
							update_class_fields -- Current might have been changed; synchronize
						end
						time (create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("rx_time"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_pos := -1 + 7 -- switch 1; break
				when 3 then -- switch 1, case 6
					Current.block := (0).to_natural_16
					update_class_fields -- Current might have been changed; synchronize
					Current.retries := 0
					update_class_fields -- Current might have been changed; synchronize
					setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (4).to_natural_16)
					update_class_fields -- Current might have been changed; synchronize
					setpacketblock (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), block)
					update_class_fields -- Current might have been changed; synchronize
					l_sbytes := sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen)
					update_class_fields -- Current might have been changed; synchronize
					if l_sbytes < 0 then
						l_tmp___2 := errno_location
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := conn.item.Curl_strerror (l_tmp___2.item)
						update_class_fields -- Current might have been changed; synchronize
						l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___3])
						update_class_fields -- Current might have been changed; synchronize
						Result := ({PC_CURLCODE}.CURLE_SEND_ERROR)
						l_return := True
					end
					if not l_return then
						Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_RX
						update_class_fields -- Current might have been changed; synchronize
						time (create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("rx_time"))).do_nothing
						update_class_fields -- Current might have been changed; synchronize
					end
					l_pos := -1 + 7 -- switch 1; break
				when 4 then -- switch 1, case 7
					Current.retries := retries + 1
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("Timeout waiting for block %%d ACK.  Retries = %%d%N"), ((block.to_integer_32 + 1)).bit_and (65_535), retries])
					update_class_fields -- Current might have been changed; synchronize
					if retries > retry_max then
						Current.error := -99
						update_class_fields -- Current might have been changed; synchronize
						Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
						update_class_fields -- Current might have been changed; synchronize
					else
						l_sbytes := sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen)
						update_class_fields -- Current might have been changed; synchronize
						if l_sbytes < 0 then
							l_tmp___4 := errno_location
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___5 := conn.item.Curl_strerror (l_tmp___4.item)
							update_class_fields -- Current might have been changed; synchronize
							l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___5])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_SEND_ERROR)
							l_return := True
						end
					end
					l_pos := -1 + 7 -- switch 1; break
				when 5 then -- switch 1, case 5
					setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (5).to_natural_16)
					update_class_fields -- Current might have been changed; synchronize
					setpacketblock (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), block)
					update_class_fields -- Current might have been changed; synchronize
					sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 7 -- switch 1; break
				when 6 then -- switch 1, default:
					l_data.item.Curl_failf ([ce_string ("%%s"), ce_string ("tftp_rx: internal error")])
					update_class_fields -- Current might have been changed; synchronize
					Result := ({PC_CURLCODE}.CURLE_TFTP_ILLEGAL)
					l_return := True
				when 7 then -- switch 1, done:
					if not l_return then
						Result := ({PC_CURLCODE}.CURLE_OK)
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	tftp_tx (a_event: INTEGER_32): NATURAL_32
		local
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_sbytes: INTEGER_32
			l_rblock: INTEGER_32
			l_res: NATURAL_32
			l_k: CE_POINTER [PD_SINGLEREQUEST]
			l_tmp: NATURAL_16
			l_tmp___0: CE_POINTER [INTEGER_32]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___2: CE_POINTER [INTEGER_32]
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_tmp___4: CE_POINTER [INTEGER_32]
			l_tmp___5: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_data := conn.item.data
					l_res := {PC_CURLCODE}.CURLE_OK
					l_k := create {CE_CLASS_POINTER [PD_SINGLEREQUEST]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("req"))
					inspect a_event -- switch 1 head
					when 4 then l_pos := -1 + 1
					when 6 then l_pos := -1 + 1
					when 7 then l_pos := -1 + 11
					when 5 then l_pos := -1 + 12
					else l_pos := -1 + 13
					end
				when 1 then -- switch 1, case 4 | switch 1, case 6
					if a_event /= 4 then
						l_pos := -1 + 9 -- skip if block
					end
				when 2 then -- if:
						l_tmp := (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("rpacket"))).item.getrpacketblock
						update_class_fields -- Current might have been changed; synchronize
						l_rblock := l_tmp.to_integer_32
						if l_rblock = block.to_integer_32 then
							l_pos := -1 + 7 -- skip if block
						end
				when 3 then -- if:
						if block.to_integer_32 /= 0 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_rblock /= 65_535 then
							l_pos := -1 + 6 -- goto _L
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L:;
						(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("Received ACK for block %%d, expecting %%d%N"), l_rblock, block.to_integer_32])
						update_class_fields -- Current might have been changed; synchronize
						Current.retries := retries + 1
						update_class_fields -- Current might have been changed; synchronize
						if retries > retry_max then
							l_data.item.Curl_failf ([ce_string ("tftp_tx: giving up waiting for block %%d ack"), block.to_integer_32])
							update_class_fields -- Current might have been changed; synchronize
							l_res := {PC_CURLCODE}.CURLE_SEND_ERROR
						else
							l_sbytes := sendto (sockfd, create {CE_VOID_POINTER}.make_cast ((create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")))), (4 + sbytes).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen)
							update_class_fields -- Current might have been changed; synchronize
							if l_sbytes < 0 then
								l_tmp___0 := errno_location
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___1 := conn.item.Curl_strerror (l_tmp___0.item)
								update_class_fields -- Current might have been changed; synchronize
								l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___1])
								update_class_fields -- Current might have been changed; synchronize
								l_res := {PC_CURLCODE}.CURLE_SEND_ERROR
							end
						end
						Result := l_res
						l_return := True
				when 7 then -- if done
						if not l_return then
							time (create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("rx_time"))).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							Current.block := (block.to_integer_32 + 1).to_natural_16
							update_class_fields -- Current might have been changed; synchronize
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						Current.block := (1).to_natural_16
						update_class_fields -- Current might have been changed; synchronize
				when 10 then -- if done
					if not l_return then
						Current.retries := 0
						update_class_fields -- Current might have been changed; synchronize
						setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (3).to_natural_16)
						update_class_fields -- Current might have been changed; synchronize
						setpacketblock (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), block)
						update_class_fields -- Current might have been changed; synchronize
						if block.to_integer_32 > 1 then
							if sbytes < blksize then
								Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
								update_class_fields -- Current might have been changed; synchronize
								Result := ({PC_CURLCODE}.CURLE_OK)
								l_return := True
							end
						end
					end
					if not l_return then
						l_res := conn.item.Curl_fillreadbuffer (blksize, create {CE_INTEGER_32_POINTER}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("sbytes")))
						update_class_fields -- Current might have been changed; synchronize
						if l_res.to_boolean then
							Result := l_res
							l_return := True
						end
					end
					if not l_return then
						l_sbytes := sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4 + sbytes).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen)
						update_class_fields -- Current might have been changed; synchronize
						if l_sbytes < 0 then
							l_tmp___2 := errno_location
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___3 := conn.item.Curl_strerror (l_tmp___2.item)
							update_class_fields -- Current might have been changed; synchronize
							l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___3])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_SEND_ERROR)
							l_return := True
						end
					end
					if not l_return then
						l_k.item.writebytecount := l_k.item.writebytecount + sbytes.to_integer_64
						update_class_fields -- Current might have been changed; synchronize
						l_data.item.Curl_pgrsSetUploadCounter (l_k.item.writebytecount)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_pos := -1 + 14 -- switch 1; break
				when 11 then -- switch 1, case 7
					Current.retries := retries + 1
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_SENDF_STATIC}).Curl_infof ([l_data, ce_string ("Timeout waiting for block %%d ACK.  Retries = %%d%N"), ((block.to_integer_32 + 1)).bit_and (65_535), retries])
					update_class_fields -- Current might have been changed; synchronize
					if retries > retry_max then
						Current.error := -99
						update_class_fields -- Current might have been changed; synchronize
						Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
						update_class_fields -- Current might have been changed; synchronize
					else
						l_sbytes := sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4 + sbytes).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen)
						update_class_fields -- Current might have been changed; synchronize
						if l_sbytes < 0 then
							l_tmp___4 := errno_location
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___5 := conn.item.Curl_strerror (l_tmp___4.item)
							update_class_fields -- Current might have been changed; synchronize
							l_data.item.Curl_failf ([ce_string ("%%s"), l_tmp___5])
							update_class_fields -- Current might have been changed; synchronize
							Result := ({PC_CURLCODE}.CURLE_SEND_ERROR)
							l_return := True
						end
						if not l_return then
							l_data.item.Curl_pgrsSetUploadCounter (l_k.item.writebytecount)
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					l_pos := -1 + 14 -- switch 1; break
				when 12 then -- switch 1, case 5
					Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
					update_class_fields -- Current might have been changed; synchronize
					setpacketevent (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), (5).to_natural_16)
					update_class_fields -- Current might have been changed; synchronize
					setpacketblock (create {CE_CLASS_POINTER [PD_TFTP_PACKET]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("spacket")), block)
					update_class_fields -- Current might have been changed; synchronize
					sendto (sockfd, create {CE_VOID_POINTER}.make_cast (spacket.data), (4).to_natural_32, 16_384, create {CE_CLASS_POINTER [SD_SOCKADDR]}.make_cast ((create {CE_CLASS_POINTER [PD_CURL_SOCKADDR_STORAGE]}.make_shared (c_pointer + (create {PD_TFTP_STATE_DATA}.make).offset_of ("remote_addr")))), remote_addrlen).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					Current.state := {PC_TFTP_STATE_T}.TFTP_STATE_FIN
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 14 -- switch 1; break
				when 13 then -- switch 1, default:
					l_data.item.Curl_failf ([ce_string ("tftp_tx: internal error, event: %%i"), a_event])
					update_class_fields -- Current might have been changed; synchronize
					l_pos := -1 + 14 -- switch 1; break
				when 14 then -- switch 1, done:
					if not l_return then
						Result := l_res
					end
				when 15 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {PD_TFTP_STATE_DATA} -- private routines

	setpacketevent (a_packet: CE_POINTER [PD_TFTP_PACKET]; a_num: NATURAL_16)
		require
			not a_packet.is_void
		do
			a_packet.item.data[0] := ((a_num.to_integer_32).bit_shift_right (8)).to_natural_8
			a_packet.item.data[1] := ((a_num.to_integer_32).bit_and (255)).to_natural_8
		end

	setpacketblock (a_packet: CE_POINTER [PD_TFTP_PACKET]; a_num: NATURAL_16)
		require
			not a_packet.is_void
		do
			a_packet.item.data[2] := ((a_num.to_integer_32).bit_shift_right (8)).to_natural_8
			a_packet.item.data[3] := ((a_num.to_integer_32).bit_and (255)).to_natural_8
		end

	tftp_option_get (a_buf: CE_POINTER [INTEGER_8]; a_len: NATURAL_32; a_option, a_value: CE_POINTER [CE_POINTER [INTEGER_8]]): CE_POINTER [INTEGER_8]
		local
			l_loc: NATURAL_32
			l_return: BOOLEAN
		do
			l_loc := Curl_strnlen (a_buf, a_len)
			l_loc := l_loc + 1
			if l_loc >= a_len then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				a_option.item := a_buf
				l_loc := l_loc + (Curl_strnlen (a_buf + l_loc.to_integer_32, a_len - l_loc))
				l_loc := l_loc + 1
				if l_loc > a_len then
					Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				a_value.item := a_buf + ((strlen (a_option.item)) + {NATURAL_32} 1).to_integer_32
				Result := (a_buf + l_loc.to_integer_32)
			end
		end

	Curl_strnlen (a_string: CE_POINTER [INTEGER_8]; a_maxlen: NATURAL_32): NATURAL_32
		local
			l_end: CE_POINTER [INTEGER_8]
			l_tmp___0: NATURAL_32
		do
			l_end := create {CE_INTEGER_8_POINTER}.make_cast ((memchr (create {CE_VOID_POINTER}.make_cast (a_string), ('%/0/').code, a_maxlen)))
			if (not l_end.is_void) then
				l_tmp___0 := (l_end.distance (a_string)).to_natural_32
			else
				l_tmp___0 := a_maxlen
			end
			Result := l_tmp___0
		end

feature {ANY}

	copy (a_other: like Current)
		do
			state := a_other.state
			mode := a_other.mode
			error := a_other.error
			event := a_other.event
			conn := a_other.conn
			sockfd := a_other.sockfd
			retries := a_other.retries
			retry_time := a_other.retry_time
			retry_max := a_other.retry_max
			start_time := a_other.start_time
			max_time := a_other.max_time
			rx_time := a_other.rx_time
			block := a_other.block
			local_addr := a_other.local_addr.twin
			remote_addr := a_other.remote_addr.twin
			remote_addrlen := a_other.remote_addrlen
			rbytes := a_other.rbytes
			sbytes := a_other.sbytes
			blksize := a_other.blksize
			requested_blksize := a_other.requested_blksize
			rpacket := a_other.rpacket.twin
			spacket := a_other.spacket.twin
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				state = a_other.state and
				mode = a_other.mode and
				error = a_other.error and
				event = a_other.event and
				conn.is_equal (a_other.conn) and
				sockfd = a_other.sockfd and
				retries = a_other.retries and
				retry_time = a_other.retry_time and
				retry_max = a_other.retry_max and
				start_time = a_other.start_time and
				max_time = a_other.max_time and
				rx_time = a_other.rx_time and
				block = a_other.block and
				local_addr.is_equal (a_other.local_addr) and
				remote_addr.is_equal (a_other.remote_addr) and
				remote_addrlen = a_other.remote_addrlen and
				rbytes = a_other.rbytes and
				sbytes = a_other.sbytes and
				blksize = a_other.blksize and
				requested_blksize = a_other.requested_blksize and
				rpacket.is_equal (a_other.rpacket) and
				spacket.is_equal (a_other.spacket)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("state"),
					create {CE_STRUCTURE_FIELD}.make ("mode"),
					create {CE_STRUCTURE_FIELD}.make ("error"),
					create {CE_STRUCTURE_FIELD}.make ("event"),
					create {CE_STRUCTURE_FIELD}.make ("conn"),
					create {CE_STRUCTURE_FIELD}.make ("sockfd"),
					create {CE_STRUCTURE_FIELD}.make ("retries"),
					create {CE_STRUCTURE_FIELD}.make ("retry_time"),
					create {CE_STRUCTURE_FIELD}.make ("retry_max"),
					create {CE_STRUCTURE_FIELD}.make ("start_time"),
					create {CE_STRUCTURE_FIELD}.make ("max_time"),
					create {CE_STRUCTURE_FIELD}.make ("rx_time"),
					create {CE_STRUCTURE_FIELD}.make ("block"),
					create {CE_STRUCTURE_FIELD}.make ("local_addr"),
					create {CE_STRUCTURE_FIELD}.make ("remote_addr"),
					create {CE_STRUCTURE_FIELD}.make ("remote_addrlen"),
					create {CE_STRUCTURE_FIELD}.make ("rbytes"),
					create {CE_STRUCTURE_FIELD}.make ("sbytes"),
					create {CE_STRUCTURE_FIELD}.make ("blksize"),
					create {CE_STRUCTURE_FIELD}.make ("requested_blksize"),
					create {CE_STRUCTURE_FIELD}.make ("rpacket"),
					create {CE_STRUCTURE_FIELD}.make ("spacket")
				>>)
			init_structure_info
		end

end
