class
	P_MEMORY_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_STDIO_STATIC

feature {ANY} -- public attributes

	gmp_allocate_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := gmp_default_allocate_address
		end

	gmp_reallocate_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := gmp_default_reallocate_address
		end

	gmp_free_func: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := gmp_default_free_address
		end

feature {ANY} -- public routines

	gmp_default_allocate (a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_ret: CE_POINTER [ANY]
		do
			l_ret := malloc (a_size)
			if l_ret.is_void then
				fprintf ([(create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("GNU MP: Cannot allocate memory (size=%%lu)%N"), a_size.to_integer_32]).do_nothing
				abort
			end
			Result := l_ret
		end

	gmp_default_reallocate (a_oldptr: CE_POINTER [ANY]; a_old_size, a_new_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_ret: CE_POINTER [ANY]
		do
			l_ret := realloc (a_oldptr, a_new_size)
			if l_ret.is_void then
				fprintf ([(create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("GNU MP: Cannot reallocate memory (old_size=%%lu new_size=%%lu)%N"), a_old_size.to_integer_32, a_new_size.to_integer_32]).do_nothing
				abort
			end
			Result := l_ret
		end

	gmp_default_free (a_blk_ptr: CE_POINTER [ANY]; a_blk_size: NATURAL_32)
		do
			free (a_blk_ptr)
		end

feature {ANY} -- routine addresses

	gmp_default_allocate_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent gmp_default_allocate)
		end

	gmp_default_reallocate_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent gmp_default_reallocate)
		end

	gmp_default_free_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent gmp_default_free)
		end

end
