class
	PD_GMP_SNPRINTF_T

inherit
	PD_GMP_SNPRINTF_T_STATIC
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_INTEGER_8_POINTER} buf.make
			Precursor
		end

feature {ANY} -- fields

	buf: CE_POINTER [INTEGER_8] assign set_buf

	size: NATURAL_32 assign set_size

feature {ANY} -- field setters

	set_buf (a_buf: CE_POINTER [INTEGER_8])
		do
			buf := a_buf
			update_memory_field ("buf")
		end

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

feature {ANY} -- public routines

	gmp_snprintf_format (a_fmt: CE_POINTER [INTEGER_8]; a_orig_ap: CE_VA_LIST): INTEGER_32
		local
			l_orig_ap: CE_VA_LIST
			l_ret: INTEGER_32
			l_step___1: INTEGER_32
			l_alloc: INTEGER_32
			l_avail: INTEGER_32
			l_ap: CE_VA_LIST
			l_p: CE_POINTER [INTEGER_8]
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			l_orig_ap := a_orig_ap.twin
			l_avail := size.to_integer_32
			if l_avail > 1 then
				l_ap := l_orig_ap.twin
				l_ret := vsnprintf (buf, l_avail.to_natural_32, a_fmt, l_ap.twin)
				update_class_fields -- Current might have been changed; synchronize
				if l_ret = -1 then
					l_ret := l_avail - 1
				end
				if l_ret < l_avail - 1 then
					l_step___1 := l_ret
				else
					l_step___1 := l_avail - 1
				end
				Current.size := size - l_step___1.to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				Current.buf := buf + l_step___1
				update_class_fields -- Current might have been changed; synchronize
				if l_ret /= l_avail - 1 then
					Result := l_ret
					l_return := True
				end
				if not l_return then
					if 128 > l_ret then
						l_alloc := 128
					else
						l_alloc := l_ret
					end
				end
			else
				l_alloc := 128
			end
			if not l_return then
				from
				until
					(l_break)
				loop
					l_break := False
					l_alloc := l_alloc * (2)
					l_tmp := create {CE_VOID_POINTER}.make_cast ((create {P_MEMORY_STATIC}).gmp_allocate_func.item.pointer_item ([l_alloc.to_natural_32 * {PLATFORM}.Integer_8_bytes.to_natural_32]))
					update_class_fields -- Current might have been changed; synchronize
					l_p := create {CE_INTEGER_8_POINTER}.make_cast (l_tmp)
					l_ap := l_orig_ap.twin
					l_ret := vsnprintf (l_p, l_alloc.to_natural_32, a_fmt, l_ap.twin)
					update_class_fields; -- Current might have been changed; synchronize
					(create {P_MEMORY_STATIC}).gmp_free_func.item.call ([create {CE_VOID_POINTER}.make_cast (l_p), l_alloc.to_natural_32])
					update_class_fields -- Current might have been changed; synchronize
					if l_ret /= l_alloc - 1 then
						if l_ret /= -1 then
							l_break := True
						end
					end
				end
				l_break := False
				Result := l_ret
			end
		end

	gmp_snprintf_memory (a_str: CE_POINTER [INTEGER_8]; a_len: NATURAL_32): INTEGER_32
		local
			l_n: NATURAL_32
		do
			if size > {NATURAL_32} 1 then
				if size - {NATURAL_32} 1 < a_len then
					l_n := size - {NATURAL_32} 1
				else
					l_n := a_len
				end
				memcpy (create {CE_VOID_POINTER}.make_cast (buf), create {CE_VOID_POINTER}.make_cast (a_str), l_n).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Current.buf := buf + l_n.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				Current.size := size - l_n
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := a_len.to_integer_32
		end

	gmp_snprintf_reps (a_c, a_reps: INTEGER_32): INTEGER_32
		local
			l_n: NATURAL_32
		do
			if size > {NATURAL_32} 1 then
				if size - {NATURAL_32} 1 < a_reps.to_natural_32 then
					l_n := size - {NATURAL_32} 1
				else
					l_n := a_reps.to_natural_32
				end
				memset (create {CE_VOID_POINTER}.make_cast (buf), a_c, l_n).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				Current.buf := buf + l_n.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				Current.size := size - l_n
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := a_reps
		end

	gmp_snprintf_final: INTEGER_32
		do
			if size >= {NATURAL_32} 1 then
				buf[0] := (('%/0/').code).to_integer_8
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := 0
		end

feature {ANY}

	copy (a_other: like Current)
		do
			buf := a_other.buf
			size := a_other.size
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				buf.is_equal (a_other.buf) and
				size = a_other.size
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("buf"),
					create {CE_STRUCTURE_FIELD}.make ("size")
				>>)
			init_structure_info
		end

end
