class
	SD_OBSTACK

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal,
			bitfield_helper
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			make_memory -- has bitfields; back with memory for correct semantics
		end

feature {ANY} -- fields

	chunk_size: INTEGER_32 assign set_chunk_size

	chunk: CE_POINTER [SD_OBSTACK_CHUNK] assign set_chunk

	object_base: CE_POINTER [INTEGER_8] assign set_object_base

	next_free: CE_POINTER [INTEGER_8] assign set_next_free

	chunk_limit: CE_POINTER [INTEGER_8] assign set_chunk_limit

	temp: SD_TEMP assign set_temp

	alignment_mask: INTEGER_32 assign set_alignment_mask

	chunkfun: CE_ROUTINE assign set_chunkfun

	freefun: CE_ROUTINE assign set_freefun

	extra_arg: CE_POINTER [ANY] assign set_extra_arg

	use_extra_arg: NATURAL_32 assign set_use_extra_arg

	maybe_empty_object: NATURAL_32 assign set_maybe_empty_object

	alloc_failed: NATURAL_32 assign set_alloc_failed

feature {ANY} -- field setters

	set_chunk_size (a_chunk_size: INTEGER_32)
		do
			chunk_size := a_chunk_size
			update_memory_field ("chunk_size")
		end

	set_chunk (a_chunk: CE_POINTER [SD_OBSTACK_CHUNK])
		do
			chunk := a_chunk
			update_memory_field ("chunk")
		end

	set_object_base (a_object_base: CE_POINTER [INTEGER_8])
		do
			object_base := a_object_base
			update_memory_field ("object_base")
		end

	set_next_free (a_next_free: CE_POINTER [INTEGER_8])
		do
			next_free := a_next_free
			update_memory_field ("next_free")
		end

	set_chunk_limit (a_chunk_limit: CE_POINTER [INTEGER_8])
		do
			chunk_limit := a_chunk_limit
			update_memory_field ("chunk_limit")
		end

	set_temp (a_temp: SD_TEMP)
		do
			temp := a_temp
			update_memory_field ("temp")
		end

	set_alignment_mask (a_alignment_mask: INTEGER_32)
		do
			alignment_mask := a_alignment_mask
			update_memory_field ("alignment_mask")
		end

	set_chunkfun (a_chunkfun: CE_ROUTINE)
		do
			chunkfun := a_chunkfun
			update_memory_field ("chunkfun")
		end

	set_freefun (a_freefun: CE_ROUTINE)
		do
			freefun := a_freefun
			update_memory_field ("freefun")
		end

	set_extra_arg (a_extra_arg: CE_POINTER [ANY])
		do
			extra_arg := a_extra_arg
			update_memory_field ("extra_arg")
		end

	set_use_extra_arg (a_use_extra_arg: NATURAL_32)
		do
			use_extra_arg := a_use_extra_arg
			update_memory_field ("use_extra_arg")
		end

	set_maybe_empty_object (a_maybe_empty_object: NATURAL_32)
		do
			maybe_empty_object := a_maybe_empty_object
			update_memory_field ("maybe_empty_object")
		end

	set_alloc_failed (a_alloc_failed: NATURAL_32)
		do
			alloc_failed := a_alloc_failed
			update_memory_field ("alloc_failed")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			chunk_size := a_other.chunk_size
			chunk := a_other.chunk
			object_base := a_other.object_base
			next_free := a_other.next_free
			chunk_limit := a_other.chunk_limit
			temp := a_other.temp.twin
			alignment_mask := a_other.alignment_mask
			chunkfun := a_other.chunkfun
			freefun := a_other.freefun
			extra_arg := a_other.extra_arg
			use_extra_arg := a_other.use_extra_arg
			maybe_empty_object := a_other.maybe_empty_object
			alloc_failed := a_other.alloc_failed
			create_memory_copy -- has bitfields; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				chunk_size = a_other.chunk_size and
				chunk.is_equal (a_other.chunk) and
				object_base.is_equal (a_other.object_base) and
				next_free.is_equal (a_other.next_free) and
				chunk_limit.is_equal (a_other.chunk_limit) and
				temp.is_equal (a_other.temp) and
				alignment_mask = a_other.alignment_mask and
				chunkfun.is_equal (a_other.chunkfun) and
				freefun.is_equal (a_other.freefun) and
				extra_arg.is_equal (a_other.extra_arg) and
				use_extra_arg = a_other.use_extra_arg and
				maybe_empty_object = a_other.maybe_empty_object and
				alloc_failed = a_other.alloc_failed
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("chunk_size"),
					create {CE_STRUCTURE_FIELD}.make ("chunk"),
					create {CE_STRUCTURE_FIELD}.make ("object_base"),
					create {CE_STRUCTURE_FIELD}.make ("next_free"),
					create {CE_STRUCTURE_FIELD}.make ("chunk_limit"),
					create {CE_STRUCTURE_FIELD}.make ("temp"),
					create {CE_STRUCTURE_FIELD}.make ("alignment_mask"),
					create {CE_STRUCTURE_FIELD}.make ("chunkfun"),
					create {CE_STRUCTURE_FIELD}.make ("freefun"),
					create {CE_STRUCTURE_FIELD}.make ("extra_arg"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("use_extra_arg"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("maybe_empty_object"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("alloc_failed")
				>>)
			init_structure_info
		end

	bitfield_helper (a_operation: INTEGER_32; a_field_name, a_struct_pointer: POINTER; a_value: NATURAL_64): NATURAL_64
		external
			"C inline use <obstack.h>"
		alias
			"[
				char *field_name = (char *)$a_field_name;
				if($a_operation == 1) {
					if(field_name == (char *)0) return sizeof(struct obstack);
				} else {
					struct obstack *p = (struct obstack *)$a_struct_pointer;
					if($a_operation == 2) {
						     if(strcmp(field_name, "use_extra_arg") == 0) p->use_extra_arg = (unsigned int)$a_value;
						else if(strcmp(field_name, "maybe_empty_object") == 0) p->maybe_empty_object = (unsigned int)$a_value;
						else if(strcmp(field_name, "alloc_failed") == 0) p->alloc_failed = (unsigned int)$a_value;
					} else if($a_operation == 3) {
						if(strcmp(field_name, "use_extra_arg") == 0) return p->use_extra_arg;
						if(strcmp(field_name, "maybe_empty_object") == 0) return p->maybe_empty_object;
						if(strcmp(field_name, "alloc_failed") == 0) return p->alloc_failed;
					}
				}
			]"
		end

end
