class
	P_BESSEL_J0_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public attributes

	bj0_data: CE_STATIC [CE_ARRAY [REAL_64]]
		once ("PROCESS")
			Result.item := create {CE_REAL_64_ARRAY}.make_initialized (<<
					0.100254161968939137, -0.665223007764405132, 0.248983703498281314, -0.0332527231700357697, 0.0023114179304694015,
					-0.0000991127741995080, 0.0000028916708643998, -0.0000000612108586630, 0.0000000009838650793, -0.0000000000124235515,
					0.0000000000001265433, -0.0000000000000010619, 0.0000000000000000074
				>>)
		end

	bj0_cs: CE_STATIC [CE_ARRAY [PD_CHEB_SERIES_STRUCT]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_CHEB_SERIES_STRUCT]}.make_initialized (<<
					create {PD_CHEB_SERIES_STRUCT}.make_initialized ([bj0_data.item, 12, (-1).to_double, (1).to_double, 9])
				>>)
		end

feature {ANY} -- public routines

	gsl_sf_bessel_J0_e2 (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		require
			not a_result.is_void
		local
			l_y: REAL_64
			l_z: REAL_64
			l_ca: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_ct: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_cp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_ca: INTEGER_32
			l_stat_ct: INTEGER_32
			l_stat_cp: INTEGER_32
			l_sqrty: REAL_64
			l_ampl: REAL_64
			l_tmp___5: REAL_64
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_tmp___10: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_ca.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_ct.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_cp.make (<<1>>)

			l_y := (fabs (a_x))
			if l_y < 2.0 * 1.4901161193847656e-08 then
				a_result.item.val := 1.0
				a_result.item.err := l_y * l_y
				Result := 0
			elseif l_y <= 4.0 then
				Result := (bj0_cs.item.item.cheb_eval_e ((0.125 * l_y) * l_y - 1.0, a_result))
			else
				l_z := 32.0 / (l_y * l_y) - 1.0
				l_stat_ca := (((create {P_BESSEL_AMP_PHASE_STATIC}).gsl_sf_bessel_amp_phase_bm0_cs.item).item.cheb_eval_e (l_z, l_ca))
				l_stat_ct := (((create {P_BESSEL_AMP_PHASE_STATIC}).gsl_sf_bessel_amp_phase_bth0_cs.item).item.cheb_eval_e (l_z, l_ct))
				l_stat_cp := (l_cp.item.gsl_sf_bessel_cos_pi4_e (l_y, l_ct.item.val / l_y))
				l_sqrty := (sqrt (l_y))
				l_ampl := (0.75 + l_ca.item.val) / l_sqrty
				a_result.item.val := l_ampl * l_cp.item.val
				l_tmp___5 := fabs (l_cp.item.val)
				a_result.item.err := (l_tmp___5 * l_ca.item.err) / l_sqrty + (fabs (l_ampl)) * l_cp.item.err
				a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
				if l_stat_ca /= 0 then
					l_tmp___10 := l_stat_ca
				else
					if l_stat_ct /= 0 then
						l_tmp___9 := l_stat_ct
					else
						if l_stat_cp /= 0 then
							l_tmp___8 := l_stat_cp
						else
							l_tmp___8 := 0
						end
						l_tmp___9 := l_tmp___8
					end
					l_tmp___10 := l_tmp___9
				end
				Result := l_tmp___10
			end
		end

	gsl_sf_bessel_J02 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_J0_e2 (a_x, l_result))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_bessel_J0_e(x, &result)"), ce_string ("bessel_J0.c"), 110, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

end
