class
	P_BESSEL_K_STATIC

inherit
	CE_UTILITIES

feature {ANY} -- public routines

	gsl_sf_bessel_kl_scaled_array (a_lmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat: INTEGER_32
			l_ell: INTEGER_32
			l_kellp1: REAL_64
			l_kell: REAL_64
			l_kellm1: REAL_64
			l_r_kell: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_r_kellm1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_r_kell.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_r_kellm1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_lmax < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_x <= 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("bessel_k.c"), 196, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_lmax = 0 then
							l_stat := (l_result.item.gsl_sf_bessel_k0_scaled_e (a_x))
							a_result_array[0] := l_result.item.val
							Result := l_stat
							l_return := True
						else
							l_r_kell.item.gsl_sf_bessel_k1_scaled_e (a_x).do_nothing
							l_r_kellm1.item.gsl_sf_bessel_k0_scaled_e (a_x).do_nothing
							l_kell := l_r_kell.item.val
							l_kellm1 := l_r_kellm1.item.val
							a_result_array[0] := l_kellm1
							a_result_array[1] := l_kell
							l_ell := 1
							from
							until
								not (l_ell < a_lmax)
							loop
								l_kellp1 := ((2 * l_ell + 1).to_double / a_x) * l_kell + l_kellm1
								a_result_array[(l_ell + 1)] := l_kellp1
								l_kellm1 := l_kell
								l_kell := l_kellp1
								l_ell := l_ell + 1
							end
							Result := 0
							l_return := True
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_bessel_k0_scaled (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_bessel_k0_scaled_e (a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_bessel_k0_scaled_e(x, &result)"), ce_string ("bessel_k.c"), 230, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_k1_scaled (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_bessel_k1_scaled_e (a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_bessel_k1_scaled_e(x, &result)"), ce_string ("bessel_k.c"), 235, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_k2_scaled (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_bessel_k2_scaled_e (a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_bessel_k2_scaled_e(x, &result)"), ce_string ("bessel_k.c"), 240, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_kl_scaled (a_l: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_bessel_kl_scaled_e (a_l, a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_bessel_kl_scaled_e(l, x, &result)"), ce_string ("bessel_k.c"), 245, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

end
