class
	P_CONVERGENCE_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	gsl_min_test_interval (a_x_lower, a_x_upper, a_epsabs, a_epsrel: REAL_64): INTEGER_32
		local
			l_lower: REAL_64
			l_upper: REAL_64
			l_abs_lower: REAL_64
			l_abs_upper: REAL_64
			l_min_abs: REAL_64
			l_tolerance: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_lower := a_x_lower
					l_upper := a_x_upper
					l_abs_lower := (fabs (l_lower))
					l_abs_upper := (fabs (l_upper))
					if a_epsrel < 0.0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("relative tolerance is negative"), ce_string ("convergence.c"), 37, 13)
							Result := 13
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return and then (a_epsabs < 0.0) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 40, 13)
							Result := 13
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return and then (l_lower > l_upper) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("lower bound larger than upper_bound"), ce_string ("convergence.c"), 43, 4)
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if not (l_lower > (0).to_double) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_upper > (0).to_double then
							l_min_abs := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MIN_DBL (l_abs_lower, l_abs_upper)
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if l_lower < (0).to_double then
							if l_upper < (0).to_double then
								l_min_abs := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MIN_DBL (l_abs_lower, l_abs_upper)
							else
								l_min_abs := (0).to_double
							end
						else
							l_min_abs := (0).to_double
						end
				when 5 then
						l_tolerance := a_epsabs + a_epsrel * l_min_abs
						if (fabs (l_upper - l_lower)) < l_tolerance then
							Result := 0
							l_return := True
						end
				when 6 then -- if done
					if not l_return then
						Result := (-2)
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_multifit_test_delta (a_dx, a_x: CE_POINTER [PD_GSL_VECTOR]; a_epsabs, a_epsrel: REAL_64): INTEGER_32
		require
			not a_x.is_void
		local
			l_i: NATURAL_32
			l_ok: INTEGER_32
			l_n: NATURAL_32
			l_xi: REAL_64
			l_dxi: REAL_64
			l_tolerance: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_ok := 1
			l_n := a_x.item.size
			if a_epsrel < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("relative tolerance is negative"), ce_string ("convergence.c"), 35, 13)
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_n))
				loop
					l_break := False
					l_xi := (a_x.item.gsl_vector_get (l_i))
					l_dxi := (a_dx.item.gsl_vector_get (l_i))
					l_tolerance := a_epsabs + a_epsrel * (fabs (l_xi))
					if (fabs (l_dxi)) < l_tolerance then
						l_ok := 1
					else
						l_ok := 0
						l_break := True
					end
					if not (l_return or l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
				if l_ok.to_boolean then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

	gsl_multimin_test_size (a_size, a_epsabs: REAL_64): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if a_epsabs < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 49, 13)
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_size < a_epsabs) then
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := (-2)
			end
		end

	gsl_multiroot_test_delta (a_dx, a_x: CE_POINTER [PD_GSL_VECTOR]; a_epsabs, a_epsrel: REAL_64): INTEGER_32
		require
			not a_x.is_void
		local
			l_i: NATURAL_32
			l_ok: INTEGER_32
			l_n: NATURAL_32
			l_xi: REAL_64
			l_dxi: REAL_64
			l_tolerance: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_ok := 1
			l_n := a_x.item.size
			if a_epsrel < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("relative tolerance is negative"), ce_string ("convergence.c"), 35, 13)
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_n))
				loop
					l_break := False
					l_xi := (a_x.item.gsl_vector_get (l_i))
					l_dxi := (a_dx.item.gsl_vector_get (l_i))
					l_tolerance := a_epsabs + a_epsrel * (fabs (l_xi))
					if (fabs (l_dxi)) < l_tolerance then
						l_ok := 1
					elseif l_dxi = (0).to_double then
						l_ok := 1
					else
						l_ok := 0
						l_break := True
					end
					if not (l_return or l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
				if l_ok.to_boolean then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

	gsl_root_test_interval (a_x_lower, a_x_upper, a_epsabs, a_epsrel: REAL_64): INTEGER_32
		local
			l_abs_lower: REAL_64
			l_abs_upper: REAL_64
			l_min_abs: REAL_64
			l_tolerance: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_abs_lower := (fabs (a_x_lower))
					l_abs_upper := (fabs (a_x_upper))
					if a_epsrel < 0.0 then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("relative tolerance is negative"), ce_string ("convergence.c"), 34, 13)
							Result := 13
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return and then (a_epsabs < 0.0) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 37, 13)
							Result := 13
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return and then (a_x_lower > a_x_upper) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("lower bound larger than upper bound"), ce_string ("convergence.c"), 40, 4)
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						if not (a_x_lower > 0.0) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_x_upper > 0.0 then
							l_min_abs := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MIN_DBL (l_abs_lower, l_abs_upper)
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if a_x_lower < 0.0 then
							if a_x_upper < 0.0 then
								l_min_abs := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MIN_DBL (l_abs_lower, l_abs_upper)
							else
								l_min_abs := (0).to_double
							end
						else
							l_min_abs := (0).to_double
						end
				when 5 then
						l_tolerance := a_epsabs + a_epsrel * l_min_abs
						if (fabs (a_x_upper - a_x_lower)) < l_tolerance then
							Result := 0
							l_return := True
						end
				when 6 then -- if done
					if not l_return then
						Result := (-2)
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_root_test_delta (a_x1___0, a_x0, a_epsabs, a_epsrel: REAL_64): INTEGER_32
		local
			l_tolerance: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_tolerance := a_epsabs + a_epsrel * (fabs (a_x1___0))
			if a_epsrel < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("relative tolerance is negative"), ce_string ("convergence.c"), 65, 13)
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_epsabs < 0.0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 68, 13)
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if (fabs (a_x1___0 - a_x0)) < l_tolerance then
					Result := 0
					l_return := True
				elseif a_x1___0 = a_x0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

	gsl_root_test_residual (a_f, a_epsabs: REAL_64): INTEGER_32
		local
			l_return, l_break: BOOLEAN
		do
			if a_epsabs < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("absolute tolerance is negative"), ce_string ("convergence.c"), 80, 13)
					Result := 13
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if (fabs (a_f)) < a_epsabs then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := (-2)
			end
		end

end
