class
	P_CSCAL_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_MATH_STATIC
	S_STRING_STATIC

feature {ANY} -- public attributes

	sc_control_type: CE_STATIC [CE_ARRAY [PD_GSL_ODEIV_CONTROL_TYPE]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_GSL_ODEIV_CONTROL_TYPE]}.make_initialized (<<
					create {PD_GSL_ODEIV_CONTROL_TYPE}.make_initialized ([ce_string ("scaled"), sc_control_alloc_address, sc_control_init_address, sc_control_hadjust_address, sc_control_free_address])
				>>)
		end

	gsl_odeiv_control_scaled: CE_STATIC [CE_POINTER [PD_GSL_ODEIV_CONTROL_TYPE]]
		once ("PROCESS")
			Result.item := sc_control_type.item
		end

feature {ANY} -- public routines

	sc_control_alloc: CE_POINTER [ANY]
		local
			l_s: CE_POINTER [PD_SC_CONTROL_STATE_T]
			l_return, l_break: BOOLEAN
		do
			l_s := create {CE_CLASS_POINTER [PD_SC_CONTROL_STATE_T]}.make_cast ((malloc ((create {PD_SC_CONTROL_STATE_T}.make).structure_size.to_natural_32)))
			if l_s.is_void then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for sc_control_state"), ce_string ("cscal.c"), 47, 8)
					Result := (create {CE_VOID_POINTER}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (create {CE_VOID_POINTER}.make_cast (l_s))
			end
		end

	sc_control_init (a_vstate: CE_POINTER [ANY]; a_eps_abs, a_eps_rel, a_a_y, a_a_dydt: REAL_64): INTEGER_32
		local
			l_s: CE_POINTER [PD_SC_CONTROL_STATE_T]
			l_return, l_break: BOOLEAN
		do
			l_s := create {CE_CLASS_POINTER [PD_SC_CONTROL_STATE_T]}.make_cast (a_vstate)
			if a_eps_abs < (0).to_double then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eps_abs is negative"), ce_string ("cscal.c"), 61, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_eps_rel < (0).to_double then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("eps_rel is negative"), ce_string ("cscal.c"), 65, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_a_y < (0).to_double then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("a_y is negative"), ce_string ("cscal.c"), 69, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_a_dydt < (0).to_double then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("a_dydt is negative"), ce_string ("cscal.c"), 73, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.item.eps_rel := a_eps_rel
				l_s.item.eps_abs := a_eps_abs
				l_s.item.a_y := a_a_y
				l_s.item.a_dydt := a_a_dydt
				Result := 0
			end
		end

	sc_control_hadjust (a_vstate: CE_POINTER [ANY]; a_dim, a_ord: NATURAL_32; a_y, a_yerr, a_yp, a_h: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_h.is_void
		local
			l_state: CE_POINTER [PD_SC_CONTROL_STATE_T]
			l_eps_abs: REAL_64
			l_eps_rel: REAL_64
			l_a_y: REAL_64
			l_a_dydt: REAL_64
			l_scale_abs: CE_POINTER [REAL_64]
			l_S: REAL_64
			l_h_old: REAL_64
			l_rmax: REAL_64
			l_i: NATURAL_32
			l_D0: REAL_64
			l_tmp: REAL_64
			l_r___3: REAL_64
			l_tmp___1: REAL_64
			l_r___4: REAL_64
			l_r___5: REAL_64
		do
			l_state := create {CE_CLASS_POINTER [PD_SC_CONTROL_STATE_T]}.make_cast (a_vstate)
			l_eps_abs := l_state.item.eps_abs
			l_eps_rel := l_state.item.eps_rel
			l_a_y := l_state.item.a_y
			l_a_dydt := l_state.item.a_dydt
			l_scale_abs := l_state.item.scale_abs
			l_S := 0.9
			l_h_old := a_h.item
			l_rmax := (real_96_from_real_64 (2.22507385850720138309e-308)).truncated_to_real_64
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_dim)
			loop
				l_tmp := fabs (a_y[l_i.to_integer_32])
				l_D0 := l_eps_rel * (l_a_y * l_tmp + l_a_dydt * (fabs (l_h_old * a_yp[l_i.to_integer_32]))) + l_eps_abs * l_scale_abs[l_i.to_integer_32]
				l_tmp___1 := fabs (a_yerr[l_i.to_integer_32])
				l_r___3 := l_tmp___1 / (fabs (l_D0))
				l_rmax := (create {P_GSL_GSL_MINMAX_STATIC}).GSL_MAX_DBL (l_r___3, l_rmax)
				l_i := l_i + 1
			end
			if l_rmax > 1.1 then
				l_r___4 := l_S / (pow (l_rmax, 1.0 / a_ord.to_real_64))
				if l_r___4 < 0.2 then
					l_r___4 := 0.2
				end
				a_h.item := l_r___4 * l_h_old
				Result := (-1)
			elseif l_rmax < 0.5 then
				l_r___5 := l_S / (pow (l_rmax, 1.0 / (a_ord.to_real_64 + 1.0)))
				if l_r___5 > 5.0 then
					l_r___5 := 5.0
				end
				if l_r___5 < 1.0 then
					l_r___5 := 1.0
				end
				a_h.item := l_r___5 * l_h_old
				Result := 1
			else
				Result := 0
			end
		end

	sc_control_free (a_vstate: CE_POINTER [ANY])
		local
			l_state: CE_POINTER [PD_SC_CONTROL_STATE_T]
		do
			l_state := create {CE_CLASS_POINTER [PD_SC_CONTROL_STATE_T]}.make_cast (a_vstate)
			free (create {CE_VOID_POINTER}.make_cast (l_state.item.scale_abs))
			free (create {CE_VOID_POINTER}.make_cast (l_state))
		end

	gsl_odeiv_control_scaled_new (a_eps_abs, a_eps_rel, a_a_y, a_a_dydt: REAL_64; a_scale_abs: CE_POINTER [REAL_64]; a_dim: NATURAL_32): CE_POINTER [PD_GSL_ODEIV_CONTROL]
		local
			l_c___1: CE_POINTER [PD_GSL_ODEIV_CONTROL]
			l_status: INTEGER_32
			l_s: CE_POINTER [PD_SC_CONTROL_STATE_T]
			l_return, l_break: BOOLEAN
		do
			l_c___1 := ((create {P_CONTROL_STATIC}).gsl_odeiv_control_alloc (gsl_odeiv_control_scaled.item))
			l_status := (l_c___1.item.gsl_odeiv_control_init (a_eps_abs, a_eps_rel, a_a_y, a_a_dydt))
			if l_status /= 0 then
				(create {P_CONTROL_STATIC}).gsl_odeiv_control_free (l_c___1)
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("error trying to initialize control"), ce_string ("cscal.c"), 174, l_status)
					Result := (create {CE_CLASS_POINTER [PD_GSL_ODEIV_CONTROL]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s := create {CE_CLASS_POINTER [PD_SC_CONTROL_STATE_T]}.make_cast (l_c___1.item.state)
				l_s.item.scale_abs := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_dim * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.scale_abs).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for scale_abs"), ce_string ("cscal.c"), 186, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_ODEIV_CONTROL]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				memcpy (create {CE_VOID_POINTER}.make_cast (l_s.item.scale_abs), create {CE_VOID_POINTER}.make_cast (a_scale_abs), a_dim * {PLATFORM}.Real_64_bytes.to_natural_32).do_nothing
				Result := l_c___1
			end
		end

feature {ANY} -- routine addresses

	sc_control_alloc_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sc_control_alloc)
		end

	sc_control_init_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sc_control_init)
		end

	sc_control_hadjust_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sc_control_hadjust)
		end

	sc_control_free_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sc_control_free)
		end

end
