class
	P_LEGENDRE_POLY_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	legendre_Pmm (a_m___4: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_p_mm: REAL_64
			l_root_factor: REAL_64
			l_tmp: REAL_64
			l_fact_coeff: REAL_64
			l_i: INTEGER_32
		do
			if a_m___4 = 0 then
				Result := (1.0)
			else
				l_p_mm := 1.0
				l_tmp := sqrt (1.0 - a_x)
				l_root_factor := l_tmp * (sqrt (1.0 + a_x))
				l_fact_coeff := 1.0
				l_i := 1
				from
				until
					not (l_i <= a_m___4)
				loop
					l_p_mm := l_p_mm * (-l_fact_coeff * l_root_factor)
					l_fact_coeff := l_fact_coeff + (2.0)
					l_i := l_i + 1
				end
				Result := l_p_mm
			end
		end

	gsl_sf_legendre_Pl_array (a_lmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_p_ellm2: REAL_64
			l_p_ellm1: REAL_64
			l_p_ell: REAL_64
			l_ell: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_lmax < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if not (a_x < -1.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if not (a_x > 1.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("legendre_poly.c"), 210, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if a_lmax = 0 then
							a_result_array[0] := 1.0
							Result := 0
							l_return := True
						elseif a_lmax = 1 then
							a_result_array[0] := 1.0
							a_result_array[1] := a_x
							Result := 0
							l_return := True
						else
							l_p_ellm2 := 1.0
							l_p_ellm1 := a_x
							l_p_ell := l_p_ellm1
							a_result_array[0] := 1.0
							a_result_array[1] := a_x
							l_ell := 2
							from
							until
								not (l_ell <= a_lmax)
							loop
								l_p_ell := ((a_x * (2 * l_ell - 1).to_double) * l_p_ellm1 - (l_ell - 1).to_double * l_p_ellm2) / l_ell.to_double
								l_p_ellm2 := l_p_ellm1
								l_p_ellm1 := l_p_ell
								a_result_array[l_ell] := l_p_ell
								l_ell := l_ell + 1
							end
							Result := 0
							l_return := True
						end
				when 10 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 11 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_legendre_Pl_deriv_array (a_lmax: INTEGER_32; a_x: REAL_64; a_result_array, a_result_deriv_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_stat_array: INTEGER_32
			l_ell: INTEGER_32
			l_pre: REAL_64
			l_sgn: REAL_64
			l_tmp___0: REAL_64
			l_pre___0: REAL_64
			l_diff_a: REAL_64
			l_diff_b: REAL_64
		do
			l_stat_array := (gsl_sf_legendre_Pl_array (a_lmax, a_x, a_result_array))
			if a_lmax >= 0 then
				a_result_deriv_array[0] := 0.0
			end
			if a_lmax >= 1 then
				a_result_deriv_array[1] := 1.0
			end
			if l_stat_array = 0 then
				if ((fabs (a_x - 1.0)) * (a_lmax.to_double + 1.0)) * (a_lmax.to_double + 1.0) < 1.4901161193847656e-08 then
					l_ell := 2
					from
					until
						not (l_ell <= a_lmax)
					loop
						l_pre := (0.5 * l_ell.to_double) * (l_ell.to_double + 1.0)
						a_result_deriv_array[l_ell] := l_pre * (1.0 - ((0.25 * (1.0 - a_x)) * (l_ell.to_double + 2.0)) * (l_ell.to_double - 1.0))
						l_ell := l_ell + 1
					end
				elseif ((fabs (a_x + 1.0)) * (a_lmax.to_double + 1.0)) * (a_lmax.to_double + 1.0) < 1.4901161193847656e-08 then
					l_ell := 2
					from
					until
						not (l_ell <= a_lmax)
					loop
						if ((l_ell).bit_and (1)).to_boolean then
							l_tmp___0 := 1.0
						else
							l_tmp___0 := -1.0
						end
						l_sgn := l_tmp___0
						l_pre___0 := ((l_sgn * 0.5) * l_ell.to_double) * (l_ell.to_double + 1.0)
						a_result_deriv_array[l_ell] := l_pre___0 * (1.0 - ((0.25 * (1.0 + a_x)) * (l_ell.to_double + 2.0)) * (l_ell.to_double - 1.0))
						l_ell := l_ell + 1
					end
				else
					l_diff_a := 1.0 + a_x
					l_diff_b := 1.0 - a_x
					l_ell := 2
					from
					until
						not (l_ell <= a_lmax)
					loop
						a_result_deriv_array[l_ell] := ((-l_ell).to_double * (a_x * a_result_array[l_ell] - a_result_array[(l_ell - 1)])) / (l_diff_a * l_diff_b)
						l_ell := l_ell + 1
					end
				end
				Result := 0
			else
				Result := l_stat_array
			end
		end

	gsl_sf_legendre_Plm_array (a_lmax, a_m___4: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_dif: REAL_64
			l_sum: REAL_64
			l_t_d: REAL_64
			l_tmp___0: REAL_64
			l_t_s: REAL_64
			l_tmp___2: REAL_64
			l_exp_check: REAL_64
			l_ell: INTEGER_32
			l_p_mm: REAL_64
			l_p_mmp1: REAL_64
			l_p_ellm2: REAL_64
			l_p_ellm1: REAL_64
			l_p_ell: REAL_64
			l_ell___0: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_dif := (a_lmax - a_m___4).to_double
					l_sum := (a_lmax + a_m___4).to_double
					if l_dif = 0.0 then
						l_tmp___0 := 0.0
					else
						l_tmp___0 := (0.5 * l_dif) * ((log (l_dif)) - 1.0)
					end
					l_t_d := l_tmp___0
					if l_dif = 0.0 then
						l_tmp___2 := 0.0
					else
						l_tmp___2 := (0.5 * l_sum) * ((log (l_sum)) - 1.0)
					end
					l_t_s := l_tmp___2
					l_exp_check := (0.5 * (log (2.0 * a_lmax.to_double + 1.0)) + l_t_d) - l_t_s
					if not (a_m___4 < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L___1
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else:
						if not (a_lmax < a_m___4) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L___1
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else:
						if not (a_x < -1.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L___1
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else:
						if not (a_x > 1.0) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L___1:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("legendre_poly.c"), 377, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else:
						if not (a_m___4 > 0) then
							l_pos := -1 + 21 -- skip if block
						end
				when 13 then -- if:
						if a_x /= 1.0 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						l_pos := -1 + 17 -- goto _L___0
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 16 then -- else:
						if a_x /= -1.0 then
							l_pos := -1 + 19 -- skip if block
						end
				when 17 then -- if: | _L___0:
						l_ell := a_m___4
						from
						until
							not (l_ell <= a_lmax)
						loop
							a_result_array[(l_ell - a_m___4)] := 0.0
							l_ell := l_ell + 1
						end
						Result := 0
						l_return := True
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:
						l_pos := -1 + 21 -- goto _L
				when 20 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L:
						if l_exp_check < -7.0839641853226408e+02 + 10.0 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("overflow"), ce_string ("legendre_poly.c"), 386, 16)
								Result := 16
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_p_mm := (legendre_Pmm (a_m___4, a_x))
							l_p_mmp1 := (a_x * (2.0 * a_m___4.to_double + 1.0)) * l_p_mm
							if a_lmax = a_m___4 then
								a_result_array[0] := l_p_mm
								Result := 0
								l_return := True
							elseif a_lmax = a_m___4 + 1 then
								a_result_array[0] := l_p_mm
								a_result_array[1] := l_p_mmp1
								Result := 0
								l_return := True
							else
								l_p_ellm2 := l_p_mm
								l_p_ellm1 := l_p_mmp1
								l_p_ell := 0.0
								a_result_array[0] := l_p_mm
								a_result_array[1] := l_p_mmp1
								l_ell___0 := a_m___4 + 2
								from
								until
									not (l_ell___0 <= a_lmax)
								loop
									l_p_ell := ((a_x * (2.0 * l_ell___0.to_double - 1.0)) * l_p_ellm1 - ((l_ell___0 + a_m___4) - 1).to_double * l_p_ellm2) / (l_ell___0 - a_m___4).to_double
									l_p_ellm2 := l_p_ellm1
									l_p_ellm1 := l_p_ell
									a_result_array[(l_ell___0 - a_m___4)] := l_p_ell
									l_ell___0 := l_ell___0 + 1
								end
								Result := 0
								l_return := True
							end
						end
				when 22 then -- if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 23 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_legendre_Plm_deriv_array (a_lmax, a_m___4: INTEGER_32; a_x: REAL_64; a_result_array, a_result_deriv_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_stat_array: INTEGER_32
			l_ell: INTEGER_32
			l_sgn: REAL_64
			l_tmp___1: REAL_64
			l_diff_a: REAL_64
			l_diff_b: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_m___4 < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L___1
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 3 then -- else:
						if not (a_m___4 > a_lmax) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L___1:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("m < 0 or m > lmax"), ce_string ("legendre_poly.c"), 431, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 6 then -- else:
						if a_m___4 /= 0 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						Result := (gsl_sf_legendre_Pl_deriv_array (a_lmax, a_x, a_result_array, a_result_deriv_array))
						l_return := True
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 9 then -- else:
						l_stat_array := (gsl_sf_legendre_Plm_array (a_lmax, a_m___4, a_x, a_result_array))
						if l_stat_array /= 0 then
							l_pos := -1 + 18 -- skip if block
						end
				when 10 then -- if:
						if a_m___4 /= 1 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if 1.0 - (fabs (a_x)) < 2.2204460492503131e-16 then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("divergence near |x| = 1.0 since m = 1"), ce_string ("legendre_poly.c"), 453, 16)
								Result := 16
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						else
							l_pos := -1 + 13 -- goto _L___0
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 13 then -- else: | _L___0:
						if a_m___4 /= 2 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						if 1.0 - (fabs (a_x)) < 2.2204460492503131e-16 then
							if (fabs (a_x - 1.0)) < 2.2204460492503131e-16 then
								l_ell := a_m___4
								from
								until
									not (l_ell <= a_lmax)
								loop
									a_result_deriv_array[(l_ell - a_m___4)] := ((((-0.25 * a_x) * (l_ell.to_double - 1.0)) * l_ell.to_double) * (l_ell.to_double + 1.0)) * (l_ell.to_double + 2.0)
									l_ell := l_ell + 1
								end
							elseif (fabs (a_x + 1.0)) < 2.2204460492503131e-16 then
								l_ell := a_m___4
								from
								until
									not (l_ell <= a_lmax)
								loop
									if ((l_ell).bit_and (1)).to_boolean then
										l_tmp___1 := 1.0
									else
										l_tmp___1 := -1.0
									end
									l_sgn := l_tmp___1
									a_result_deriv_array[(l_ell - a_m___4)] := (((((-0.25 * l_sgn) * a_x) * (l_ell.to_double - 1.0)) * l_ell.to_double) * (l_ell.to_double + 1.0)) * (l_ell.to_double + 2.0)
									l_ell := l_ell + 1
								end
							end
							Result := 0
							l_return := True
						else
							l_pos := -1 + 16 -- goto _L
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 16 then -- else: | _L:
						if 1.0 - (fabs (a_x)) < 2.2204460492503131e-16 then
							l_ell := a_m___4
							from
							until
								not (l_ell <= a_lmax)
							loop
								a_result_deriv_array[(l_ell - a_m___4)] := 0.0
								l_ell := l_ell + 1
							end
							Result := 0
							l_return := True
						else
							l_diff_a := 1.0 + a_x
							l_diff_b := 1.0 - a_x
							a_result_deriv_array[0] := (((-a_m___4).to_double * a_x) / (l_diff_a * l_diff_b)) * a_result_array[0]
							if a_lmax - a_m___4 >= 1 then
								a_result_deriv_array[1] := (2.0 * a_m___4.to_double + 1.0) * (a_x * a_result_deriv_array[0] + a_result_array[0])
							end
							l_ell := a_m___4 + 2
							from
							until
								not (l_ell <= a_lmax)
							loop
								a_result_deriv_array[(l_ell - a_m___4)] := -((l_ell.to_double * a_x) * a_result_array[(l_ell - a_m___4)] - (l_ell + a_m___4).to_double * a_result_array[((l_ell - 1) - a_m___4)]) / (l_diff_a * l_diff_b)
								l_ell := l_ell + 1
							end
							Result := 0
							l_return := True
						end
				when 17 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else:
						Result := l_stat_array
						l_return := True
				when 19 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_legendre_sphPlm_array (a_lmax, a_m___4: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_ell: INTEGER_32
			l_y_mm: REAL_64
			l_y_mmp1: REAL_64
			l_lncirc: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_lnpoch: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_lnpre: REAL_64
			l_sgn: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_y_ell: REAL_64
			l_ell___0: INTEGER_32
			l_rat1: REAL_64
			l_rat2: REAL_64
			l_factor1: REAL_64
			l_factor2: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_lncirc.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_lnpoch.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_m___4 < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L___1
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else:
						if not (a_lmax < a_m___4) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L___1
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else:
						if not (a_x < -1.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L___1
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else:
						if not (a_x > 1.0) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L___1:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("error"), ce_string ("legendre_poly.c"), 602, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else:
						if not (a_m___4 > 0) then
							l_pos := -1 + 21 -- skip if block
						end
				when 13 then -- if:
						if a_x /= 1.0 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						l_pos := -1 + 17 -- goto _L___0
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 16 then -- else:
						if a_x /= -1.0 then
							l_pos := -1 + 19 -- skip if block
						end
				when 17 then -- if: | _L___0:
						l_ell := a_m___4
						from
						until
							not (l_ell <= a_lmax)
						loop
							a_result_array[(l_ell - a_m___4)] := 0.0
							l_ell := l_ell + 1
						end
						Result := 0
						l_return := True
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:
						l_pos := -1 + 21 -- goto _L
				when 20 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L:
						if a_m___4 = 0 then
							l_y_mm := 0.5 / 1.77245385090551602729816748334
							l_y_mmp1 := (a_x * 1.73205080756887729352744634151) * l_y_mm
						else
							if ((a_m___4).bit_and (1)).to_boolean then
								l_tmp := -1.0
							else
								l_tmp := 1.0
							end
							l_sgn := l_tmp
							l_lncirc.item.gsl_sf_log_1plusx_e (-a_x * a_x).do_nothing
							l_lnpoch.item.gsl_sf_lnpoch_e (a_m___4.to_double, 0.5).do_nothing
							l_lnpre := -0.25 * 1.14472988584940017414342735135 + 0.5 * (l_lnpoch.item.val + a_m___4.to_double * l_lncirc.item.val)
							l_tmp___0 := sqrt ((2.0 + 1.0 / a_m___4.to_double) / (4.0 * 3.14159265358979323846))
							l_y_mm := (l_tmp___0 * l_sgn) * (exp (l_lnpre))
							l_y_mmp1 := (a_x * (sqrt (2.0 * a_m___4.to_double + 3.0))) * l_y_mm
						end
						if a_lmax = a_m___4 then
							a_result_array[0] := l_y_mm
							Result := 0
							l_return := True
						elseif a_lmax = a_m___4 + 1 then
							a_result_array[0] := l_y_mm
							a_result_array[1] := l_y_mmp1
							Result := 0
							l_return := True
						else
							a_result_array[0] := l_y_mm
							a_result_array[1] := l_y_mmp1
							l_ell___0 := a_m___4 + 2
							from
							until
								not (l_ell___0 <= a_lmax)
							loop
								l_rat1 := (l_ell___0 - a_m___4).to_double / (l_ell___0 + a_m___4).to_double
								l_rat2 := ((l_ell___0 - a_m___4).to_double - 1.0) / ((l_ell___0 + a_m___4).to_double - 1.0)
								l_factor1 := (sqrt ((l_rat1 * (2 * l_ell___0 + 1).to_double) * (2 * l_ell___0 - 1).to_double))
								l_factor2 := (sqrt (((l_rat1 * l_rat2) * (2 * l_ell___0 + 1).to_double) / (2 * l_ell___0 - 3).to_double))
								l_y_ell := ((a_x * l_y_mmp1) * l_factor1 - (((l_ell___0 + a_m___4) - 1).to_double * l_y_mm) * l_factor2) / (l_ell___0 - a_m___4).to_double
								l_y_mm := l_y_mmp1
								l_y_mmp1 := l_y_ell
								a_result_array[(l_ell___0 - a_m___4)] := l_y_ell
								l_ell___0 := l_ell___0 + 1
							end
						end
						if not l_return then
							Result := 0
							l_return := True
						end
				when 22 then -- if done | if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 23 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_legendre_sphPlm_deriv_array (a_lmax, a_m___4: INTEGER_32; a_x: REAL_64; a_result_array, a_result_deriv_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_stat_array: INTEGER_32
			l_ell: INTEGER_32
			l_prefactor: REAL_64
			l_stat_array___0: INTEGER_32
			l_ell___0: INTEGER_32
			l_prefactor___0: REAL_64
			l_stat_array___1: INTEGER_32
			l_ell___1: INTEGER_32
			l_diff_a: REAL_64
			l_diff_b: REAL_64
			l_c1___2: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_m___4 < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if not (a_lmax < a_m___4) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (a_x < -1.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if not (a_x > 1.0) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain"), ce_string ("legendre_poly.c"), 673, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						if a_m___4 = 0 then
							l_stat_array := (gsl_sf_legendre_Pl_deriv_array (a_lmax, a_x, a_result_array, a_result_deriv_array))
							l_ell := 0
							from
							until
								not (l_ell <= a_lmax)
							loop
								l_prefactor := (sqrt ((2.0 * l_ell.to_double + 1.0) / (4.0 * 3.14159265358979323846)))
								a_result_array[l_ell] := a_result_array[l_ell] * l_prefactor
								a_result_deriv_array[l_ell] := a_result_deriv_array[l_ell] * l_prefactor
								l_ell := l_ell + 1
							end
							Result := l_stat_array
							l_return := True
						elseif a_m___4 = 1 then
							l_stat_array___0 := (gsl_sf_legendre_Plm_deriv_array (a_lmax, a_m___4, a_x, a_result_array, a_result_deriv_array))
							l_ell___0 := 1
							from
							until
								not (l_ell___0 <= a_lmax)
							loop
								l_prefactor___0 := (sqrt (((2.0 * l_ell___0.to_double + 1.0) / (l_ell___0.to_double + 1.0)) / ((4.0 * 3.14159265358979323846) * l_ell___0.to_double)))
								a_result_array[(l_ell___0 - 1)] := a_result_array[(l_ell___0 - 1)] * l_prefactor___0
								a_result_deriv_array[(l_ell___0 - 1)] := a_result_deriv_array[(l_ell___0 - 1)] * l_prefactor___0
								l_ell___0 := l_ell___0 + 1
							end
							Result := l_stat_array___0
							l_return := True
						else
							l_stat_array___1 := (gsl_sf_legendre_sphPlm_array (a_lmax, a_m___4, a_x, a_result_array))
							if l_stat_array___1 = 0 then
								if 1.0 - (fabs (a_x)) < 2.2204460492503131e-16 then
									l_ell___1 := a_m___4
									from
									until
										not (l_ell___1 <= a_lmax)
									loop
										a_result_deriv_array[(l_ell___1 - a_m___4)] := 0.0
										l_ell___1 := l_ell___1 + 1
									end
									Result := 0
									l_return := True
								else
									l_diff_a := 1.0 + a_x
									l_diff_b := 1.0 - a_x
									a_result_deriv_array[0] := (((-a_m___4).to_double * a_x) / (l_diff_a * l_diff_b)) * a_result_array[0]
									if a_lmax - a_m___4 >= 1 then
										a_result_deriv_array[1] := (sqrt (2.0 * a_m___4.to_double + 3.0)) * (a_x * a_result_deriv_array[0] + a_result_array[0])
									end
									l_ell___1 := a_m___4 + 2
									from
									until
										not (l_ell___1 <= a_lmax)
									loop
										l_c1___2 := (sqrt (((2.0 * l_ell___1.to_double + 1.0) / (2.0 * l_ell___1.to_double - 1.0)) * ((l_ell___1 - a_m___4).to_double / (l_ell___1 + a_m___4).to_double)))
										a_result_deriv_array[(l_ell___1 - a_m___4)] := -((l_ell___1.to_double * a_x) * a_result_array[(l_ell___1 - a_m___4)] - (l_c1___2 * (l_ell___1 + a_m___4).to_double) * a_result_array[((l_ell___1 - 1) - a_m___4)]) / (l_diff_a * l_diff_b)
										l_ell___1 := l_ell___1 + 1
									end
									Result := 0
									l_return := True
								end
							else
								Result := l_stat_array___1
								l_return := True
							end
						end
				when 13 then -- if done | if done | if done
					if not l_return then
						Result := 0
					end
				when 14 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_sf_legendre_array_size (a_lmax, a_m___4: INTEGER_32): INTEGER_32
		do
			Result := ((a_lmax - a_m___4) + 1)
		end

	gsl_sf_legendre_P1 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_legendre_P1_e (a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_legendre_P1_e(x, &result)"), ce_string ("legendre_poly.c"), 752, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_legendre_P2 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_legendre_P2_e (a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_legendre_P2_e(x, &result)"), ce_string ("legendre_poly.c"), 757, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_legendre_P3 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_legendre_P3_e (a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_legendre_P3_e(x, &result)"), ce_string ("legendre_poly.c"), 762, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_legendre_Pl (a_l: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_legendre_Pl_e (a_l, a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_legendre_Pl_e(l, x, &result)"), ce_string ("legendre_poly.c"), 767, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_legendre_Plm (a_l, a_m___4: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_legendre_Plm_e (a_l, a_m___4, a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_legendre_Plm_e(l, m, x, &result)"), ce_string ("legendre_poly.c"), 772, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_legendre_sphPlm (a_l, a_m___4: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_result.make (<<1>>)

			l_status := (l_result.item.gsl_sf_legendre_sphPlm_e (a_l, a_m___4, a_x))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("gsl_sf_legendre_sphPlm_e(l, m, x, &result)"), ce_string ("legendre_poly.c"), 777, l_status)
					Result := (l_result.item.val)
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item.val)
			end
		end

end
