class
	PD_GSL_DHT_STRUCT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} j.make
			create {CE_REAL_64_POINTER} Jjj.make
			create {CE_REAL_64_POINTER} J2.make
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	nu: REAL_64 assign set_nu

	xmax: REAL_64 assign set_xmax

	kmax: REAL_64 assign set_kmax

	j: CE_POINTER [REAL_64] assign set_j

	Jjj: CE_POINTER [REAL_64] assign set_Jjj

	J2: CE_POINTER [REAL_64] assign set_J2

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_nu (a_nu: REAL_64)
		do
			nu := a_nu
			update_memory_field ("nu")
		end

	set_xmax (a_xmax: REAL_64)
		do
			xmax := a_xmax
			update_memory_field ("xmax")
		end

	set_kmax (a_kmax: REAL_64)
		do
			kmax := a_kmax
			update_memory_field ("kmax")
		end

	set_j (a_j: CE_POINTER [REAL_64])
		do
			j := a_j
			update_memory_field ("j")
		end

	set_Jjj (a_Jjj: CE_POINTER [REAL_64])
		do
			Jjj := a_Jjj
			update_memory_field ("Jjj")
		end

	set_J2 (a_J2: CE_POINTER [REAL_64])
		do
			J2 := a_J2
			update_memory_field ("J2")
		end

feature {ANY} -- public routines

	dht_bessel_zeros: INTEGER_32
		local
			l_s: NATURAL_32
			l_z: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_z: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_z.make (<<1>>)

			l_stat_z := 0
			j[0] := 0.0
			update_class_fields -- Current might have been changed; synchronize
			l_s := {NATURAL_32} 1
			from
			until
				not (l_s < size + {NATURAL_32} 2)
			loop
				l_tmp := l_z.item.gsl_sf_bessel_zero_Jnu_e (nu, l_s)
				update_class_fields -- Current might have been changed; synchronize
				l_stat_z := l_stat_z + l_tmp
				j[l_s.to_integer_32] := l_z.item.val
				update_class_fields -- Current might have been changed; synchronize
				l_s := l_s + 1
			end
			if l_stat_z /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("could not compute bessel zeroes"), ce_string ("dht.c"), 91, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_dht_init (a_nu, a_xmax: REAL_64): INTEGER_32
		local
			l_n: NATURAL_32
			l_m___4: NATURAL_32
			l_stat_bz: INTEGER_32
			l_stat_J: INTEGER_32
			l_jN: REAL_64
			l_J: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp: INTEGER_32
			l_arg: REAL_64
			l_J___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___0: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_J.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_J___0.make (<<1>>)

			if a_xmax <= 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("xmax is not positive"), ce_string ("dht.c"), 120, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_nu < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("nu is negative"), ce_string ("dht.c"), 122, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stat_bz := 0
				l_stat_J := 0
				if a_nu /= nu then
					Current.nu := a_nu
					update_class_fields -- Current might have been changed; synchronize
					l_stat_bz := ((create {CE_CLASS_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (c_pointer))).item.dht_bessel_zeros
					update_class_fields -- Current might have been changed; synchronize
				end
				l_jN := j[(size + {NATURAL_32} 1).to_integer_32]
				Current.xmax := a_xmax
				update_class_fields -- Current might have been changed; synchronize
				Current.kmax := l_jN / a_xmax
				update_class_fields -- Current might have been changed; synchronize
				J2[0] := 0.0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4 := (1).to_natural_32
				from
				until
					not (l_m___4 < size + {NATURAL_32} 1)
				loop
					l_tmp := l_J.item.gsl_sf_bessel_Jnu_e (a_nu + 1.0, j[l_m___4.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_stat_J := l_stat_J + l_tmp
					J2[l_m___4.to_integer_32] := l_J.item.val * l_J.item.val
					update_class_fields -- Current might have been changed; synchronize
					l_m___4 := l_m___4 + 1
				end
				l_n := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_n < size + {NATURAL_32} 1))
				loop
					l_m___4 := (1).to_natural_32
					from
					until
						not (l_m___4 <= l_n)
					loop
						l_arg := (j[l_n.to_integer_32] * j[l_m___4.to_integer_32]) / l_jN
						l_tmp___0 := l_J___0.item.gsl_sf_bessel_Jnu_e (a_nu, l_arg)
						update_class_fields -- Current might have been changed; synchronize
						l_stat_J := l_stat_J + l_tmp___0
						Jjj[(((l_n * (l_n - {NATURAL_32} 1)) // {NATURAL_32} 2 + l_m___4) - {NATURAL_32} 1).to_integer_32] := l_J___0.item.val
						update_class_fields -- Current might have been changed; synchronize
						l_m___4 := l_m___4 + 1
					end
					l_n := l_n + 1
				end
				if l_stat_J /= 0 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("error computing bessel function"), ce_string ("dht.c"), 160, 5)
						update_class_fields -- Current might have been changed; synchronize
						Result := 5
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				else
					Result := l_stat_bz
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_dht_x_sample (a_n: INTEGER_32): REAL_64
		do
			Result := ((j[(a_n + 1)] / j[(size + {NATURAL_32} 1).to_integer_32]) * xmax)
		end

	gsl_dht_k_sample (a_n: INTEGER_32): REAL_64
		do
			Result := (j[(a_n + 1)] / xmax)
		end

	gsl_dht_apply (a_f_in, a_f_out: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_jN: REAL_64
			l_r___3: REAL_64
			l_m___4: NATURAL_32
			l_i: NATURAL_32
			l_sum: REAL_64
			l_Y: REAL_64
			l_m_local: NATURAL_32
			l_n_local: NATURAL_32
			l_return: BOOLEAN
		do
			l_jN := j[(size + {NATURAL_32} 1).to_integer_32]
			l_r___3 := (xmax / l_jN)
			l_m___4 := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_m___4 < size))
			loop
				l_sum := 0.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < size)
				loop
					if l_i < l_m___4 then
						l_m_local := l_i
						l_n_local := l_m___4
					else
						l_m_local := l_m___4
						l_n_local := l_i
					end
					l_Y := Jjj[((l_n_local * (l_n_local + {NATURAL_32} 1)) // {NATURAL_32} 2 + l_m_local).to_integer_32] / J2[(l_i + {NATURAL_32} 1).to_integer_32]
					l_sum := l_sum + (l_Y * a_f_in[l_i.to_integer_32])
					l_i := l_i + 1
				end
				a_f_out[l_m___4.to_integer_32] := ((l_sum * 2.0) * l_r___3) * l_r___3
				update_class_fields -- Current might have been changed; synchronize
				l_m___4 := l_m___4 + 1
			end
			Result := 0
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			nu := a_other.nu
			xmax := a_other.xmax
			kmax := a_other.kmax
			j := a_other.j
			Jjj := a_other.Jjj
			J2 := a_other.J2
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				nu = a_other.nu and
				xmax = a_other.xmax and
				kmax = a_other.kmax and
				j.is_equal (a_other.j) and
				Jjj.is_equal (a_other.Jjj) and
				J2.is_equal (a_other.J2)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("nu"),
					create {CE_STRUCTURE_FIELD}.make ("xmax"),
					create {CE_STRUCTURE_FIELD}.make ("kmax"),
					create {CE_STRUCTURE_FIELD}.make ("j"),
					create {CE_STRUCTURE_FIELD}.make ("Jjj"),
					create {CE_STRUCTURE_FIELD}.make ("J2")
				>>)
			init_structure_info
		end

end
