class
	PD_GSL_HISTOGRAM2D_PDF

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} xrange.make
			create {CE_REAL_64_POINTER} yrange.make
			create {CE_REAL_64_POINTER} sum.make
		end

feature {ANY} -- fields

	nx: NATURAL_32 assign set_nx

	ny: NATURAL_32 assign set_ny

	xrange: CE_POINTER [REAL_64] assign set_xrange

	yrange: CE_POINTER [REAL_64] assign set_yrange

	sum: CE_POINTER [REAL_64] assign set_sum

feature {ANY} -- field setters

	set_nx (a_nx: NATURAL_32)
		do
			nx := a_nx
			update_memory_field ("nx")
		end

	set_ny (a_ny: NATURAL_32)
		do
			ny := a_ny
			update_memory_field ("ny")
		end

	set_xrange (a_xrange: CE_POINTER [REAL_64])
		do
			xrange := a_xrange
			update_memory_field ("xrange")
		end

	set_yrange (a_yrange: CE_POINTER [REAL_64])
		do
			yrange := a_yrange
			update_memory_field ("yrange")
		end

	set_sum (a_sum: CE_POINTER [REAL_64])
		do
			sum := a_sum
			update_memory_field ("sum")
		end

feature {ANY} -- public routines

	gsl_histogram2d_pdf_sample (a_r1___1, a_r2___0: REAL_64; a_x, a_y: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_r1___1: REAL_64
			l_r2___0: REAL_64
			l_k: CE_ARRAY [NATURAL_32]
			l_status: INTEGER_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_delta___0: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_32_ARRAY} l_k.make (<<1>>)

			l_r1___1 := a_r1___1
			l_r2___0 := a_r2___0
			if l_r2___0 = 1.0 then
				l_r2___0 := 0.0
			end
			if l_r1___1 = 1.0 then
				l_r1___1 := 0.0
			end
			l_status := find___3 ((nx * ny), sum, l_r1___1, l_k)
			update_class_fields -- Current might have been changed; synchronize
			if l_status.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("cannot find r1 in cumulative pdf"), ce_string ("pdf2d.c"), 53, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_i := l_k.item // ny
				l_j := l_k.item - l_i * ny
				l_delta___0 := (l_r1___1 - sum[(l_k.item).to_integer_32]) / (sum[(l_k.item + {NATURAL_32} 1).to_integer_32] - sum[(l_k.item).to_integer_32])
				a_x.item := xrange[l_i.to_integer_32] + l_delta___0 * (xrange[(l_i + {NATURAL_32} 1).to_integer_32] - xrange[l_i.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				a_y.item := yrange[l_j.to_integer_32] + l_r2___0 * (yrange[(l_j + {NATURAL_32} 1).to_integer_32] - yrange[l_j.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_histogram2d_pdf_init (a_h: CE_POINTER [PD_GSL_HISTOGRAM2D]): INTEGER_32
		require
			not a_h.is_void
		local
			l_i: NATURAL_32
			l_nx: NATURAL_32
			l_ny: NATURAL_32
			l_n: NATURAL_32
			l_mean: REAL_64
			l_sum: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nx := nx
					l_ny := ny
					l_n := l_nx * l_ny
					if l_nx = a_h.item.nx then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_ny = a_h.item.ny then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram2d size must match pdf size"), ce_string ("pdf2d.c"), 136, 1)
							update_class_fields -- Current might have been changed; synchronize
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_n))
						loop
							if a_h.item.bin[l_i.to_integer_32] < (0).to_double then
								from
								until
									(l_return or l_break)
								loop
									l_break := False;
									(create {P_ERROR_STATIC}).gsl_error (ce_string ("histogram bins must be non-negative to computea probability distribution"), ce_string ("pdf2d.c"), 144, 1)
									update_class_fields -- Current might have been changed; synchronize
									Result := 1
									l_return := True
									if not l_return then
										l_break := True
									end
								end
								if not l_return then
									l_break := False
								end
							end
							if not l_return then
								l_i := l_i + 1
							end
						end
					end
					if not l_return then
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_nx + {NATURAL_32} 1)
						loop
							xrange[l_i.to_integer_32] := a_h.item.xrange[l_i.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_ny + {NATURAL_32} 1)
						loop
							yrange[l_i.to_integer_32] := a_h.item.yrange[l_i.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_mean := (0).to_double
						l_sum := (0).to_double
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_n)
						loop
							l_mean := l_mean + ((a_h.item.bin[l_i.to_integer_32] - l_mean) / (l_i + {NATURAL_32} 1).to_real_64)
							l_i := l_i + 1
						end
						sum[0] := (0).to_double
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_n)
						loop
							l_sum := l_sum + ((a_h.item.bin[l_i.to_integer_32] / l_mean) / l_n.to_real_64)
							sum[(l_i + {NATURAL_32} 1).to_integer_32] := l_sum
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {PD_GSL_HISTOGRAM2D_PDF} -- private routines

	find___3 (a_n: NATURAL_32; a_range: CE_POINTER [REAL_64]; a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]): INTEGER_32
		require
			not a_range.is_void
		local
			l_i_linear: NATURAL_32
			l_lower: NATURAL_32
			l_upper: NATURAL_32
			l_mid: NATURAL_32
			l_u: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_x < a_range[0] then
						Result := (-1)
						l_return := True
					end
					if not l_return and then (a_x >= a_range[a_n.to_integer_32]) then
						Result := 1
						l_return := True
					end
					if not l_return then
						l_u := (a_x - a_range[0]) / (a_range[a_n.to_integer_32] - a_range[0])
						l_i_linear := (l_u * a_n.to_real_64).truncated_to_integer_64.to_natural_32
						if a_x >= a_range[l_i_linear.to_integer_32] then
							if a_x < a_range[(l_i_linear + {NATURAL_32} 1).to_integer_32] then
								a_i.item := l_i_linear
								Result := 0
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_upper := a_n
						l_lower := (0).to_natural_32
						from
						until
							not (l_upper - l_lower > {NATURAL_32} 1)
						loop
							l_mid := (l_upper + l_lower) // {NATURAL_32} 2
							if a_x >= a_range[l_mid.to_integer_32] then
								l_lower := l_mid
							else
								l_upper := l_mid
							end
						end
						a_i.item := l_lower
						if not (a_x < a_range[l_lower.to_integer_32]) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 6
				when 4 then -- else:
						if not (a_x >= a_range[(l_lower + {NATURAL_32} 1).to_integer_32]) then
							l_pos := -1 + 6 -- skip if block
						end
				when 5 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x not found in range"), ce_string ("find.c"), 81, 7)
							Result := 7
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 6 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			nx := a_other.nx
			ny := a_other.ny
			xrange := a_other.xrange
			yrange := a_other.yrange
			sum := a_other.sum
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				nx = a_other.nx and
				ny = a_other.ny and
				xrange.is_equal (a_other.xrange) and
				yrange.is_equal (a_other.yrange) and
				sum.is_equal (a_other.sum)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("nx"),
					create {CE_STRUCTURE_FIELD}.make ("ny"),
					create {CE_STRUCTURE_FIELD}.make ("xrange"),
					create {CE_STRUCTURE_FIELD}.make ("yrange"),
					create {CE_STRUCTURE_FIELD}.make ("sum")
				>>)
			init_structure_info
		end

end
