class
	PD_GSL_IEEE_FLOAT_REP

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_INTEGER_8_ARRAY} mantissa.make (<<24>>)
		end

feature {ANY} -- fields

	sign: INTEGER_32 assign set_sign

	mantissa: CE_ARRAY [INTEGER_8]

	exponent: INTEGER_32 assign set_exponent

	type: INTEGER_32 assign set_type

feature {ANY} -- field setters

	set_sign (a_sign: INTEGER_32)
		do
			sign := a_sign
			update_memory_field ("sign")
		end

	set_exponent (a_exponent: INTEGER_32)
		do
			exponent := a_exponent
			update_memory_field ("exponent")
		end

	set_type (a_type: INTEGER_32)
		do
			type := a_type
			update_memory_field ("type")
		end

feature {ANY} -- public routines

	gsl_ieee_float_to_rep (a_x: CE_POINTER [REAL_32])
		require
			not a_x.is_void
		local
			l_e: INTEGER_32
			l_non_zero: INTEGER_32
			l_u: CE_ARRAY [PD_U4]
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_U4]} l_u.make (<<1>>)

			l_u.item.f := a_x.item
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := (create {P_ENDIAN_STATIC}).little_endian_p
			update_class_fields -- Current might have been changed; synchronize
			if l_tmp.to_boolean then
				make_float_bigendian (create {CE_REAL_32_POINTER}.make_shared (l_u.item.c_pointer + (create {PD_U4}.make).offset_of ("f")))
				update_class_fields -- Current might have been changed; synchronize
			end
			if (((l_u.item.ieee.byte[3]).to_integer_32).bit_shift_right (7)).to_boolean then
				Current.sign := 1
				update_class_fields -- Current might have been changed; synchronize
			else
				Current.sign := 0
				update_class_fields -- Current might have been changed; synchronize
			end
			l_e := ((((((l_u.item.ieee.byte[3]).to_integer_32).bit_and (127))).bit_shift_left (1))).bit_or ((((((l_u.item.ieee.byte[2]).to_integer_32).bit_and (128))).bit_shift_right (7)))
			Current.exponent := l_e - 127
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_MAKE_REP_STATIC}).sprint_byte (((((l_u.item.ieee.byte[2]).to_integer_32).bit_and (127))).bit_shift_left (1), mantissa)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_MAKE_REP_STATIC}).sprint_byte ((l_u.item.ieee.byte[1]).to_integer_32, mantissa + 7)
			update_class_fields; -- Current might have been changed; synchronize
			(create {P_MAKE_REP_STATIC}).sprint_byte ((l_u.item.ieee.byte[0]).to_integer_32, mantissa + 15)
			update_class_fields -- Current might have been changed; synchronize
			mantissa[23] := (('%/0/').code).to_integer_8
			update_class_fields -- Current might have been changed; synchronize
			if (l_u.item.ieee.byte[0]).to_boolean then
				l_tmp___0 := 1
			elseif (l_u.item.ieee.byte[1]).to_boolean then
				l_tmp___0 := 1
			elseif (((l_u.item.ieee.byte[2]).to_integer_32).bit_and (127)).to_boolean then
				l_tmp___0 := 1
			else
				l_tmp___0 := 0
			end
			l_non_zero := l_tmp___0
			Current.type := (create {P_MAKE_REP_STATIC}).determine_ieee_type (l_non_zero, l_e, 255)
			update_class_fields -- Current might have been changed; synchronize
		end

feature {PD_GSL_IEEE_FLOAT_REP} -- private routines

	make_float_bigendian (a_x: CE_POINTER [REAL_32])
		require
			not a_x.is_void
		local
			l_u: PD_U2
			l_v: PD_U2
		do
			create l_u.make
			create l_v.make

			l_u.f := a_x.item
			l_u.update_class_fields -- synchronize union
			l_v.b[0] := l_u.b[3]
			l_v.update_class_fields -- synchronize union
			l_v.b[1] := l_u.b[2]
			l_v.update_class_fields -- synchronize union
			l_v.b[2] := l_u.b[1]
			l_v.update_class_fields -- synchronize union
			l_v.b[3] := l_u.b[0]
			l_v.update_class_fields -- synchronize union
			a_x.item := l_v.f
		end

feature {ANY}

	copy (a_other: like Current)
		do
			sign := a_other.sign
			mantissa := a_other.mantissa.twin
			exponent := a_other.exponent
			type := a_other.type
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				sign = a_other.sign and
				mantissa.is_equal (a_other.mantissa) and
				exponent = a_other.exponent and
				type = a_other.type
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("sign"),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("mantissa", <<24>>),
					create {CE_STRUCTURE_FIELD}.make ("exponent"),
					create {CE_STRUCTURE_FIELD}.make ("type")
				>>)
			init_structure_info
		end

end
