class
	PD_GSL_INTERP_ACCEL

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {ANY} -- fields

	cache: NATURAL_32 assign set_cache

	miss_count: NATURAL_32 assign set_miss_count

	hit_count: NATURAL_32 assign set_hit_count

feature {ANY} -- field setters

	set_cache (a_cache: NATURAL_32)
		do
			cache := a_cache
			update_memory_field ("cache")
		end

	set_miss_count (a_miss_count: NATURAL_32)
		do
			miss_count := a_miss_count
			update_memory_field ("miss_count")
		end

	set_hit_count (a_hit_count: NATURAL_32)
		do
			hit_count := a_hit_count
			update_memory_field ("hit_count")
		end

feature {ANY} -- public routines

	gsl_interp_accel_find (a_xa: CE_POINTER [REAL_64]; a_len: NATURAL_32; a_x: REAL_64): NATURAL_32
		require
			not a_xa.is_void
		local
			l_x_index: NATURAL_32
		do
			l_x_index := cache
			if a_x < a_xa[l_x_index.to_integer_32] then
				Current.miss_count := miss_count + 1
				update_class_fields -- Current might have been changed; synchronize
				Current.cache := (create {P_GSL_GSL_INTERP_STATIC}).gsl_interp_bsearch (a_xa, a_x, (0).to_natural_32, l_x_index)
				update_class_fields -- Current might have been changed; synchronize
			elseif a_x >= a_xa[(l_x_index + {NATURAL_32} 1).to_integer_32] then
				Current.miss_count := miss_count + 1
				update_class_fields -- Current might have been changed; synchronize
				Current.cache := (create {P_GSL_GSL_INTERP_STATIC}).gsl_interp_bsearch (a_xa, a_x, l_x_index, a_len - {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
			else
				Current.hit_count := hit_count + 1
				update_class_fields -- Current might have been changed; synchronize
			end
			Result := cache
		end

	gsl_interp_accel_reset: INTEGER_32
		do
			Current.cache := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			Current.hit_count := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			Current.miss_count := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

feature {ANY}

	copy (a_other: like Current)
		do
			cache := a_other.cache
			miss_count := a_other.miss_count
			hit_count := a_other.hit_count
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				cache = a_other.cache and
				miss_count = a_other.miss_count and
				hit_count = a_other.hit_count
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("cache"),
					create {CE_STRUCTURE_FIELD}.make ("miss_count"),
					create {CE_STRUCTURE_FIELD}.make ("hit_count")
				>>)
			init_structure_info
		end

end
