class
	PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT

inherit
	PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT_STATIC
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create f.make
			create df.make
			create fdf.make
			create {CE_VOID_POINTER} params.make
			Precursor
		end

feature {ANY} -- fields

	f: CE_ROUTINE assign set_f

	df: CE_ROUTINE assign set_df

	fdf: CE_ROUTINE assign set_fdf

	n: NATURAL_32 assign set_n

	params: CE_POINTER [ANY] assign set_params

feature {ANY} -- field setters

	set_f (a_f: CE_ROUTINE)
		do
			f := a_f
			update_memory_field ("f")
		end

	set_df (a_df: CE_ROUTINE)
		do
			df := a_df
			update_memory_field ("df")
		end

	set_fdf (a_fdf: CE_ROUTINE)
		do
			fdf := a_fdf
			update_memory_field ("fdf")
		end

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_params (a_params: CE_POINTER [ANY])
		do
			params := a_params
			update_memory_field ("params")
		end

feature {ANY} -- public routines

	newton_set (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_state: CE_POINTER [PD_NEWTON_STATE_T]
			l_i: NATURAL_32
			l_n: NATURAL_32
		do
			l_state := create {CE_CLASS_POINTER [PD_NEWTON_STATE_T]}.make_cast (a_vstate)
			l_n := n
			l_state := create {CE_CLASS_POINTER [PD_NEWTON_STATE_T]}.make_shared (default_pointer)
			fdf.call ([a_x, params, a_f, a_J])
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				a_dx.item.gsl_vector_set (l_i, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	newton_iterate (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_J.is_void
		local
			l_state: CE_POINTER [PD_NEWTON_STATE_T]
			l_signum: CE_ARRAY [INTEGER_32]
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_e: REAL_64
			l_tmp: REAL_64
			l_y: REAL_64
			l_tmp___0: REAL_64
			l_status: INTEGER_32
			l_tmp___1: INTEGER_32
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_signum.make (<<1>>)

			l_state := create {CE_CLASS_POINTER [PD_NEWTON_STATE_T]}.make_cast (a_vstate)
			l_n := n;
			(l_state.item.lu).item.gsl_matrix_memcpy (a_J).do_nothing
			update_class_fields; -- Current might have been changed; synchronize
			(l_state.item.lu).item.gsl_linalg_LU_decomp (l_state.item.permutation, l_signum).do_nothing
			update_class_fields; -- Current might have been changed; synchronize
			(l_state.item.lu).item.gsl_linalg_LU_solve (l_state.item.permutation, a_f, a_dx).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := a_dx.item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_e := l_tmp
				l_tmp___0 := a_x.item.gsl_vector_get (l_i)
				update_class_fields -- Current might have been changed; synchronize
				l_y := l_tmp___0
				a_dx.item.gsl_vector_set (l_i, -l_e)
				update_class_fields -- Current might have been changed; synchronize
				a_x.item.gsl_vector_set (l_i, l_y - l_e)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_tmp___1 := fdf.integer_32_item ([a_x, params, a_f, a_J])
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp___1
			if l_status /= 0 then
				Result := 9
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gnewton_set (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_f.is_void
		local
			l_state: CE_POINTER [PD_GNEWTON_STATE_T]
			l_i: NATURAL_32
			l_n: NATURAL_32
		do
			l_state := create {CE_CLASS_POINTER [PD_GNEWTON_STATE_T]}.make_cast (a_vstate)
			l_n := n
			fdf.call ([a_x, params, a_f, a_J])
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				a_dx.item.gsl_vector_set (l_i, 0.0)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_state.item.phi := enorm___0 (a_f)
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	gnewton_iterate (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_J.is_void
		local
			l_state: CE_POINTER [PD_GNEWTON_STATE_T]
			l_signum: CE_ARRAY [INTEGER_32]
			l_t: REAL_64
			l_phi0: REAL_64
			l_phi1: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_di: REAL_64
			l_tmp: REAL_64
			l_xi: REAL_64
			l_tmp___0: REAL_64
			l_status: INTEGER_32
			l_tmp___1: INTEGER_32
			l_theta: REAL_64
			l_u: REAL_64
			l_tmp___2: REAL_64
			l_di___0: REAL_64
			l_tmp___3: REAL_64
			l_status___0: INTEGER_32
			l_tmp___4: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_signum.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state := create {CE_CLASS_POINTER [PD_GNEWTON_STATE_T]}.make_cast (a_vstate)
					l_n := n;
					(l_state.item.lu).item.gsl_matrix_memcpy (a_J).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					(l_state.item.lu).item.gsl_linalg_LU_decomp (l_state.item.permutation, l_signum).do_nothing
					update_class_fields; -- Current might have been changed; synchronize
					(l_state.item.lu).item.gsl_linalg_LU_solve (l_state.item.permutation, a_f, l_state.item.d).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_t := (1).to_double
					l_phi0 := l_state.item.phi
				when 1 then -- new_step:
					l_i := (0).to_natural_32
					from
					until
						not (l_i < l_n)
					loop
						l_tmp := (l_state.item.d).item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_di := l_tmp
						l_tmp___0 := a_x.item.gsl_vector_get (l_i)
						update_class_fields -- Current might have been changed; synchronize
						l_xi := l_tmp___0;
						(l_state.item.x_trial).item.gsl_vector_set (l_i, l_xi - l_t * l_di)
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
					l_tmp___1 := f.integer_32_item ([l_state.item.x_trial, params, a_f])
					update_class_fields -- Current might have been changed; synchronize
					l_status := l_tmp___1
					if l_status /= 0 then
						Result := 9
						l_return := True
					end
					if l_return then
						l_pos := -1 + 4 -- skip if block
					end
				when 2 then -- if:
						l_phi1 := enorm___0 (a_f)
						update_class_fields -- Current might have been changed; synchronize
						if l_phi1 > l_phi0 then
							if l_t > 2.2204460492503131e-16 then
								l_theta := l_phi1 / l_phi0
								l_tmp___2 := sqrt (1.0 + 6.0 * l_theta)
								update_class_fields -- Current might have been changed; synchronize
								l_u := (l_tmp___2 - 1.0) / (3.0 * l_theta)
								l_t := l_t * l_u
								l_pos := -1 + 1 -- goto new_step
							end
						end
				when 3 then
						a_x.item.gsl_vector_memcpy (l_state.item.x_trial).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_i := (0).to_natural_32
						from
						until
							not (l_i < l_n)
						loop
							l_tmp___3 := (l_state.item.d).item.gsl_vector_get (l_i)
							update_class_fields -- Current might have been changed; synchronize
							l_di___0 := l_tmp___3
							a_dx.item.gsl_vector_set (l_i, -l_t * l_di___0)
							update_class_fields -- Current might have been changed; synchronize
							l_i := l_i + 1
						end
						l_tmp___4 := df.integer_32_item ([a_x, params, a_J])
						update_class_fields -- Current might have been changed; synchronize
						l_status___0 := l_tmp___4
						if l_status___0 /= 0 then
							Result := 9
							l_return := True
						end
				when 4 then -- if done
					if not l_return then
						l_state.item.phi := l_phi1
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	hybridj_set_impl (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]; a_scale: INTEGER_32): INTEGER_32
		require
			not a_x.is_void
			not a_f.is_void
			not a_J.is_void
			not a_dx.is_void
		local
			l_state: CE_POINTER [PD_HYBRIDJ_STATE_T]
			l_q___3: CE_POINTER [PD_GSL_MATRIX]
			l_r___3: CE_POINTER [PD_GSL_MATRIX]
			l_tau: CE_POINTER [PD_GSL_VECTOR]
			l_diag: CE_POINTER [PD_GSL_VECTOR]
		do
			l_state := create {CE_CLASS_POINTER [PD_HYBRIDJ_STATE_T]}.make_cast (a_vstate)
			l_q___3 := l_state.item.q
			l_r___3 := l_state.item.r
			l_tau := l_state.item.tau
			l_diag := l_state.item.diag
			fdf.call ([a_x, params, a_f, a_J])
			update_class_fields -- Current might have been changed; synchronize
			l_state.item.iter := (1).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_state.item.fnorm := a_f.item.enorm___1
			update_class_fields -- Current might have been changed; synchronize
			l_state.item.ncfail := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_state.item.ncsuc := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_state.item.nslow1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_state.item.nslow2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			a_dx.item.gsl_vector_set_all (0.0)
			update_class_fields -- Current might have been changed; synchronize
			if a_scale.to_boolean then
				compute_diag___0 (a_J, l_diag)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_diag.item.gsl_vector_set_all (1.0)
				update_class_fields -- Current might have been changed; synchronize
			end
			l_state.item.delta := compute_delta___0 (l_diag, a_x)
			update_class_fields -- Current might have been changed; synchronize
			a_J.item.gsl_linalg_QR_decomp (l_tau).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			a_J.item.gsl_linalg_QR_unpack (l_tau, l_q___3, l_r___3).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	hybridj_iterate_impl (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]; a_scale: INTEGER_32): INTEGER_32
		require
			not a_x.is_void
			not a_f.is_void
			not a_dx.is_void
		local
			l_state: CE_POINTER [PD_HYBRIDJ_STATE_T]
			l_fnorm: REAL_64
			l_q___3: CE_POINTER [PD_GSL_MATRIX]
			l_r___3: CE_POINTER [PD_GSL_MATRIX]
			l_tau: CE_POINTER [PD_GSL_VECTOR]
			l_diag: CE_POINTER [PD_GSL_VECTOR]
			l_qtf: CE_POINTER [PD_GSL_VECTOR]
			l_x_trial: CE_POINTER [PD_GSL_VECTOR]
			l_f_trial: CE_POINTER [PD_GSL_VECTOR]
			l_df: CE_POINTER [PD_GSL_VECTOR]
			l_qtdf: CE_POINTER [PD_GSL_VECTOR]
			l_rdx: CE_POINTER [PD_GSL_VECTOR]
			l_w: CE_POINTER [PD_GSL_VECTOR]
			l_v: CE_POINTER [PD_GSL_VECTOR]
			l_prered: REAL_64
			l_actred: REAL_64
			l_pnorm: REAL_64
			l_fnorm1: REAL_64
			l_fnorm1p: REAL_64
			l_ratio: REAL_64
			l_p1: REAL_64
			l_p5: REAL_64
			l_p001: REAL_64
			l_p0001: REAL_64
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_tmp___0: REAL_64
			l_status___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state := create {CE_CLASS_POINTER [PD_HYBRIDJ_STATE_T]}.make_cast (a_vstate)
					l_fnorm := l_state.item.fnorm
					l_q___3 := l_state.item.q
					l_r___3 := l_state.item.r
					l_tau := l_state.item.tau
					l_diag := l_state.item.diag
					l_qtf := l_state.item.qtf
					l_x_trial := l_state.item.x_trial
					l_f_trial := l_state.item.f_trial
					l_df := l_state.item.df
					l_qtdf := l_state.item.qtdf
					l_rdx := l_state.item.rdx
					l_w := l_state.item.w
					l_v := l_state.item.v
					l_p1 := 0.1
					l_p5 := 0.5
					l_p001 := 0.001
					l_p0001 := 0.0001
					compute_qtf (l_q___3, a_f, l_qtf)
					update_class_fields -- Current might have been changed; synchronize
					l_r___3.item.dogleg (l_qtf, l_diag, l_state.item.delta, l_state.item.newton, l_state.item.gradient, a_dx).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					compute_trial_step___0 (a_x, a_dx, l_state.item.x_trial)
					update_class_fields -- Current might have been changed; synchronize
					l_pnorm := (create {P_DOGLEG_STATIC}).scaled_enorm___0 (l_diag, a_dx)
					update_class_fields -- Current might have been changed; synchronize
					if l_state.item.iter = {NATURAL_32} 1 then
						if l_pnorm < l_state.item.delta then
							l_state.item.delta := l_pnorm
							update_class_fields -- Current might have been changed; synchronize
						end
					end
					l_tmp := f.integer_32_item ([l_x_trial, params, l_f_trial])
					update_class_fields -- Current might have been changed; synchronize
					l_status := l_tmp
					if l_status /= 0 then
						Result := 9
						l_return := True
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						compute_df (l_f_trial, a_f, l_df)
						update_class_fields -- Current might have been changed; synchronize
						l_fnorm1 := l_f_trial.item.enorm___1
						update_class_fields -- Current might have been changed; synchronize
						l_actred := compute_actual_reduction___0 (l_fnorm, l_fnorm1)
						update_class_fields -- Current might have been changed; synchronize
						compute_rdx (l_r___3, a_dx, l_rdx)
						update_class_fields -- Current might have been changed; synchronize
						l_fnorm1p := enorm_sum (l_qtf, l_rdx)
						update_class_fields -- Current might have been changed; synchronize
						l_prered := compute_predicted_reduction (l_fnorm, l_fnorm1p)
						update_class_fields -- Current might have been changed; synchronize
						if l_prered > (0).to_double then
							l_ratio := l_actred / l_prered
						else
							l_ratio := (0).to_double
						end
						if not (l_ratio < l_p1) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_state.item.ncsuc := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_state.item.ncfail := (l_state.item.ncfail) + 1
						update_class_fields -- Current might have been changed; synchronize
						l_state.item.delta := l_state.item.delta * l_p5
						update_class_fields -- Current might have been changed; synchronize
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 4 then -- else:
						l_state.item.ncfail := (0).to_natural_32
						update_class_fields -- Current might have been changed; synchronize
						l_state.item.ncsuc := (l_state.item.ncsuc) + 1
						update_class_fields -- Current might have been changed; synchronize
						if not (l_ratio >= l_p5) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 7 then -- else:
						if not (l_state.item.ncsuc > {NATURAL_32} 1) then
							l_pos := -1 + 9 -- skip if block
						end
				when 8 then -- if: | _L:
						if l_state.item.delta > l_pnorm / l_p5 then
							l_state.item.delta := l_state.item.delta
							update_class_fields -- Current might have been changed; synchronize
						else
							l_state.item.delta := l_pnorm / l_p5
							update_class_fields -- Current might have been changed; synchronize
						end
				when 9 then -- if done
						l_tmp___0 := fabs (l_ratio - (1).to_double)
						update_class_fields -- Current might have been changed; synchronize
						if l_tmp___0 <= l_p1 then
							l_state.item.delta := l_pnorm / l_p5
							update_class_fields -- Current might have been changed; synchronize
						end
				when 10 then -- if done
						if l_ratio >= l_p0001 then
							a_x.item.gsl_vector_memcpy (l_x_trial).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							a_f.item.gsl_vector_memcpy (l_f_trial).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_state.item.fnorm := l_fnorm1
							update_class_fields -- Current might have been changed; synchronize
							l_state.item.iter := (l_state.item.iter) + 1
							update_class_fields -- Current might have been changed; synchronize
						end
						l_state.item.nslow1 := (l_state.item.nslow1) + 1
						update_class_fields -- Current might have been changed; synchronize
						if l_actred >= l_p001 then
							l_state.item.nslow1 := (0).to_natural_32
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_actred >= l_p1 then
							l_state.item.nslow2 := (0).to_natural_32
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_state.item.ncfail = {NATURAL_32} 2 then
							l_tmp___1 := df.integer_32_item ([a_x, params, a_J])
							update_class_fields -- Current might have been changed; synchronize
							l_status___0 := l_tmp___1
							if l_status___0 /= 0 then
								Result := 9
								l_return := True
							end
							if not l_return then
								l_state.item.nslow2 := (l_state.item.nslow2) + 1
								update_class_fields -- Current might have been changed; synchronize
								if l_state.item.iter = {NATURAL_32} 1 then
									if a_scale.to_boolean then
										compute_diag___0 (a_J, l_diag)
										update_class_fields -- Current might have been changed; synchronize
									end
									l_state.item.delta := compute_delta___0 (l_diag, a_x)
									update_class_fields -- Current might have been changed; synchronize
								elseif a_scale.to_boolean then
									update_diag___0 (a_J, l_diag)
									update_class_fields -- Current might have been changed; synchronize
								end
								a_J.item.gsl_linalg_QR_decomp (l_tau).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								a_J.item.gsl_linalg_QR_unpack (l_tau, l_q___3, l_r___3).do_nothing
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							end
						end
				when 11 then -- if done
					if not l_return then
						compute_qtf (l_q___3, l_df, l_qtdf)
						update_class_fields -- Current might have been changed; synchronize
						compute_wv (l_qtdf, l_rdx, a_dx, l_diag, l_pnorm, l_w, l_v)
						update_class_fields -- Current might have been changed; synchronize
						l_q___3.item.gsl_linalg_QR_update (l_r___3, l_w, l_v).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						if l_state.item.nslow2 = {NATURAL_32} 5 then
							Result := 28
							l_return := True
						end
					end
					if not l_return and then (l_state.item.nslow1 = {NATURAL_32} 10) then
						Result := 27
						l_return := True
					end
					if not l_return then
						Result := 0
					end
				when 12 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	hybridj_set (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT]}.make_shared (c_pointer))).item.hybridj_set_impl (a_vstate, a_x, a_f, a_J, a_dx, 0)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	hybridsj_set (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT]}.make_shared (c_pointer))).item.hybridj_set_impl (a_vstate, a_x, a_f, a_J, a_dx, 1)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	hybridj_iterate (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT]}.make_shared (c_pointer))).item.hybridj_iterate_impl (a_vstate, a_x, a_f, a_J, a_dx, 0)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	hybridsj_iterate (a_vstate: CE_POINTER [ANY]; a_x, a_f: CE_POINTER [PD_GSL_VECTOR]; a_J: CE_POINTER [PD_GSL_MATRIX]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT]}.make_shared (c_pointer))).item.hybridj_iterate_impl (a_vstate, a_x, a_f, a_J, a_dx, 1)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

feature {PD_GSL_MULTIROOT_FUNCTION_FDF_STRUCT} -- private routines

	enorm___0 (a_f: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		require
			not a_f.is_void
		local
			l_e2: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_fi: REAL_64
		do
			l_e2 := (0).to_double
			l_n := a_f.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_fi := (a_f.item.gsl_vector_get (l_i))
				l_e2 := l_e2 + (l_fi * l_fi)
				l_i := l_i + 1
			end
			Result := (sqrt (l_e2))
		end

	compute_diag___0 (a_J: CE_POINTER [PD_GSL_MATRIX]; a_diag: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_diag.is_void
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_n: NATURAL_32
			l_sum: REAL_64
			l_Jij: REAL_64
			l_return: BOOLEAN
		do
			l_n := a_diag.item.size
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_Jij := (a_J.item.gsl_matrix_get (l_i, l_j))
					l_sum := l_sum + (l_Jij * l_Jij)
					l_i := l_i + 1
				end
				if l_sum = (0).to_double then
					l_sum := 1.0
				end
				a_diag.item.gsl_vector_set (l_j, (sqrt (l_sum)))
				l_j := l_j + 1
			end
		end

	compute_delta___0 (a_diag, a_x: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		require
			not a_x.is_void
		local
			l_Dx: REAL_64
			l_factor: REAL_64
			l_tmp___0: REAL_64
		do
			l_Dx := ((create {P_DOGLEG_STATIC}).scaled_enorm___0 (a_diag, a_x))
			l_factor := (100).to_double
			if l_Dx > (0).to_double then
				l_tmp___0 := l_factor * l_Dx
			else
				l_tmp___0 := l_factor
			end
			Result := l_tmp___0
		end

	enorm_sum (a_a___5, a_b___0: CE_POINTER [PD_GSL_VECTOR]): REAL_64
		require
			not a_a___5.is_void
		local
			l_e2: REAL_64
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_ai: REAL_64
			l_bi: REAL_64
			l_u: REAL_64
		do
			l_e2 := (0).to_double
			l_n := a_a___5.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_ai := (a_a___5.item.gsl_vector_get (l_i))
				l_bi := (a_b___0.item.gsl_vector_get (l_i))
				l_u := l_ai + l_bi
				l_e2 := l_e2 + (l_u * l_u)
				l_i := l_i + 1
			end
			Result := (sqrt (l_e2))
		end

	compute_wv (a_qtdf, a_rdx, a_dx, a_diag: CE_POINTER [PD_GSL_VECTOR]; a_pnorm: REAL_64; a_w, a_v: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_qtdf.is_void
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_qtdfi: REAL_64
			l_rdxi: REAL_64
			l_dxi: REAL_64
			l_diagi: REAL_64
		do
			l_n := a_qtdf.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_qtdfi := (a_qtdf.item.gsl_vector_get (l_i))
				l_rdxi := (a_rdx.item.gsl_vector_get (l_i))
				l_dxi := (a_dx.item.gsl_vector_get (l_i))
				l_diagi := (a_diag.item.gsl_vector_get (l_i))
				a_w.item.gsl_vector_set (l_i, (l_qtdfi - l_rdxi) / a_pnorm)
				a_v.item.gsl_vector_set (l_i, ((l_diagi * l_diagi) * l_dxi) / a_pnorm)
				l_i := l_i + 1
			end
		end

	compute_df (a_f_trial, a_f, a_df: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_f.is_void
		local
			l_i: NATURAL_32
			l_n: NATURAL_32
			l_dfi: REAL_64
			l_tmp: REAL_64
		do
			l_n := a_f.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_tmp := a_f_trial.item.gsl_vector_get (l_i)
				l_dfi := l_tmp - (a_f.item.gsl_vector_get (l_i))
				a_df.item.gsl_vector_set (l_i, l_dfi)
				l_i := l_i + 1
			end
		end

	update_diag___0 (a_J: CE_POINTER [PD_GSL_MATRIX]; a_diag: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_diag.is_void
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_n: NATURAL_32
			l_cnorm: REAL_64
			l_diagj: REAL_64
			l_sum: REAL_64
			l_Jij: REAL_64
			l_return: BOOLEAN
		do
			l_n := a_diag.item.size
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_n)
				loop
					l_Jij := (a_J.item.gsl_matrix_get (l_i, l_j))
					l_sum := l_sum + (l_Jij * l_Jij)
					l_i := l_i + 1
				end
				if l_sum = (0).to_double then
					l_sum := 1.0
				end
				l_cnorm := sqrt (l_sum)
				l_diagj := a_diag.item.gsl_vector_get (l_j)
				if l_cnorm > l_diagj then
					a_diag.item.gsl_vector_set (l_j, l_cnorm)
				end
				l_j := l_j + 1
			end
		end

	compute_actual_reduction___0 (a_fnorm, a_fnorm1: REAL_64): REAL_64
		local
			l_actred: REAL_64
			l_u: REAL_64
		do
			if a_fnorm1 < a_fnorm then
				l_u := a_fnorm1 / a_fnorm
				l_actred := (1).to_double - l_u * l_u
			else
				l_actred := (-1).to_double
			end
			Result := l_actred
		end

	compute_predicted_reduction (a_fnorm, a_fnorm1: REAL_64): REAL_64
		local
			l_prered: REAL_64
			l_u: REAL_64
		do
			if a_fnorm1 < a_fnorm then
				l_u := a_fnorm1 / a_fnorm
				l_prered := (1).to_double - l_u * l_u
			else
				l_prered := (0).to_double
			end
			Result := l_prered
		end

	compute_qtf (a_q___3: CE_POINTER [PD_GSL_MATRIX]; a_f, a_qtf: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_f.is_void
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_N: NATURAL_32
			l_sum: REAL_64
			l_tmp: REAL_64
			l_return: BOOLEAN
		do
			l_N := a_f.item.size
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_N))
			loop
				l_sum := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					l_tmp := a_q___3.item.gsl_matrix_get (l_i, l_j)
					l_sum := l_sum + (l_tmp * (a_f.item.gsl_vector_get (l_i)))
					l_i := l_i + 1
				end
				a_qtf.item.gsl_vector_set (l_j, l_sum)
				l_j := l_j + 1
			end
		end

	compute_rdx (a_r___3: CE_POINTER [PD_GSL_MATRIX]; a_dx, a_rdx: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_dx.is_void
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_N: NATURAL_32
			l_sum: REAL_64
			l_tmp: REAL_64
			l_return: BOOLEAN
		do
			l_N := a_dx.item.size
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_sum := (0).to_double
				l_j := l_i
				from
				until
					not (l_j < l_N)
				loop
					l_tmp := a_r___3.item.gsl_matrix_get (l_i, l_j)
					l_sum := l_sum + (l_tmp * (a_dx.item.gsl_vector_get (l_j)))
					l_j := l_j + 1
				end
				a_rdx.item.gsl_vector_set (l_i, l_sum)
				l_i := l_i + 1
			end
		end

	compute_trial_step___0 (a_x, a_dx, a_x_trial: CE_POINTER [PD_GSL_VECTOR])
		require
			not a_x.is_void
		local
			l_i: NATURAL_32
			l_N: NATURAL_32
			l_pi: REAL_64
			l_xi: REAL_64
		do
			l_N := a_x.item.size
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_pi := (a_dx.item.gsl_vector_get (l_i))
				l_xi := (a_x.item.gsl_vector_get (l_i))
				a_x_trial.item.gsl_vector_set (l_i, l_xi + l_pi)
				l_i := l_i + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			f := a_other.f
			df := a_other.df
			fdf := a_other.fdf
			n := a_other.n
			params := a_other.params
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				f.is_equal (a_other.f) and
				df.is_equal (a_other.df) and
				fdf.is_equal (a_other.fdf) and
				n = a_other.n and
				params.is_equal (a_other.params)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("f"),
					create {CE_STRUCTURE_FIELD}.make ("df"),
					create {CE_STRUCTURE_FIELD}.make ("fdf"),
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("params")
				>>)
			init_structure_info
		end

end
