class
	PD_GSL_ODEIV_EVOLVE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} y0.make
			create {CE_REAL_64_POINTER} yerr.make
			create {CE_REAL_64_POINTER} dydt_in.make
			create {CE_REAL_64_POINTER} dydt_out.make
			Precursor
		end

feature {ANY} -- fields

	dimension: NATURAL_32 assign set_dimension

	y0: CE_POINTER [REAL_64] assign set_y0

	yerr: CE_POINTER [REAL_64] assign set_yerr

	dydt_in: CE_POINTER [REAL_64] assign set_dydt_in

	dydt_out: CE_POINTER [REAL_64] assign set_dydt_out

	last_step: REAL_64 assign set_last_step

	count: NATURAL_32 assign set_count

	failed_steps: NATURAL_32 assign set_failed_steps

feature {ANY} -- field setters

	set_dimension (a_dimension: NATURAL_32)
		do
			dimension := a_dimension
			update_memory_field ("dimension")
		end

	set_y0 (a_y0: CE_POINTER [REAL_64])
		do
			y0 := a_y0
			update_memory_field ("y0")
		end

	set_yerr (a_yerr: CE_POINTER [REAL_64])
		do
			yerr := a_yerr
			update_memory_field ("yerr")
		end

	set_dydt_in (a_dydt_in: CE_POINTER [REAL_64])
		do
			dydt_in := a_dydt_in
			update_memory_field ("dydt_in")
		end

	set_dydt_out (a_dydt_out: CE_POINTER [REAL_64])
		do
			dydt_out := a_dydt_out
			update_memory_field ("dydt_out")
		end

	set_last_step (a_last_step: REAL_64)
		do
			last_step := a_last_step
			update_memory_field ("last_step")
		end

	set_count (a_count: NATURAL_32)
		do
			count := a_count
			update_memory_field ("count")
		end

	set_failed_steps (a_failed_steps: NATURAL_32)
		do
			failed_steps := a_failed_steps
			update_memory_field ("failed_steps")
		end

feature {ANY} -- public routines

	gsl_odeiv_evolve_reset: INTEGER_32
		do
			Current.count := {NATURAL_32} 0
			update_class_fields -- Current might have been changed; synchronize
			Current.failed_steps := {NATURAL_32} 0
			update_class_fields -- Current might have been changed; synchronize
			Current.last_step := 0.0
			update_class_fields -- Current might have been changed; synchronize
			Result := 0
		end

	gsl_odeiv_evolve_apply (a_con: CE_POINTER [PD_GSL_ODEIV_CONTROL]; a_step: CE_POINTER [PD_GSL_ODEIV_STEP]; a_dydt: CE_POINTER [PD_GSL_ODEIV_SYSTEM]; a_t: CE_POINTER [REAL_64]; a_t1: REAL_64; a_h, a_y: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_step.is_void
			not a_t.is_void
			not a_h.is_void
		local
			l_t0: REAL_64
			l_h0: CE_ARRAY [REAL_64]
			l_step_status: INTEGER_32
			l_final_step: INTEGER_32
			l_dt: REAL_64
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_h_old: REAL_64
			l_hadjust_status: INTEGER_32
			l_tmp___0: INTEGER_32
			l_t_curr: REAL_64
			l_tmp___1: REAL_64
			l_t_next: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_REAL_64_ARRAY} l_h0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_t0 := a_t.item
					l_h0.item := a_h.item
					update_class_fields -- Current might have been changed; synchronize
					l_final_step := 0
					l_dt := a_t1 - l_t0
					if dimension /= a_step.item.dimension then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("step dimension must match evolution size"), ce_string ("evolve.c"), 128, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 7 -- skip if block
					end
				when 1 then -- if:
						if not (l_dt < 0.0) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_h0.item > 0.0 then
							l_pos := -1 + 6 -- goto _L
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 4 then -- else: | _L___0:
						if not (l_dt > 0.0) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if not (l_h0.item < 0.0) then
							l_pos := -1 + 7 -- skip if block
						end
				when 6 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("step direction must match interval direction"), ce_string ("evolve.c"), 133, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 7 then -- if done | if done | if done
					if not l_return then
						if not a_con.is_void then
							memcpy (create {CE_VOID_POINTER}.make_cast (y0), create {CE_VOID_POINTER}.make_cast (a_y), dimension * {PLATFORM}.Real_64_bytes.to_natural_32).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						if ((a_step.item.type).item.can_use_dydt_in).to_boolean then
							l_tmp := (a_dydt.item.function).integer_32_item ([l_t0, a_y, dydt_in, a_dydt.item.params])
							update_class_fields -- Current might have been changed; synchronize
							l_status := l_tmp
							if l_status.to_boolean then
								Result := l_status
								l_return := True
							end
						end
					end
					if l_return then
						l_pos := -1 + 13 -- skip if block
					end
				when 8 then -- if: | try_step:
						if not (l_dt >= 0.0) then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						if l_h0.item > l_dt then
							l_h0.item := l_dt
							update_class_fields -- Current might have been changed; synchronize
							l_final_step := 1
						else
							l_pos := -1 + 11 -- goto _L___1
						end
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else: | _L___1:
						if l_dt < 0.0 then
							if l_h0.item < l_dt then
								l_h0.item := l_dt
								update_class_fields -- Current might have been changed; synchronize
								l_final_step := 1
							else
								l_final_step := 0
							end
						else
							l_final_step := 0
						end
				when 12 then
						if ((a_step.item.type).item.can_use_dydt_in).to_boolean then
							l_step_status := a_step.item.gsl_odeiv_step_apply (l_t0, l_h0.item, a_y, yerr, dydt_in, dydt_out, a_dydt)
							update_class_fields -- Current might have been changed; synchronize
						else
							l_step_status := a_step.item.gsl_odeiv_step_apply (l_t0, l_h0.item, a_y, yerr, create {CE_REAL_64_POINTER}.make_shared (default_pointer), dydt_out, a_dydt)
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_step_status /= 0 then
							a_h.item := l_h0.item
							update_class_fields -- Current might have been changed; synchronize
							a_t.item := l_t0
							update_class_fields -- Current might have been changed; synchronize
							Result := l_step_status
							l_return := True
						end
				when 13 then -- if done
					if l_return then
						l_pos := -1 + 16 -- skip if block
					end
				when 14 then -- if:
						Current.count := count + 1
						update_class_fields -- Current might have been changed; synchronize
						Current.last_step := l_h0.item
						update_class_fields -- Current might have been changed; synchronize
						if l_final_step.to_boolean then
							a_t.item := a_t1
							update_class_fields -- Current might have been changed; synchronize
						else
							a_t.item := l_t0 + l_h0.item
							update_class_fields -- Current might have been changed; synchronize
						end
						if not a_con.is_void then
							l_h_old := l_h0.item
							l_tmp___0 := gsl_odeiv_control_hadjust (a_con, a_step, a_y, yerr, dydt_out, l_h0)
							update_class_fields -- Current might have been changed; synchronize
							l_hadjust_status := l_tmp___0
							if l_hadjust_status = -1 then
								l_tmp___1 := (create {P_COERCE_STATIC}).gsl_coerce_double (a_t.item)
								update_class_fields -- Current might have been changed; synchronize
								l_t_curr := l_tmp___1
								l_tmp___2 := (create {P_COERCE_STATIC}).gsl_coerce_double (a_t.item + l_h0.item)
								update_class_fields -- Current might have been changed; synchronize
								l_t_next := l_tmp___2
								l_tmp___3 := fabs (l_h0.item)
								update_class_fields -- Current might have been changed; synchronize
								l_tmp___4 := fabs (l_h_old)
								update_class_fields -- Current might have been changed; synchronize
								if l_tmp___3 < l_tmp___4 then
									if l_t_next /= l_t_curr then
										memcpy (create {CE_VOID_POINTER}.make_cast (a_y), create {CE_VOID_POINTER}.make_cast (y0), (a_dydt.item.dimension * {PLATFORM}.Real_64_bytes.to_natural_32)).do_nothing
										update_class_fields -- Current might have been changed; synchronize
										Current.failed_steps := failed_steps + 1
										update_class_fields -- Current might have been changed; synchronize
										l_pos := -1 + 8 -- goto try_step
									else
										l_h0.item := l_h_old
										update_class_fields -- Current might have been changed; synchronize
									end
								else
									l_h0.item := l_h_old
									update_class_fields -- Current might have been changed; synchronize
								end
							end
						end
				when 15 then
						a_h.item := l_h0.item
						update_class_fields -- Current might have been changed; synchronize
						Result := l_step_status
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {PD_GSL_ODEIV_EVOLVE} -- private routines

	gsl_odeiv_control_hadjust (a_c___1: CE_POINTER [PD_GSL_ODEIV_CONTROL]; a_s: CE_POINTER [PD_GSL_ODEIV_STEP]; a_y, a_yerr, a_dydt, a_h: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_c___1.is_void
			not a_s.is_void
		do
			Result := (((a_c___1.item.type).item.hadjust).integer_32_item ([a_c___1.item.state, a_s.item.dimension, (((a_s.item.type).item.order).natural_32_item ([a_s.item.state])), a_y, a_yerr, a_dydt, a_h]))
		end

feature {ANY}

	copy (a_other: like Current)
		do
			dimension := a_other.dimension
			y0 := a_other.y0
			yerr := a_other.yerr
			dydt_in := a_other.dydt_in
			dydt_out := a_other.dydt_out
			last_step := a_other.last_step
			count := a_other.count
			failed_steps := a_other.failed_steps
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				dimension = a_other.dimension and
				y0.is_equal (a_other.y0) and
				yerr.is_equal (a_other.yerr) and
				dydt_in.is_equal (a_other.dydt_in) and
				dydt_out.is_equal (a_other.dydt_out) and
				last_step = a_other.last_step and
				count = a_other.count and
				failed_steps = a_other.failed_steps
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("dimension"),
					create {CE_STRUCTURE_FIELD}.make ("y0"),
					create {CE_STRUCTURE_FIELD}.make ("yerr"),
					create {CE_STRUCTURE_FIELD}.make ("dydt_in"),
					create {CE_STRUCTURE_FIELD}.make ("dydt_out"),
					create {CE_STRUCTURE_FIELD}.make ("last_step"),
					create {CE_STRUCTURE_FIELD}.make ("count"),
					create {CE_STRUCTURE_FIELD}.make ("failed_steps")
				>>)
			init_structure_info
		end

end
