class
	PD_GSL_STACK_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_POINTER} v.make
			Precursor
		end

feature {ANY} -- fields

	N: NATURAL_32 assign set_N

	v: CE_POINTER [NATURAL_32] assign set_v

	i: NATURAL_32 assign set_i

feature {ANY} -- field setters

	set_N (a_N: NATURAL_32)
		do
			N := a_N
			update_memory_field ("N")
		end

	set_v (a_v: CE_POINTER [NATURAL_32])
		do
			v := a_v
			update_memory_field ("v")
		end

	set_i (a_i: NATURAL_32)
		do
			i := a_i
			update_memory_field ("i")
		end

feature {ANY} -- public routines

	push_stack (a_v: NATURAL_32): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if i >= N then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				v[i.to_integer_32] := a_v
				update_class_fields -- Current might have been changed; synchronize
				Current.i := i + 1
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	pop_stack: NATURAL_32
		local
			l_return, l_break: BOOLEAN
		do
			if i = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("internal error - stack exhausted"), ce_string ("discrete.c"), 191, 7)
					update_class_fields -- Current might have been changed; synchronize
					Result := (7).to_natural_32
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.i := i - 1
				update_class_fields -- Current might have been changed; synchronize
				Result := (v[i.to_integer_32])
			end
		end

	size_stack: NATURAL_32
		do
			Result := i
		end

	free_stack
		do
			free (create {CE_VOID_POINTER}.make_cast (v))
			update_class_fields -- Current might have been changed; synchronize
			free (create {CE_VOID_POINTER}.make_shared (c_pointer))
			update_class_fields -- Current might have been changed; synchronize
		end

feature {ANY}

	copy (a_other: like Current)
		do
			N := a_other.N
			v := a_other.v
			i := a_other.i
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				N = a_other.N and
				v.is_equal (a_other.v) and
				i = a_other.i
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("N"),
					create {CE_STRUCTURE_FIELD}.make ("v"),
					create {CE_STRUCTURE_FIELD}.make ("i")
				>>)
			init_structure_info
		end

end
