class
	PD_NMSIMPLEX_STATE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} x1.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} y1.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} ws1.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} ws2.make
		end

feature {ANY} -- fields

	x1: CE_POINTER [PD_GSL_MATRIX] assign set_x1

	y1: CE_POINTER [PD_GSL_VECTOR] assign set_y1

	ws1: CE_POINTER [PD_GSL_VECTOR] assign set_ws1

	ws2: CE_POINTER [PD_GSL_VECTOR] assign set_ws2

feature {ANY} -- field setters

	set_x1 (a_x1: CE_POINTER [PD_GSL_MATRIX])
		do
			x1 := a_x1
			update_memory_field ("x1")
		end

	set_y1 (a_y1: CE_POINTER [PD_GSL_VECTOR])
		do
			y1 := a_y1
			update_memory_field ("y1")
		end

	set_ws1 (a_ws1: CE_POINTER [PD_GSL_VECTOR])
		do
			ws1 := a_ws1
			update_memory_field ("ws1")
		end

	set_ws2 (a_ws2: CE_POINTER [PD_GSL_VECTOR])
		do
			ws2 := a_ws2
			update_memory_field ("ws2")
		end

feature {ANY} -- public routines

	nmsimplex_move_corner (a_coeff: REAL_64; a_corner: NATURAL_32; a_xc: CE_POINTER [PD_GSL_VECTOR]; a_f: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_STRUCT]): REAL_64
		require
			not a_f.is_void
		local
			l_x1___0: CE_POINTER [PD_GSL_MATRIX]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_newval: REAL_64
			l_mp: REAL_64
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_return: BOOLEAN
		do
			l_x1___0 := x1
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_x1___0.item.size2))
			loop
				l_mp := 0.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_x1___0.item.size1)
				loop
					if l_i /= a_corner then
						l_tmp := l_x1___0.item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_mp := l_mp + l_tmp
					end
					l_i := l_i + 1
				end
				l_mp := l_mp / (l_x1___0.item.size1 - {NATURAL_32} 1).to_real_64
				l_tmp___0 := l_x1___0.item.gsl_matrix_get (a_corner, l_j)
				update_class_fields -- Current might have been changed; synchronize
				l_newval := l_mp - a_coeff * (l_mp - l_tmp___0)
				a_xc.item.gsl_vector_set (l_j, l_newval)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
			l_newval := (a_f.item.f).real_64_item ([a_xc, a_f.item.params])
			update_class_fields -- Current might have been changed; synchronize
			Result := l_newval
		end

	nmsimplex_contract_by_best (a_best: NATURAL_32; a_xc: CE_POINTER [PD_GSL_VECTOR]; a_f: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_STRUCT]): INTEGER_32
		local
			l_x1___0: CE_POINTER [PD_GSL_MATRIX]
			l_y1___0: CE_POINTER [PD_GSL_VECTOR]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_newval: REAL_64
			l_status: INTEGER_32
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: INTEGER_32
			l_return: BOOLEAN
		do
			l_x1___0 := x1
			l_y1___0 := y1
			l_status := 0
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_x1___0.item.size1))
			loop
				if l_i /= a_best then
					l_j := (0).to_natural_32
					from
					until
						not (l_j < l_x1___0.item.size2)
					loop
						l_tmp := l_x1___0.item.gsl_matrix_get (l_i, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___0 := l_x1___0.item.gsl_matrix_get (a_best, l_j)
						update_class_fields -- Current might have been changed; synchronize
						l_newval := 0.5 * (l_tmp + l_tmp___0)
						l_x1___0.item.gsl_matrix_set (l_i, l_j, l_newval)
						update_class_fields -- Current might have been changed; synchronize
						l_j := l_j + 1
					end
					(create {P_GETSET_SOURCE_STATIC}).gsl_matrix_get_row (a_xc, l_x1___0, l_i).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_newval := (a_f.item.f).real_64_item ([a_xc, a_f.item.params])
					update_class_fields -- Current might have been changed; synchronize
					l_y1___0.item.gsl_vector_set (l_i, l_newval)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___1 := (create {P_INFNAN_STATIC}).gsl_finite (l_newval)
					update_class_fields -- Current might have been changed; synchronize
					if not l_tmp___1.to_boolean then
						l_status := 9
					end
				end
				l_i := l_i + 1
			end
			Result := l_status
		end

	nmsimplex_calc_center (a_mp: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_x1___0: CE_POINTER [PD_GSL_MATRIX]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_val: REAL_64
			l_tmp: REAL_64
			l_return: BOOLEAN
		do
			l_x1___0 := x1
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_x1___0.item.size2))
			loop
				l_val := 0.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_x1___0.item.size1)
				loop
					l_tmp := l_x1___0.item.gsl_matrix_get (l_i, l_j)
					update_class_fields -- Current might have been changed; synchronize
					l_val := l_val + l_tmp
					l_i := l_i + 1
				end
				l_val := l_val / (l_x1___0.item.size1).to_real_64
				a_mp.item.gsl_vector_set (l_j, l_val)
				update_class_fields -- Current might have been changed; synchronize
				l_j := l_j + 1
			end
			Result := 0
		end

	nmsimplex_size: REAL_64
		local
			l_s: CE_POINTER [PD_GSL_VECTOR]
			l_mp: CE_POINTER [PD_GSL_VECTOR]
			l_x1___0: CE_POINTER [PD_GSL_MATRIX]
			l_i: NATURAL_32
			l_ss: REAL_64
			l_tmp: REAL_64
		do
			l_s := ws1
			l_mp := ws2
			l_x1___0 := x1
			l_ss := 0.0;
			((create {CE_CLASS_POINTER [PD_NMSIMPLEX_STATE_T]}.make_shared (c_pointer))).item.nmsimplex_calc_center (l_mp).do_nothing
			update_class_fields -- Current might have been changed; synchronize
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_x1___0.item.size1)
			loop
				(create {P_GETSET_SOURCE_STATIC}).gsl_matrix_get_row (l_s, l_x1___0, l_i).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_mp.item.gsl_blas_daxpy (-1.0, l_s).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_tmp := l_s.item.gsl_blas_dnrm2
				update_class_fields -- Current might have been changed; synchronize
				l_ss := l_ss + l_tmp
				l_i := l_i + 1
			end
			Result := (l_ss / (l_x1___0.item.size1).to_real_64)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			x1 := a_other.x1
			y1 := a_other.y1
			ws1 := a_other.ws1
			ws2 := a_other.ws2
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				x1.is_equal (a_other.x1) and
				y1.is_equal (a_other.y1) and
				ws1.is_equal (a_other.ws1) and
				ws2.is_equal (a_other.ws2)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("x1"),
					create {CE_STRUCTURE_FIELD}.make ("y1"),
					create {CE_STRUCTURE_FIELD}.make ("ws1"),
					create {CE_STRUCTURE_FIELD}.make ("ws2")
				>>)
			init_structure_info
		end

end
