class
	PD_RK2IMP_STATE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} Y1.make
			create {CE_REAL_64_POINTER} y0.make
			create {CE_REAL_64_POINTER} ytmp.make
			create {CE_REAL_64_POINTER} y_onestep.make
			create {CE_REAL_64_POINTER} y0_orig.make
		end

feature {ANY} -- fields

	Y1: CE_POINTER [REAL_64] assign set_Y1

	y0: CE_POINTER [REAL_64] assign set_y0

	ytmp: CE_POINTER [REAL_64] assign set_ytmp

	y_onestep: CE_POINTER [REAL_64] assign set_y_onestep

	y0_orig: CE_POINTER [REAL_64] assign set_y0_orig

feature {ANY} -- field setters

	set_Y1 (a_Y1: CE_POINTER [REAL_64])
		do
			Y1 := a_Y1
			update_memory_field ("Y1")
		end

	set_y0 (a_y0: CE_POINTER [REAL_64])
		do
			y0 := a_y0
			update_memory_field ("y0")
		end

	set_ytmp (a_ytmp: CE_POINTER [REAL_64])
		do
			ytmp := a_ytmp
			update_memory_field ("ytmp")
		end

	set_y_onestep (a_y_onestep: CE_POINTER [REAL_64])
		do
			y_onestep := a_y_onestep
			update_memory_field ("y_onestep")
		end

	set_y0_orig (a_y0_orig: CE_POINTER [REAL_64])
		do
			y0_orig := a_y0_orig
			update_memory_field ("y0_orig")
		end

feature {ANY} -- public routines

	rk2imp_step (a_y: CE_POINTER [REAL_64]; a_h, a_t: REAL_64; a_dim: NATURAL_32; a_sys: CE_POINTER [PD_GSL_ODEIV_SYSTEM]): INTEGER_32
		local
			l_y0___0: CE_POINTER [REAL_64]
			l_Y1: CE_POINTER [REAL_64]
			l_ytmp: CE_POINTER [REAL_64]
			l_max_iter: INTEGER_32
			l_nu: INTEGER_32
			l_i: NATURAL_32
			l_s: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
		do
			l_y0___0 := y0
			l_Y1 := Y1
			l_ytmp := ytmp
			l_max_iter := 3
			l_nu := 0
			from
			until
				not (not (l_return) and then (l_nu < l_max_iter))
			loop
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					l_ytmp[l_i.to_integer_32] := (l_y0___0[l_i.to_integer_32] + ((0.5 * a_h) * l_Y1[l_i.to_integer_32]))
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_tmp := (a_sys.item.function).integer_32_item ([a_t + 0.5 * a_h, l_ytmp, l_Y1, a_sys.item.params])
				update_class_fields -- Current might have been changed; synchronize
				l_s := l_tmp
				if l_s /= 0 then
					Result := l_s
					l_return := True
				end
				if not l_return then
					l_nu := l_nu + 1
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					a_y[l_i.to_integer_32] := (l_y0___0[l_i.to_integer_32] + (a_h * l_Y1[l_i.to_integer_32]))
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			Y1 := a_other.Y1
			y0 := a_other.y0
			ytmp := a_other.ytmp
			y_onestep := a_other.y_onestep
			y0_orig := a_other.y0_orig
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				Y1.is_equal (a_other.Y1) and
				y0.is_equal (a_other.y0) and
				ytmp.is_equal (a_other.ytmp) and
				y_onestep.is_equal (a_other.y_onestep) and
				y0_orig.is_equal (a_other.y0_orig)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("Y1"),
					create {CE_STRUCTURE_FIELD}.make ("y0"),
					create {CE_STRUCTURE_FIELD}.make ("ytmp"),
					create {CE_STRUCTURE_FIELD}.make ("y_onestep"),
					create {CE_STRUCTURE_FIELD}.make ("y0_orig")
				>>)
			init_structure_info
		end

end
