class
	PD_RK4_STATE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} k.make
			create {CE_REAL_64_POINTER} k1.make
			create {CE_REAL_64_POINTER} y0.make
			create {CE_REAL_64_POINTER} ytmp.make
			create {CE_REAL_64_POINTER} y_onestep.make
		end

feature {ANY} -- fields

	k: CE_POINTER [REAL_64] assign set_k

	k1: CE_POINTER [REAL_64] assign set_k1

	y0: CE_POINTER [REAL_64] assign set_y0

	ytmp: CE_POINTER [REAL_64] assign set_ytmp

	y_onestep: CE_POINTER [REAL_64] assign set_y_onestep

feature {ANY} -- field setters

	set_k (a_k: CE_POINTER [REAL_64])
		do
			k := a_k
			update_memory_field ("k")
		end

	set_k1 (a_k1: CE_POINTER [REAL_64])
		do
			k1 := a_k1
			update_memory_field ("k1")
		end

	set_y0 (a_y0: CE_POINTER [REAL_64])
		do
			y0 := a_y0
			update_memory_field ("y0")
		end

	set_ytmp (a_ytmp: CE_POINTER [REAL_64])
		do
			ytmp := a_ytmp
			update_memory_field ("ytmp")
		end

	set_y_onestep (a_y_onestep: CE_POINTER [REAL_64])
		do
			y_onestep := a_y_onestep
			update_memory_field ("y_onestep")
		end

feature {ANY} -- public routines

	rk4_step (a_y: CE_POINTER [REAL_64]; a_h, a_t: REAL_64; a_dim: NATURAL_32; a_sys: CE_POINTER [PD_GSL_ODEIV_SYSTEM]): INTEGER_32
		require
			not a_sys.is_void
		local
			l_y0: CE_POINTER [REAL_64]
			l_ytmp: CE_POINTER [REAL_64]
			l_k: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_s: INTEGER_32
			l_tmp: INTEGER_32
			l_s___0: INTEGER_32
			l_tmp___0: INTEGER_32
			l_s___1: INTEGER_32
			l_tmp___1: INTEGER_32
			l_return: BOOLEAN
		do
			l_y0 := y0
			l_ytmp := ytmp
			l_k := k
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_dim)
			loop
				a_y[l_i.to_integer_32] := a_y[l_i.to_integer_32] + ((a_h / 6.0) * l_k[l_i.to_integer_32])
				update_class_fields -- Current might have been changed; synchronize
				l_ytmp[l_i.to_integer_32] := (l_y0[l_i.to_integer_32] + ((0.5 * a_h) * l_k[l_i.to_integer_32]))
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_tmp := (a_sys.item.function).integer_32_item ([a_t + 0.5 * a_h, l_ytmp, l_k, a_sys.item.params])
			update_class_fields -- Current might have been changed; synchronize
			l_s := l_tmp
			if l_s /= 0 then
				Result := l_s
				l_return := True
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					a_y[l_i.to_integer_32] := a_y[l_i.to_integer_32] + ((a_h / 3.0) * l_k[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_ytmp[l_i.to_integer_32] := (l_y0[l_i.to_integer_32] + ((0.5 * a_h) * l_k[l_i.to_integer_32]))
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_tmp___0 := (a_sys.item.function).integer_32_item ([a_t + 0.5 * a_h, l_ytmp, l_k, a_sys.item.params])
				update_class_fields -- Current might have been changed; synchronize
				l_s___0 := l_tmp___0
				if l_s___0 /= 0 then
					Result := l_s___0
					l_return := True
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					a_y[l_i.to_integer_32] := a_y[l_i.to_integer_32] + ((a_h / 3.0) * l_k[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_ytmp[l_i.to_integer_32] := (l_y0[l_i.to_integer_32] + (a_h * l_k[l_i.to_integer_32]))
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_tmp___1 := (a_sys.item.function).integer_32_item ([a_t + a_h, l_ytmp, l_k, a_sys.item.params])
				update_class_fields -- Current might have been changed; synchronize
				l_s___1 := l_tmp___1
				if l_s___1 /= 0 then
					Result := l_s___1
					l_return := True
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					a_y[l_i.to_integer_32] := a_y[l_i.to_integer_32] + ((a_h / 6.0) * l_k[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			k := a_other.k
			k1 := a_other.k1
			y0 := a_other.y0
			ytmp := a_other.ytmp
			y_onestep := a_other.y_onestep
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				k.is_equal (a_other.k) and
				k1.is_equal (a_other.k1) and
				y0.is_equal (a_other.y0) and
				ytmp.is_equal (a_other.ytmp) and
				y_onestep.is_equal (a_other.y_onestep)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("k"),
					create {CE_STRUCTURE_FIELD}.make ("k1"),
					create {CE_STRUCTURE_FIELD}.make ("y0"),
					create {CE_STRUCTURE_FIELD}.make ("ytmp"),
					create {CE_STRUCTURE_FIELD}.make ("y_onestep")
				>>)
			init_structure_info
		end

end
