class
	PD_RK4IMP_STATE_T

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} k1nu.make
			create {CE_REAL_64_POINTER} k2nu.make
			create {CE_REAL_64_POINTER} ytmp1.make
			create {CE_REAL_64_POINTER} ytmp2.make
			create {CE_REAL_64_POINTER} y0.make
			create {CE_REAL_64_POINTER} y0_orig.make
			create {CE_REAL_64_POINTER} y_onestep.make
		end

feature {ANY} -- fields

	k1nu: CE_POINTER [REAL_64] assign set_k1nu

	k2nu: CE_POINTER [REAL_64] assign set_k2nu

	ytmp1: CE_POINTER [REAL_64] assign set_ytmp1

	ytmp2: CE_POINTER [REAL_64] assign set_ytmp2

	y0: CE_POINTER [REAL_64] assign set_y0

	y0_orig: CE_POINTER [REAL_64] assign set_y0_orig

	y_onestep: CE_POINTER [REAL_64] assign set_y_onestep

feature {ANY} -- field setters

	set_k1nu (a_k1nu: CE_POINTER [REAL_64])
		do
			k1nu := a_k1nu
			update_memory_field ("k1nu")
		end

	set_k2nu (a_k2nu: CE_POINTER [REAL_64])
		do
			k2nu := a_k2nu
			update_memory_field ("k2nu")
		end

	set_ytmp1 (a_ytmp1: CE_POINTER [REAL_64])
		do
			ytmp1 := a_ytmp1
			update_memory_field ("ytmp1")
		end

	set_ytmp2 (a_ytmp2: CE_POINTER [REAL_64])
		do
			ytmp2 := a_ytmp2
			update_memory_field ("ytmp2")
		end

	set_y0 (a_y0: CE_POINTER [REAL_64])
		do
			y0 := a_y0
			update_memory_field ("y0")
		end

	set_y0_orig (a_y0_orig: CE_POINTER [REAL_64])
		do
			y0_orig := a_y0_orig
			update_memory_field ("y0_orig")
		end

	set_y_onestep (a_y_onestep: CE_POINTER [REAL_64])
		do
			y_onestep := a_y_onestep
			update_memory_field ("y_onestep")
		end

feature {ANY} -- public routines

	rk4imp_step (a_y: CE_POINTER [REAL_64]; a_h, a_t: REAL_64; a_dim: NATURAL_32; a_sys: CE_POINTER [PD_GSL_ODEIV_SYSTEM]): INTEGER_32
		local
			l_ir3: REAL_64
			l_iter_steps: INTEGER_32
			l_nu: INTEGER_32
			l_i: NATURAL_32
			l_k1nu: CE_POINTER [REAL_64]
			l_k2nu: CE_POINTER [REAL_64]
			l_ytmp1: CE_POINTER [REAL_64]
			l_ytmp2: CE_POINTER [REAL_64]
			l_s: INTEGER_32
			l_tmp: INTEGER_32
			l_s___0: INTEGER_32
			l_tmp___0: INTEGER_32
			l_d_i: REAL_64
			l_return: BOOLEAN
		do
			l_ir3 := 1.0 / 1.73205080756887729352744634151
			l_iter_steps := 3
			l_k1nu := k1nu
			l_k2nu := k2nu
			l_ytmp1 := ytmp1
			l_ytmp2 := ytmp2
			l_nu := 0
			from
			until
				not (not (l_return) and then (l_nu < l_iter_steps))
			loop
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					l_ytmp1[l_i.to_integer_32] := a_y[l_i.to_integer_32] + a_h * (0.25 * l_k1nu[l_i.to_integer_32] + (0.5 * (0.5 - l_ir3)) * l_k2nu[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_ytmp2[l_i.to_integer_32] := a_y[l_i.to_integer_32] + a_h * (0.25 * l_k2nu[l_i.to_integer_32] + (0.5 * (0.5 + l_ir3)) * l_k1nu[l_i.to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				l_tmp := (a_sys.item.function).integer_32_item ([a_t + (0.5 * a_h) * (1.0 - l_ir3), l_ytmp1, l_k1nu, a_sys.item.params])
				update_class_fields -- Current might have been changed; synchronize
				l_s := l_tmp
				if l_s /= 0 then
					Result := l_s
					l_return := True
				end
				if not l_return then
					l_tmp___0 := (a_sys.item.function).integer_32_item ([a_t + (0.5 * a_h) * (1.0 + l_ir3), l_ytmp2, l_k2nu, a_sys.item.params])
					update_class_fields -- Current might have been changed; synchronize
					l_s___0 := l_tmp___0
					if l_s___0 /= 0 then
						Result := l_s___0
						l_return := True
					end
				end
				if not l_return then
					l_nu := l_nu + 1
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_dim)
				loop
					l_d_i := 0.5 * (l_k1nu[l_i.to_integer_32] + l_k2nu[l_i.to_integer_32])
					a_y[l_i.to_integer_32] := a_y[l_i.to_integer_32] + (a_h * l_d_i)
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			k1nu := a_other.k1nu
			k2nu := a_other.k2nu
			ytmp1 := a_other.ytmp1
			ytmp2 := a_other.ytmp2
			y0 := a_other.y0
			y0_orig := a_other.y0_orig
			y_onestep := a_other.y_onestep
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				k1nu.is_equal (a_other.k1nu) and
				k2nu.is_equal (a_other.k2nu) and
				ytmp1.is_equal (a_other.ytmp1) and
				ytmp2.is_equal (a_other.ytmp2) and
				y0.is_equal (a_other.y0) and
				y0_orig.is_equal (a_other.y0_orig) and
				y_onestep.is_equal (a_other.y_onestep)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("k1nu"),
					create {CE_STRUCTURE_FIELD}.make ("k2nu"),
					create {CE_STRUCTURE_FIELD}.make ("ytmp1"),
					create {CE_STRUCTURE_FIELD}.make ("ytmp2"),
					create {CE_STRUCTURE_FIELD}.make ("y0"),
					create {CE_STRUCTURE_FIELD}.make ("y0_orig"),
					create {CE_STRUCTURE_FIELD}.make ("y_onestep")
				>>)
			init_structure_info
		end

end
